import 'package:firebase_analytics/firebase_analytics.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/screens/splash.dart';
import 'package:foapp/util/const.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:intl/intl.dart';
import 'firebase_options.dart';




Future<void> main() async{
  WidgetsFlutterBinding.ensureInitialized();
  await Firebase.initializeApp(
    options: DefaultFirebaseOptions.currentPlatform,
  );
  Intl.defaultLocale = 'ru';
  SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp, DeviceOrientation.portraitDown]);
  SystemChrome.setSystemUIOverlayStyle(
      SystemUiOverlayStyle(statusBarBrightness: Brightness.light) // Or Brightness.dark
  );

  runApp(
    MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => AppProvider()),
        ChangeNotifierProvider(create: (_)=>DataProvider()),
        ChangeNotifierProvider(create: (BuildContext context)=>UserProvider(context: context)),
      ],
      child: MyApp(),
    ),
  );
}

class MyApp extends StatelessWidget {

  @override
  Widget build(BuildContext context) {
    //FbAnalitics.analytics=FirebaseAnalytics();
    return Consumer3<AppProvider, DataProvider, UserProvider>(
      builder: (BuildContext context, AppProvider appProvider,
          DataProvider menuProvider, UserProvider userProvider, Widget? child) {
        return MaterialApp(
          key: appProvider.key,
          debugShowCheckedModeBanner: false,
          navigatorKey: appProvider.navigatorKey,
          title: Constants.appName,
          themeMode: ThemeMode.light,
          theme: appProvider.theme,
          //darkTheme: appProvider.theme,
          home: SplashScreen(),
          navigatorObservers: [
            FirebaseAnalyticsObserver(analytics: FbAnalitics.analytics),
          ],
          locale: const Locale('ru'),
          localizationsDelegates: [
            GlobalMaterialLocalizations.delegate,
            GlobalWidgetsLocalizations.delegate,
            GlobalCupertinoLocalizations.delegate,
          ],
          supportedLocales: [const Locale('ru', 'RU')],
        );
      },
    );
  }
}