import 'dart:async';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/fosend.dart';
import 'package:foapp/util/local_auth.dart';
import 'package:foapp/util/menu_category.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:foapp/util/dadata.dart';
import 'package:foapp/util/loadingmess.dart';
import 'package:foapp/util/platform_info.dart';

class DataProvider extends ChangeNotifier{
  List<MenuCategory> categories=[];
  List<MenuCategory> addingCategories=[];
  List<MenuProduct> products=[];
  List banners=[];
  //String deliveryMap='';
  //List deliveryTexts=[];
  Map deliveryOptions={};
  Map globalSettings={};
  bool noAgreements=true;
  String departmentDef='';
  List listMenuRest=[];
  Function? onLoaded;
  Map theme={};
  String hash='';
  int timeCheckData=300;
  //String errorMess='';
  DataProvider(){
    update();
  }
  onLoadedData(Function func){
    hash='';
    onLoaded=func;
  }
  setDepartment(String dp){
    departmentDef=dp;
    update();
  }
  update() async{
    bool hasInternet=await haveInternet();
    if(!hasInternet){
      Timer(Duration(seconds: 5), update);
      return;
    }
    var fo=FoSend();
    Map ar=await getPlatformInfo();
    Map snd= {
      'datahash':hash,
      'department': departmentDef,
      'platform':ar['platform'],
      'platformVer':ar['platformVer'],
      'version':ar['version'],
    };
    var data = await fo.send('menu', snd);
    if(data['timeCheckData']!=null) timeCheckData=data['timeCheckData'];
    Timer(Duration(seconds: timeCheckData), update);
    if(data['hash']==hash) return;
    hash=data['hash'];
    LoadingMess.setMess(data['needUpdate']);
    categories=[];
    products=[];
    for(var el in data['categories']){
      MenuCategory sec=MenuCategory.fromJson(el);
      if(sec.active) {
        categories.add(sec);
      }else{
        addingCategories.add(sec);
      }
    }
    for(var el in data['products']){
      products.add(MenuProduct.fromJson(el));
    }
    if(data['auth']=='local') LocalAuthPhone.enabled=true;
    banners=data['banners'];
    //deliveryMap=data['deliveryMap'];
    //deliveryTexts=data['deliveryTexts'];
    noAgreements=data['noAgreements'];
    listMenuRest=data['listMenuRest'];
    globalSettings=data['globalSettings'];
    theme=data['theme'];
    Constants.initStyle(data['theme']);
    DadataSuggest.config=data['dadata'];
    deliveryOptions=data['delivery'];
    if(onLoaded!=null){
      onLoaded!();
      onLoaded=null;
    }
    notifyListeners();
  }
  MenuProduct? getMenuProdByFO(String fo){
    MenuProduct? res;
    products.forEach((el) {
      el.prices.forEach((e) {
        if(e['code']==fo){
          res=el;
        }
      });
    });
    return res;
  }
  MenuProduct? getMenuProdByID(int id){
    MenuProduct? res;
    products.forEach((el) {
      if(el.id==id){
        res=el;
      }
    });
    return res;
  }

  Future<Map> getReservData() async{
    var fo=FoSend();
    var data = await fo.send('get-reservation', {});
    return data;
  }
  Future putReservData(Map _data) async{
    var fo=FoSend();
    Map reserv={};
    _data.forEach((k,e){
      reserv[k]=e['val'];
    });
    reserv={'reserv':reserv};
    var data = await fo.send('put-reservation', reserv);
    print(data);
  }
}