import 'dart:async';
import 'dart:convert';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/local_auth.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:flutter/services.dart';
import 'package:foapp/util/fosend.dart';
import 'package:foapp/widgets/req_department.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:foapp/util/platform_info.dart';

class UserStorage {
  String varName='';
  UserStorage({required this.varName});

  Future<Map> get() async {
    try {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      String contents = prefs.getString(varName)!;
      return json.decode(contents);
    } catch (e) {
      return {};
    }
  }
  Future write(Map dt) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString(varName, json.encode(dt));
  }
}



class UserProvider extends ChangeNotifier{
  Map userDef={
    'phone':'',
    'name':'',
    'email':'',
    'gender':'',
    'birthday':'',
    'bonuses':'0',
    'sms':true,
    'discount':{'discount':0,'coupon':'','error':'','personal':0,'action':0},
    'department':'',
  };
  late BuildContext mainContext;
  String? firebaseAppToken;
  late UserStorage storage;
  Map user={};
  List basket=[];
  Map addressNew={};
  late OrderMotor order;
  UserProvider({required BuildContext context}){
    mainContext=context;
    storage=UserStorage(varName:'user');
    init();
    update();
  }

  init() async{

    Map data=await storage.get();
    user=data['user']??Map.from(userDef);
    //user=Map.from(userDef);
    String phone=LocalAuthPhone.phone;
    if(phone.length>0) user['phone']=phone;
    user['discount']['error']='';
    //user['department']='';
    //userSync();
    basket=data['basket']??[];
    order=OrderMotor(userProv:this);
    DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);

    if(user.containsKey('department') && user['department'].length>0) {
      dataProv.setDepartment(user['department']);
      order.setDepartment=true;
    }
  }
  void setDepartment(String dp){
    DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);
    dataProv.setDepartment(user['department']);
    user['department']=dp;
    save(noRefresh:true);
    order.setDepartment=true;
    order.checkDiscount();
  }

  save({bool noRefresh=false}) async{
    basket.removeWhere((element){
      if(element['qty']<=0 && element['props'].containsKey('present')) user['discount']['coupon']='';
      return element['qty']<=0;
    });
    storage.write({
      'user':user,
      'basket':basket,
    });
    if(!noRefresh) notifyListeners();
  }

  void setFirebaseToken(String? token){
    firebaseAppToken=token;
    var fo=FoSend();
    getPlatformInfo().then((Map packageInfo) {
      fo.send('pushtoken', {'token': firebaseAppToken, 'platform': packageInfo['platform'],'platformVer':packageInfo['platformVer'],'version':packageInfo['version']});
      userSync();
    });
  }

  num getBasketCnt(){
    num cnt=0;
    basket.forEach((value){cnt+=value['qty'];});
    return cnt;
  }
  test(){
    basket.add({'qty':1});
    print(basket);
    update();
  }
  update() async{
    //Timer(Duration(seconds: 2), test);
    //var fo=FoSend();
    //var data = await fo.send('menu', {});


    notifyListeners();
  }
  /*** USER ***/
  void userLogout(){
    LocalAuthPhone.logout();
    user=Map.from(userDef);
    order.initFieldsDef();
    order.save();
    FbAnalitics.userLogout();
    notifyListeners();
  }
  void userLogin() async{
    user['phone']=LocalAuthPhone.phone;
    FbAnalitics.userLogin();
    userSync();
    notifyListeners();
  }
  Future<void> userSync() async{
    if(user['phone'].length<=0 && LocalAuthPhone.phone.length>0) user['phone']=LocalAuthPhone.phone;
    if(user['phone'].length>0){
      var fo=FoSend();
      Map data = await fo.send('userinfo', {'phone':user['phone'],'basket':basket,'token':firebaseAppToken});
      if(data.containsKey('name')) user['name']=data['name'];
      if(data.containsKey('email')) user['email']=data['email'];
      if(data.containsKey('gender')) user['gender']=data['gender'];
      if(data.containsKey('birthday')) user['birthday']=data['birthday'];
      if(data.containsKey('bonuses')) user['bonuses']=data['bonuses'].toString();
      if(data.containsKey('sms')) user['sms']=data['sms'];
      if(user['sms']!=true){

      }

      if(data.containsKey('discount')){
        user['discount']['personal']=int.parse(data['discount'].toString());
        checkUserDefDiscount();
      }
      save(noRefresh: false);
    }
  }
  void userSave() async{
    if(user['phone'].length>0) {
      if(user['sms']){

      }else{

      }
      var fo = FoSend();
      fo.send('usersave', user);
      FbAnalitics.event(event: 'eventUserSave');
    }
  }
  bool couponDiscount(){
    bool res=false;
    try{
      DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);
      if(dataProv.globalSettings['couponDiscount']=='y') res=true;
    } catch (e) {

    }
    return res;
  }

  void checkUserDefDiscount(){
    if(couponDiscount() || user['discount']['coupon'].length<=0){
      user['discount']['discount']=(user['discount']['personal']>user['discount']['action']?user['discount']['personal']:user['discount']['action']);
    }
  }
  void userDeleteAddress(String id){
    var fo=FoSend();
    fo.send('deleteaddress', {'phone':user['phone'],'id':id});
  }
  Future<String> userAddAddress(Map newAddr) async{
    var fo=FoSend();
    Map data = await fo.send('newaddress', {'phone':user['phone'],'address':newAddr});
    newAddr['id']=data['id'].toString();
    order.addresses.add(newAddr);
    FbAnalitics.event(event: 'eventUserAddressAdd');
    return data['id'].toString();
  }
  Future<dynamic> userOrders() async{
    var fo=FoSend();
    Map data=await fo.send('userorders', {'phone':user['phone']});
    return data['orders'];
  }

  /*** BASKET ***/
  Map basketGetCart(){
    DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);
    Map res={
      'price':0.0,
      'discount':0.0,
      'delivery':0.0,
      'amount':0.0,
      'warning':'',
      'wartype':'',
      'basket':[],
      'nodelivery':false,
      'noorder':false,
    };
    int jb=0;
    basket.forEach((el) {
      MenuProduct? prod=dataProv.getMenuProdByFO(el['fo']);
      if(prod==null){
        el['qty']=0;
        return;
      }
      List<String> parts=[];
      Map price=Map.from(prod.prices.where((pr) => pr['code']==el['fo']).toList()[0]);
      price['price']=double.parse(price['price'].toString());
      price['inBasket']=el['qty'];
      //el['props']= {};
      if(el['props'].containsKey('parts') && el['props']['parts']!=null){
        el['props']['parts'].forEach((k,cd){
          MenuProduct? pr=dataProv.getMenuProdByFO(cd);
          if(pr!=null) parts.add(pr.name);
        });
      }
      if(el['props'].containsKey('ings') && el['props']['ings']!=null){
        el['props']['ings'].forEach((cd,q){
          q=double.parse(q.toString());
          MenuProduct? pr=dataProv.getMenuProdByFO(cd);
          if(pr!=null && pr.prices.length>0) {
            Map pc = pr.prices.where((p) => p['code'] == cd).toList()[0];
            parts.add(pr.name + (q > 1
                ? (' (' + q.round().toString() +' '+ getMessage('units') + ')')
                : ''));
            price['price'] += double.parse(pc['price'].toString()) * double.parse(q.toString());
          }
        });
      }
      double discount=price['price']*(price['discount']?(user['discount']['discount']/100):0.0);
      discount=double.parse(discount.toStringAsFixed(2));

      Map bs={
        'index':jb,
        'product':prod,
        'id':el['id'],
        'qty':el['qty'],
        'props':el['props'],
        'name':prod.name,
        'pic':prod.picm,
        'desc':prod.desc,
        'parts':parts.join(', '),
        'price':price['price'],
        'weight':price['weight'],
        'stop':price['stop'],
        'discount':discount,
        'amount':price['price']-discount,
        'isPresent':el['props'].containsKey('present'),
      };
      if(!price['stop']){
        res['price']+=bs['price']*el['qty'];
        res['discount']+=bs['discount']*el['qty'];
        res['amount']+=bs['amount']*el['qty'];
      }
      res['basket'].add(bs);
      jb++;
    });
    int minDelivOrder=int.parse(dataProv.deliveryOptions['minDeliveryOrder'].toString());
    if(minDelivOrder>res['amount']){
      res['wartype']='error';
      res['warning']=getMessage('delivery_min_order'+(dataProv.globalSettings['no_samovivoz']=='y'?'_ws':''),{'amount':minDelivOrder});
      res['nodelivery']=true;
    }
    if(res['nodelivery'] && dataProv.globalSettings['no_samovivoz']=='y') res['noorder']=true;
    return res;
  }
  int basketAdd(String fo,int qty,[props]){
    DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);
    if(dataProv.globalSettings['req_department']=='y' && !order.setDepartment){
      openReqDepartment(
        context: mainContext,
        addingTovar: {
          'fo':fo,
          'qty':qty,
          'props':props,
        },
      );
      return 0;
    }
    if(props==null) props={};
    bool isNew=true;
    props.removeWhere((k,v) => v==null);
    int idTovar=0;
    if(!props.containsKey('noAddItems') && !props.containsKey('present')){
      basket.forEach((el) {
        if((!el['props'].containsKey('ings') && !el['props'].containsKey('parts')) && el['fo']==fo){
          isNew=false;
          idTovar=el['id'];
          basketFbAnaliticsChange(fo,qty);
          el['qty']+=qty;
        }
      });
    }
    if(isNew && qty>0) {
      idTovar=basketUnicId();
      basket.add({'id':idTovar,'qty': qty, 'fo': fo, 'props': props});
      basketFbAnaliticsChange(fo,qty);
    }
    HapticFeedback.lightImpact();
    save();
    return idTovar;
  }
  void basketFbAnaliticsChange(String fo,int qty){
    if(FbAnalitics.enabled){
      DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);
      MenuProduct? prod=dataProv.getMenuProdByFO(fo);
      if(prod!=null) {
        String cat='';
        Map price={'price':0};
        try{
          cat = dataProv.categories.where((e) => e.id == prod.sid.first).toList().first.name;
          price = Map.from(prod.prices.where((pr) => pr['code'] == fo).toList().first);
        }catch(e){}
        FbAnalitics.cartAction(
          name: prod.name,
          fo: fo,
          cat:cat,
          qty: qty,
          price: double.parse(price['price'].toString()),
        );
      }
    }
  }
  void basketChangeQty(id,qty){
    Map bs=basket.where((el) => el['id']==id).first;
    basketFbAnaliticsChange(bs['fo'],qty-bs['qty']);
    basket.where((el) => el['id']==id).forEach((el){el['qty']=qty;});
    HapticFeedback.lightImpact();
    save();
  }
  int basketUnicId(){
    int id=0;
    basket.forEach((el){
      if(el['id']>id) id=el['id'];
    });
    id++;
    return id;
  }
  int basketCodeQty(String fo){
    int res=0;
    basket.forEach((el) {
      if((!el['props'].containsKey('ings') && !el['props'].containsKey('parts')) && el['fo']==fo) res=el['qty'];
    });
    return res;
  }
  Future<Map> basketCoupon({required String coupon, double amount=0,Function? funcAfter}) async{
    Map res={'type':''};
    if(coupon.length>0) {
      var fo = FoSend();
      var data = await fo.send('coupon', {'coupon': coupon,'amount':amount,'phone':user['phone']});
      if (data['error']) {
        res['type']='error';
        res['mess']=data['mess'];
      }else{
        FbAnalitics.event(event: 'eventCartCoupon',params:{'coupon':coupon});
        res['type']='mess';
        res['mess']=data['mess'];
        user['discount']['coupon']=coupon;
        if(!couponDiscount()) user['discount']['discount']=0;
        basket.removeWhere((el) => el['props'].containsKey('present'));
        if(data['type']=='discount'){
          user['discount']['discount']=data['discount'];
        }else if(data['type']=='present'){
          basket.add({'id':basketUnicId(),'qty': 1, 'fo': data['present']['fo_code'], 'props': {'present':true,'coupon':coupon}});
        }
        save();
      }
    }else{
      if(user['discount']['coupon']!='') {
        user['discount']['coupon'] = '';
        user['discount']['discount'] = 0;
        checkUserDefDiscount();
        save();
      }
    }
    if(funcAfter!=null) funcAfter(res);
    return res;
  }
  void basketCopyOrder(Map order){
    basket=[];
    DataProvider dataProv=Provider.of<DataProvider>(mainContext, listen: false);
    order['products'].forEach((el){
      MenuProduct? prodCatalog=dataProv.getMenuProdByFO(el['code']);
      if(prodCatalog!=null){
        Map props={};
        if(el['ings']!=null){
          props['ings']=el['ings'];
        }
        if(el['parts']!=null){
          props['parts']=el['parts'];
        }
        basketAdd(el['code'], int.parse(el['qty'].toString().replaceAll('.00', '')),props);
      }
    });
    FbAnalitics.event(event: 'copyOrder');
  }
  void basketReset(){
    basket=[];
    user['coupon']=userDef['coupon'];
    user['discount']['discount'] = 0;
    if(user['discount']['personal']>0) user['discount']['discount']=user['discount']['personal'];
    if(order.inited) {
      order.fields['remark']!['value'] = '';
      order.fields['persons']!['value'] = 1;
      order.fields['date']!['value'] = 'now';
      order.fields['time']!['value'] = '';
      order.fields['odd']!['value'] = '';
      order.fields['bonuses']!['value'] = '0';
      order.save();
    }
    save();
  }

  /** Order **/
  Future<Map> payAppleGoogle({required dynamic orderId,required String paySystem,required String payToken})async{
    var fo=FoSend();
    //print({'orderId':orderId,'paySystem':paySystem,'payToken':payToken});
    Map res=await fo.send('applegooglepay', {'orderId':orderId,'paySystem':paySystem,'payToken':payToken});
    return res;
  }

}


class OrderMotor{
  late UserProvider userProv;
  late UserStorage storage;
  late List departments;
  late List payments;
  late List addresses;
  double deliveryPrice=0.0;
  late Map<String, Map> fields;
  bool inited=false;
  bool setDepartment=false;
  void initFieldsDef() {
    fields = {
      'fio': {
        'title': getMessage('of_fio'),
        'value': '',
        'type': 'text',
        'error': '',
      },
      'persons': {
        'title': getMessage('of_persons'),
        'value': 1,
        'type': 'plm',
        'error': '',
      },
      'type': {
        'title': getMessage('of_type'),
        'value': 'delivery',
        'type': 'typeaddress',
        'list': {
          'delivery': getMessage('delivery'),
          'samovivoz': getMessage('samovivoz'),
        },
        'error': '',
      },
      'address': {
        'title': getMessage('of_address'),
        'value': '',
        'type': 'address',
        'error': '',
      },
      'date': {
        'title': getMessage('date'),
        'value': 'now',
        'type': 'list',
        'list': {},
        'error': '',
      },
      'time': {
        'title': getMessage('time'),
        'value': '',
        'type': 'list',
        'list': {},
        'error': '',
      },
      'department': {
        'title': getMessage('of_department'),
        'value': '',
        'type': 'department',
        'error': '',
      },
      'payment': {
        'title': getMessage('of_payment'),
        'value': '',
        'type': 'payment',
        'error': '',
      },
      'odd': {
        'title': getMessage('of_odd'),
        'value': '',
        'type': 'text',
        'error': '',
      },
      'bonuses': {
        'title': getMessage('of_bonuses'),
        'value': '0',
        'type': 'bonuses',
        'max':'0',
        'error': '',
      },
      'remark': {
        'title': getMessage('of_remark'),
        'value': '',
        'type': 'textarea',
        'error': '',
      },

    };
  }
  OrderMotor({required this.userProv}){
    storage=UserStorage(varName:'order');
    //init();
  }
  Future<void> init({bool noDeliv=false}) async{
    initFieldsDef();
    deliveryPrice=0.0;
    Map data=await storage.get();
    if(!data.containsKey('fields')) data['fields']={};
    data['fields'].forEach((k,v){
      if(fields.containsKey(k)) fields[k]!['value']=v;
    });

    if(noDeliv) fields['type']!['value']='samovivoz';
    if(fields['fio']!['value'].length<=0) fields['fio']!['value']=userProv.user['name'];
    if(fields['department']!['value'].length<=0) fields['department']!['value']=userProv.user['department'];
    var fo=FoSend();
    Map basketCart=userProv.basketGetCart();
    Map arDs={
      'phone':userProv.user['phone'],
      'type':fields['type']!['value'],
      'discount':userProv.user['discount'],
      'amount':basketCart['amount'],
      'address':fields['address']!['value'],
      'department':fields['department']!['value'],
      'basket':userProv.basket,
    };
    data = await fo.send('orderinf', arDs);
    if(fo.errorFo.length>0){
      data = await fo.send('orderinf', arDs);
    }
//print(arDs);
    departments=data['departments'];
    departments.forEach((el) {el['code']=el['code'].toString();});
    payments=data['payments'];
    departments.forEach((el) {el['id']=el['id'].toString();});
    addresses=data['addresses'];

    fields['bonuses']!['max']=data['bonuses'].toString();

    fields['date']!['list']=data['when_delivery']['date'];
    fields['time']!['list']=data['when_delivery']['time'];

    userProv.user['discount']['action']=int.parse(data['discount'].toString());

    if(fields['address']!['value'].length>0){
      bool hs=false;
      addresses.forEach((el){
        if(el['id']==fields['address']!['value']) hs=true;
      });
      if(!hs){
        fields['address']!['value']='';
        save();
      }
    }
    if(addresses.length>0 && fields['address']!['value'].length<=0) fields['address']!['value']=addresses[0]['id'];
    await checkDiscount();
    //await Future.delayed(const Duration(seconds: 1));
    inited=true;
  }
  Future<void> checkDiscount() async{
    var fo=FoSend();
    Map basketCart=userProv.basketGetCart();
    Map arDs={
      'phone':userProv.user['phone'],
      'type':fields['type']!['value'],
      'discount':userProv.user['discount'],
      'delivery':deliveryPrice,
      'amount':basketCart['amount'],
      'address':fields['address']!['value'],
      'department':fields['department']!['value'],
      'basket':userProv.basket,
    };
    Map data = await fo.send('orderdiscount', arDs);
    userProv.user['discount']['action']=int.parse(data['discount'].toString());
    userProv.checkUserDefDiscount();
  }
  Future<void> save() async{
    Map fl={};
    fields.forEach((k, v) {fl[k]=v['value'];});
    storage.write({
      'fields':fl,
    });
  }
  void setField(String key, String val){
    if(fields.containsKey(key)){
      if(key=='fio'){
        userProv.user['name']=val;
        userProv.save(noRefresh:true);
        userProv.userSave();
      }
      if(key=='department'){
        userProv.setDepartment(val);
      }
      fields[key]!['value']=val;
      save();
    }
  }
  Future<Map> send() async{
    Map res={'error':'','data':{}};
    var hasError=false;
    fields['fio']!['error']='';
    if(fields['fio']!['value'].length<=0){
      fields['fio']!['error']=getMessage('req_field');
      hasError=true;
    }
    if(fields['type']!['value']=='samovivoz'){
      if(fields['department']!['value'].length<=0){
        hasError=true;
        res['error']=getMessage('of_department_choose_for_samovivoz');
      }
    }else{
      if(fields['address']!['value'].length<=0){
        hasError=true;
        res['error']=getMessage('of_address_choose');
      }
    }
    if(!hasError){
      var fo=FoSend();
      Map basketCart=userProv.basketGetCart();
      Map fldList={};
      fields.forEach((k, el) {fldList[k]=el['value'];});
      Map r=await fo.send('orderadd', {'phone':userProv.user['phone'],'firebase':userProv.firebaseAppToken,'deliveryPrice':deliveryPrice,'basket':userProv.basket,'user':userProv.user,'amount':basketCart['amount'],'order':fldList});
      //print(r);
      if(r['error'].length>0) res['error']=r['error'];
      else{
        userProv.user['discount']['action']=0;
        res['data']=r['data'];
        res['data']['orderNum']=res['data']['orderNum'].toString();
        FbAnalitics.event(event: 'eventOrderSaved',params: {'order':res['data']['orderNum'].toString(),'sum':basketCart['amount']});
        FbAnalitics.ecommercePurchase(oid: res['data']['orderNum'],sum: basketCart['amount'],coupon: userProv.user['discount']['coupon']);
        userProv.basketReset();
        userProv.setDepartment('');
        setDepartment=false;
      }
    }
    res['hasError']=hasError;
    return res;
  }
  Future<Map> checkAddress() async{
    var fo=FoSend();
    Map basketCart=userProv.basketGetCart();
    Map fldList={};
    fields.forEach((k, el) {fldList[k]=el['value'];});
    Map res=await fo.send('checkaddress', {'phone':userProv.user['phone'],'basket':userProv.basket,'user':userProv.user,'amount':basketCart['amount'],'order':fldList});
    //print(res);
    return res;
  }
  Future<Map> checkAddressAnonim(Map address) async{
    var fo=FoSend();
    Map res=await fo.send('checkaddressanonim', {'phone':userProv.user['phone'],'address':address});
    return res;
  }
}