import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/html_block.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/page_button.dart';
import 'package:foapp/widgets/webview_page.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:foapp/screens/reservation.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/tools.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/screens/deliverymap.dart';
import 'package:foapp/util/fosend.dart';
import 'package:foapp/screens/agreementdetail.dart';
import 'package:foapp/util/popup.dart';
import 'package:url_launcher/url_launcher.dart' as UrlLauncher;
import 'package:foapp/widgets/empty_widget.dart';

import 'cafes.dart';

class AboutScreen extends StatefulWidget {
  @override
  _AboutScreenState createState() => _AboutScreenState();
}

class _AboutScreenState extends State<AboutScreen> with AutomaticKeepAliveClientMixin<AboutScreen>{
  late DataProvider  dataProv;
  double deliveryTextHeight=200;

  @override
  void initState() {
    super.initState();
    dataProv=Provider.of<DataProvider>(context, listen: false);
    FbAnalitics.pageSet('About');
  }

  Future<List> _getAgreements() async{
    var fo=FoSend();
    Map data = await fo.send('agreements', {});
    return data['agreements'];
  }


  @override
  Widget build(BuildContext context) {
    super.build(context);

    List<Widget> _arSocs=[];
    dataProv.deliveryOptions['socs'].forEach((k,l){
      IconData icon=FontAwesomeIcons.facebook;
      if(k=='ig'){
        icon=FontAwesomeIcons.instagram;
        _arSocs.add(InkWell(
          child: SvgPicture.asset(
            "assets/instagram.svg",
            fit: BoxFit.cover,
            width: 34.0,
            height: 34.0,
          ),
          onTap: (){
            UrlLauncher.launch(l);
          },
        ));
      }else{
        Color clr=getConstantColor('main');
        if(k=='fb'){
          icon=FontAwesomeIcons.facebook;
          clr=Color(0xff4267B2);
        }else if(k=='vk') {
          icon = FontAwesomeIcons.vk;
          clr=Color(0xff4c75a3);
        }
        _arSocs.add(InkWell(
          child: Icon(
            icon,
            size: 36,
            color: clr,
          ),
          onTap: (){
            UrlLauncher.launch(l);
          },
        ));
      }
    });

    return Scaffold(
      appBar:AppBarSimple(
        title: getMessage('about_us'),
        noBack: true,
        act: GestureDetector(
          onTap: (){
            UrlLauncher.launch('tel:'+dataProv.deliveryOptions['phone_clear']);
          },
          child:Container(
            margin:EdgeInsets.only(right:10),
            child:Row(
              children:[
                Icon(
                  Icons.call,
                  size: 22,
                  color:getConstantColor('call'),
                ),
                Text(
                  ' '+getMessage('call').toLowerCase(),
                  style: TextStyle(
                    fontSize:18,
                    height: 1,
                    color:getConstantColor('call'),
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
      body: ListView(
        children:[
          GridView.count(
            primary: false,
            shrinkWrap: true,
            padding:EdgeInsets.zero,
            crossAxisCount: 3,
            crossAxisSpacing: 0,
            mainAxisSpacing: 0,
            childAspectRatio: 1,
            children: [
              PageButton(
                text:getMessage('about_map_delivery'),
                icon: FontAwesomeIcons.mapMarkedAlt,
                onClick: (){
                  Navigator.of(context).push(
                    MaterialPageRoute(
                      builder: (BuildContext context){
                        return DeliveryMap();
                      },
                    ),
                  );
                },
              ),
              PageButton(
                text:getMessage('about_list_cafe'),
                icon: FontAwesomeIcons.utensils,
                onClick: (){
                  Navigator.of(context).push(
                    MaterialPageRoute(
                      builder: (BuildContext context){
                        return Cafes();
                      },
                    ),
                  );
                },
              ),
              PageButton(
                text:getMessage('make_reservation'),
                icon: FontAwesomeIcons.chair,
                onClick: (){
                  Navigator.of(context).push(
                      MaterialPageRoute(
                          builder: (BuildContext context){
                            return Reservation();
                          }
                      )
                  );
                },
              ),
              /*Expanded(
                flex: 1,
                child:btnPage(
                  text:getMessage('menu_restaurant'),
                  icon: Icons.menu_book,
                  onClick: (){
                    _openMenuRest();
                  },
                ),
              ),*/
              /*Expanded(
                flex: 1,
                child:Container(
                  margin:EdgeInsets.only(left:20,right: 20),
                  child:Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: _arSocs,
                  ),
                ),
              ),*/
            ],
          ),
          Container(
            margin:EdgeInsets.only(top:20,bottom: 20,left:70,right:70),
            child:Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: _arSocs,
            ),
          ),
          Container(
            height: deliveryTextHeight,
            padding:EdgeInsets.fromLTRB(10.0, 10.0, 10.0, 0),
            child:HtmlBlock(
              url:dataProv.deliveryOptions['deliveryTextUrl'],
              webViewHandle:(args){
                setState(() {
                  deliveryTextHeight=double.parse(args[0]['height'].toString());
                });
                return {};
              }
            ),
          ),
          widgetByCondition(
            cond:!dataProv.noAgreements,
            func: ()=>Container(
              margin:EdgeInsets.only(top:10),
              padding:EdgeInsets.only(top:10),
              decoration: BoxDecoration(
                border: Border(
                  top: BorderSide(width: 2, color: Theme.of(context).dividerColor),
                ),
              ),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children:[
                  Container(
                    margin:EdgeInsets.only(bottom:5,top:5,left:15),
                    child: Text(
                      getMessage('agreement_inf'),
                      style:TextStyle(
                        fontSize: 20,
                        fontWeight: FontWeight.bold
                      )
                    ),
                  ),
                  FutureBuilder<List>(
                      future: _getAgreements(),
                      builder: (BuildContext context, AsyncSnapshot<List> snapshot) {
                        if(snapshot.hasData){
                          List agrL=snapshot.data!;
                          return getListAgreements(
                              agrL,
                                  (el){
                                Navigator.of(context).push(
                                  MaterialPageRoute(
                                    builder: (BuildContext context){
                                      return AgreementDetail(aggrement: el);
                                    },
                                  ),
                                );
                              }
                          );
                        }else{
                          return EmptyWidget();
                      }
                    }
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
  void _openMenuRest(){
    popupContentShow(
      context: context,
      sizePopup: (dataProv.listMenuRest.length/2).ceil()/10+0.05,
      content: GridView.count(
        primary: false,
        shrinkWrap: true,
        padding:EdgeInsets.all(10),
        crossAxisCount: 2,
        crossAxisSpacing: 10,
        mainAxisSpacing: 10,
        childAspectRatio: 4,
        children: dataProv.listMenuRest.map<Widget>((e){
          return MainButton(
            title:e['title'],
            onClick: (){
              Navigator.of(context).push(
                MaterialPageRoute(
                  builder: (BuildContext context){
                    return WebViewPage(
                      title:e['title'],
                      url:e['url'],
                    );
                  }
                )
              );
            }
          );
        }).toList(),
      ),
    );
  }
  Widget getListAgreements(List agrL,Function funcDetail){
    List<Widget> lst=[];
    agrL.forEach((el) {
      lst.add(TextButton(
        style: TextButton.styleFrom(
          backgroundColor: Colors.transparent,
        ),
        child: Card(
          margin: EdgeInsets.fromLTRB(0,0,0,10),
          child: ListTile(
            title: Text(el['name']),
            trailing: Icon(Icons.arrow_forward_ios),
          ),
        ),
        onPressed: (){
          funcDetail(el);
        },
      ));
    });
    return Column(
      children: lst,
    );
  }

  @override
  bool get wantKeepAlive => true;
}
