import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/confirm_dialog.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/screens/addressnew.dart';


class Addresses extends StatefulWidget {
  List addresses;
  final Function? selectAddress;
  final Function? backFrom;
  final bool? noCloseAfterNewAdd;

  Addresses({
    required this.addresses,
    this.selectAddress,
    this.backFrom,
    this.noCloseAfterNewAdd,
  })
      :super();

  @override
  _AddressesState createState() => _AddressesState();
}

class _AddressesState extends State<Addresses> {


  @override
  Widget build(BuildContext context) {
    int lastI=widget.addresses.length;
    return Scaffold(
      appBar: AppBarSimple(
        title:getMessage('my_address'),
        backFrom:widget.backFrom,
      ),
      body: ListView.builder(
        itemCount: widget.addresses.length+1,
        itemBuilder:(BuildContext context,int i){
          return Container(
            padding:EdgeInsets.only(top:5,bottom:8,left: 8),
            decoration: BoxDecoration (
              border: Border(
                bottom: BorderSide(
                  color: Constants.colors['grayMain'],
                  width: 1.0,
                ),
              ),
            ),
            child:(lastI==i?_addressLineNew():_addressLine(widget.addresses[i])),
          );
        },
      ),
    );
  }
  Widget _addressLineNew(){
    return InkWell(
      child:Container(
        padding: EdgeInsets.only(top:5,bottom: 5),
        child:Row(
          children: [
            Expanded(
                child:Text(
                  getMessage('addr_new_add'),
                  style: TextStyle(
                    fontSize: 20,
                  ),
                )
            ),
            IconButton(
              icon: Icon(Icons.add,color: Constants.colors['red']),
              color: getConstantColor('main'),
              onPressed: _addAddress,
            ),
          ],
        ),
      ),
      onTap: _addAddress,
    );
  }
  void _addAddressAfter(String id) async{
    if(widget.selectAddress!=null) {
      dynamic data=await widget.selectAddress!(id);
      if(data!=null && widget.noCloseAfterNewAdd==true){
        widget.addresses=data;
      }
    }
    if(widget.noCloseAfterNewAdd!=true){
      Navigator.pop(context);
    }
    setState(() {});
  }
  void _addAddress(){
    Navigator.of(context).push(
      MaterialPageRoute(
        builder: (BuildContext context){
          return AddressNew(selectAddress: _addAddressAfter);
        },
      ),
    );
  }
  Widget _addressLine(address){
    return InkWell(
      child:Row(
        children: [
          Expanded(
            child:Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children:[
                Text(
                    address['AddressStr'],
                    style:TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 16,
                    )
                ),
                Text(
                    addressSecondDesc(address),
                    style:TextStyle(
                      color: Colors.grey,
                      fontSize: 14,
                    )
                ),
              ],
            ),
          ),
          IconButton(
            icon: Icon(Icons.delete, size: 22),
            color: Constants.colors['red'],
            onPressed: (){
              confirmDialog(
                context: context,
                title:getMessage('addr_delete_real'),
                funcYes: (){
                  UserProvider userProv = Provider.of<UserProvider>(context, listen: false);
                  userProv.userDeleteAddress(address['id']);
                  widget.addresses.removeWhere((el) => el['id']==address['id']);
                  setState(() {});
                },
              );
            },
          ),
        ],
      ),
      onTap: (){
        if(widget.selectAddress!=null){
          widget.selectAddress!(address['id']);
          if(widget.noCloseAfterNewAdd!=true) {
            Navigator.pop(context);
          }
        }
      },
    );
  }
}
