import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/dadata.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/main_input_field.dart';
import 'package:provider/provider.dart';


class AddressNew extends StatefulWidget {
  final Function selectAddress;

  AddressNew({
    required this.selectAddress,
    })
      :super();
  @override
  _AddressNewState createState() => _AddressNewState();
}

class _AddressNewState extends State<AddressNew> {
  Map fields={
    'AddressStr':{
      'name':getMessage('addr_start_enter'),
      'error':'',
      'value':'',
      'req':true,
    },
    'Lat':{
      'value':'0.0',
      'req':false,
    },
    'Long':{
      'value':'0.0',
      'req':false,
    },
    'CityName':{
      'name':getMessage('addr_city'),
      'error':'',
      'value':'',
      'req':false,
    },
    'StreetName':{
      'name':getMessage('addr_street'),
      'error':'',
      'value':'',
      'req':false,
    },
    'House':{
      'name':getMessage('addr_house'),
      'error':'',
      'value':'',
      'req':true,
    },
    'Building':{
      'name':getMessage('addr_building'),
      'error':'',
      'value':'',
      'req':false,
    },
    'Flat':{
      'name':getMessage('addr_flat'),
      'error':'',
      'value':'',
      'req':false,
    },
    'Porch':{
      'name':getMessage('addr_porch'),
      'error':'',
      'value':'',
      'req':false,
    },
    'Floor':{
      'name':getMessage('addr_floor'),
      'error':'',
      'value':'',
      'req':false,
    },
    'DoorCode':{
      'name':getMessage('addr_doorcode'),
      'error':'',
      'value':'',
      'req':false,
    },
    'Remark':{
      'name':getMessage('addr_remark'),
      'error':'',
      'value':'',
      'req':false,
    },
  };
  bool isLoading=false;
  late TextEditingController controllerDadata;
  late FocusNode focusNodeDadata;

  @override
  void initState() {
    super.initState();
    UserProvider userProv = Provider.of<UserProvider>(context, listen: false);
    userProv.addressNew.forEach((k, val) {
      if(fields.containsKey(k)){
        fields[k]['value']=val;
      }
    });
    controllerDadata=TextEditingController();
    focusNodeDadata=FocusNode();
  }

  @override
  void dispose() {
    controllerDadata.dispose();
    focusNodeDadata.dispose();
    super.dispose();

  }
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBarSimple(
        title:getMessage('addr_new'),
        act:TextButton(
          onPressed: _saveAddress,
          child: Text(
            getMessage('save'),
            style: TextStyle(
              color:getConstantColor('red'),
            ),
          )
        ),
      ),
      body: Container(
        padding: EdgeInsets.only(left:10,right:10),
        child:ListView(
          children: [
            Row(
              children: [
                Expanded(
                  child:dadataTextField(
                    context: context,
                    controller: controllerDadata,
                    focusNode: focusNodeDadata,
                    field:fields['AddressStr'],
                    selectItem: _selectDadata,
                    autofocus: fields['House']['value'].length<=0,
                    readonly:isLoading
                  ),
                )
              ],
            ),
            /*Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(right:15),
                    child:getField('House'),
                  ),
                ),
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(left:15),
                    child:getField('Building'),
                  ),
                ),
              ],
            ),*/
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(right:15),
                    child:getField('Flat'),
                  ),
                ),
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(left:15),
                    child:getField('Porch'),
                  ),
                ),
              ],
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(right:15),
                    child:getField('Floor'),
                  ),
                ),
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(left:15),
                    child:getField('DoorCode'),
                  ),
                ),
              ],
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children:[
                Expanded(
                  child: Container(
                    child:getField('Remark'),
                  ),
                ),
              ],
            ),
            widgetByCondition(
              cond:isLoading,
              func:()=>SpinKitThreeBounce(color: getConstantColor('main'))
            ),
          ],
        )
      ),
    );
  }
  Widget getField(String fldName,[int lines=1]){
    Map field=fields[fldName];
    return SizedBox(
      height: 75,
      child:MainInputField(
        enabled: !isLoading,
        //keyboardType: TextInputType.text,//lines>1?TextInputType.multiline:TextInputType.text,
        lines: lines,
        labelText: field['name'],
        errorText: field['error'],
        controller: TextEditingController(text: field['value']),
        onChanged: (v){
          _changeField(fldName,v);
        }
      ),
    );
  }
  void _changeField(String fldName,String val){
    fields[fldName]['value']=val;
    if(fldName=='House' || fldName=='Building'){
      _researchAddress();
      _checkErrors();
    }
    if(fields[fldName]['req']){
      _checkErrors();
    }
  }
  void _researchAddress() async{
    if((fields['AddressStr']['value'].length>0 || fields['StreetName']['value'].length>0) && fields['House']['value'].length>0){
      String str=(fields['AddressStr']['value'].length>0?fields['AddressStr']['value']:fields['StreetName']['value'])+' '+fields['House']['name']+' '+fields['House']['value'];
      if(fields['Building']['value'].length>0) str+=(' '+fields['Building']['name']+' '+fields['Building']['value']);
      List els=await DadataSuggest.autocomplate(str);
      fields['Lat']['value']=els[0]['data']['geo_lat'];
      fields['Long']['value']=els[0]['data']['geo_lon'];
      //fields['AddressStr']['value']=str;
      //_selectDadata(els[0]);
    }
  }
  void _selectDadata(Map data){
    setState(() {
      fields['AddressStr']['value']=data['value'];
      if(data['data']['qc_geo']!=null && int.parse(data['data']['qc_geo'].toString())<4) {
        fields['Lat']['value'] = data['data']['geo_lat'];
        fields['Long']['value'] = data['data']['geo_lon'];
      }
      fields['CityName']['value']=data['data']['city']!=null?data['data']['city']:'';
      fields['StreetName']['value']=data['data']['street_with_type']!=null?data['data']['street_with_type'].toString():'';
      fields['House']['value']=data['data']['house']!=null?data['data']['house'].toString():'';
      fields['Building']['value']=data['data']['block']!=null?data['data']['block'].toString():'';
      fields['Flat']['value']=data['data']['flat']!=null?data['data']['flat'].toString():'';
      _checkErrors();
      if(fields['House']['value'].length<=0){
        fields['AddressStr']['value']+=' ';
        Timer(Duration(milliseconds: 100), (){
          focusNodeDadata.requestFocus();
        });

      }
    });
  }
  void _checkErrors(){
    fields.forEach((k, el) {
      if(el['value']==null) el['value']='';
      if (el['value'].length > 0) el['error'] = '';
    });
  }
  void _saveAddress() async{
    bool hasError=false;
    fields.forEach((k, el) {
      if(el['req'] && el['value'].length<=0){
        hasError=true;
        el['error']=getMessage('req_field');
      }
    });
    if(hasError){
      setState(() {

      });
    }else {
      setState(() {
        isLoading = true;
      });
      Map newAddr = {};
      fields.forEach((k, el) {
        newAddr[k] = el['value'];
      });
      UserProvider userProv = Provider.of<UserProvider>(context, listen: false);
      String addrId = await userProv.userAddAddress(newAddr);
      await widget.selectAddress(addrId);
      isLoading = false;
      Navigator.pop(context);
    }
  }
}
