import 'package:carousel_slider/carousel_slider.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:foapp/util/phone.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/empty_widget.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/webview_page.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/screens/reservation.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/fosend.dart';
import 'package:foapp/util/popup.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/slider_item.dart';
import 'package:photo_view/photo_view.dart';
import 'package:photo_view/photo_view_gallery.dart';
import 'package:url_launcher/url_launcher.dart' as UrlLauncher;


class Cafes extends StatefulWidget {

  @override
  _CafesState createState() => _CafesState();
}

class _CafesState extends State<Cafes> {
  List cafes=[];
  bool initCafes=false;

  Future<bool> _initCafeList() async{
    if(initCafes) return true;
    var fo=FoSend();
    Map data=await fo.send('cafes', {});
    cafes=data['cafes'];
    initCafes=true;
    return true;
  }

  @override
  void initState() {
    FbAnalitics.pageSet('Cafes');
    super.initState();
  }


  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBarSimple(
        title: getMessage('cafes_title'),
      ),
      body:FutureBuilder<bool>(
          future: _initCafeList(),
          builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
            if (snapshot.hasData) {
              return getCafesList();
            } else {
              return Center(
                child:CircularProgressIndicator(),
              );
            }
          }
      ),
    );
  }

  Widget getCafesList(){
    return ListView.builder(
      itemCount: cafes.length,
      itemBuilder: (context, index) {
        return  _cafeItem(cafe: cafes[index]);
      }
    );
  }
  Widget _getGallery({required Map cafe}){
    return CarouselSlider(
      options: CarouselOptions(
        aspectRatio:2,
        autoPlay: true,
        viewportFraction: 1.0,
      ),
      items: cafe['pics'].map<Widget>((pic){
        return SliderItem(
          pic: pic['small'],
          onTap: (){
            _getGallerySlider(cafe: cafe);
          },
        );
      }).toList(),
    );
  }
  void _getGallerySlider({required Map cafe}){
    Navigator.of(context).push(
      MaterialPageRoute(
        builder: (BuildContext context){
          return Scaffold(
            appBar: AppBarSimple(
              title:cafe['name'],
            ),
            body:Container(
              child: PhotoViewGallery.builder(
                scrollPhysics: const BouncingScrollPhysics(),
                backgroundDecoration: BoxDecoration(
                  color: Theme.of(context).backgroundColor,
                ),
                builder: (BuildContext context, int index) {
                  return PhotoViewGalleryPageOptions(
                    imageProvider: NetworkImage(cafe['pics'][index]['src']),
                    minScale: PhotoViewComputedScale.contained,
                  );
                },
                itemCount: cafe['pics'].length,
                loadingBuilder: (context, progress) => Center(
                  child: Container(
                    width: 60.0,
                    height: 60.0,
                    child: CircularProgressIndicator(),
                  ),
                ),
              )
            ),
          );
        },
      ),
    );
  }
  Widget _cafeItem({required Map cafe}){
    return Container(
      margin: EdgeInsets.only(left:15.0,right:15.0,bottom:15),
      padding:EdgeInsets.fromLTRB(0, 10, 0, 10),
      decoration: BoxDecoration(
        border: Border(bottom: BorderSide(width: 3, color: Theme.of(context).dividerColor)),
      ),
      child:Column(
        children: [
          Container(
            margin:EdgeInsets.only(bottom: 10),
            child:_getGallery(cafe:cafe),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Flexible(
                child:Text(
                  cafe['name'],
                  style:TextStyle(
                    fontSize:22.0,
                    color: getConstantColor('text'),
                    fontWeight: FontWeight.w500,
                  ),
                ),
              ),
            ],
          ),
          InkWell(
            child:Container(
              margin:EdgeInsets.only(top:10,bottom: 5),
              child:Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Flexible(
                    child:Text(
                      cafe['address'],
                      style:TextStyle(
                        fontSize:14.0,
                      ),
                    ),
                  ),
                  Container(
                    padding:EdgeInsets.only(left:10),
                    child:Text(
                      getMessage('cafe_at_map'),
                      style:TextStyle(
                        color:getConstantColor('red'),
                      )
                    ),
                  ),
                ],
              ),
            ),
            onTap: (){
              var fo=FoSend();
              Navigator.of(context).push(
                  MaterialPageRoute(
                      builder: (BuildContext context){
                        return WebViewPage(
                          title:cafe['name'],
                          url:fo.getUrlAction('cafes',['id='+cafe['id']]),
                        );
                      }
                  )
              );
              /*popupContentShow(
                context: context,
                sizePopup: 0.75,
                content: Container(
                  child: WebView(
                    javascriptMode: JavascriptMode.unrestricted,
                    initialUrl: fo.getUrlAction('cafes',['id='+cafe['id']]),
                  )
                ),
              );*/
            },
          ),
          Container(
            margin: EdgeInsets.only(top:10),
            padding:EdgeInsets.fromLTRB(0, 10, 0, 10),
            decoration: BoxDecoration(
              border: Border(top: BorderSide(width: 1, color: Theme.of(context).dividerColor.withAlpha(20))),
            ),
            child:Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children:[
                Row(
                  children: [
                    Icon(
                      FontAwesomeIcons.clock,
                      size: 18,
                      color:Theme.of(context).dividerColor
                    ),
                    Flexible(
                      child:Container(
                        padding:EdgeInsets.only(left:10),
                        child:Text(
                          cafe['timework'],
                          style:TextStyle(

                          ),
                        ),
                      ),
                    )
                  ],
                ),
                widgetByCondition(
                  cond:cafe['warning'].length>0,
                  func:()=>Container(
                    padding:EdgeInsets.only(top:10),
                    child:Text(
                      cafe['warning'],
                      style:TextStyle(
                        color:getConstantColor('red'),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
          Container(
            margin: EdgeInsets.only(top:10),
            padding:EdgeInsets.fromLTRB(0, 10, 0, 5),
            decoration: BoxDecoration(
              border: Border(top: BorderSide(width: 1, color: Theme.of(context).dividerColor.withAlpha(20))),
            ),
            child:Container(
              child:Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  InkWell(
                    child:Container(
                      child:Row(
                        children: [
                          Icon(
                              FontAwesomeIcons.phone,
                              size: 16,
                              color:Theme.of(context).dividerColor
                          ),
                          Container(
                            padding:EdgeInsets.only(left:10),
                            child:Text(
                              cafe['phone'].toString(),
                              style:TextStyle(

                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                    onTap: (){
                      UrlLauncher.launch('tel:+7'+getPhoneClear(cafe['phone']));
                    },
                  ),
                  widgetByCondition(
                    cond:cafe['email']!=null,
                    func:()=>InkWell(
                      child:Container(
                        child:Row(
                          children: [
                            Icon(
                                FontAwesomeIcons.at,
                                size: 19,
                                color:Theme.of(context).dividerColor
                            ),
                            Container(
                              padding:EdgeInsets.only(left:10),
                              child:Text(
                                cafe['email'],
                                style:TextStyle(

                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                      onTap: (){
                        UrlLauncher.launch('mailto:'+cafe['email']);
                      },
                    ),
                  ),
                  widgetByCondition(
                    cond:cafe['parking'],
                    func:()=>Container(
                      child:Row(
                        children: [
                          Icon(
                              FontAwesomeIcons.car,
                              size: 19,
                              color:Theme.of(context).dividerColor
                          ),
                          Container(
                            padding:EdgeInsets.only(left:10),
                            child:Text(
                              getMessage('cafe_parking'),
                              style:TextStyle(

                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
          Container(
            width: double.infinity,
            margin: EdgeInsets.only(top:10),
            padding:EdgeInsets.fromLTRB(0, 10, 0, 5),
            decoration: BoxDecoration(
              border: Border(top: BorderSide(width: 1, color: Theme.of(context).dividerColor.withAlpha(20))),
            ),
            child:Container(
              child:Row(
                //mainAxisAlignment: MainAxisAlignment.spaceEvenly,// .spaceBetween,
                children: [
                  Expanded(
                    flex: 8,
                    child:MainButton(
                      title:getMessage('make_reservation'),
                      onClick: (){
                        Navigator.of(context).push(
                            MaterialPageRoute(
                                builder: (BuildContext context){
                                  return Reservation(defRest:cafe['id'].toString());
                                }
                            )
                        );
                      }
                    )
                  ),
                  widgetByCondition(
                    cond:cafe['menu']!=null && cafe['menu'].length>0,
                    func:()=>Expanded(
                      flex: 1,
                      child:EmptyWidget(),
                    ),
                  ),
                  widgetByCondition(
                    cond:cafe['menu']!=null && cafe['menu'].length>0,
                    func:()=>Expanded(
                      flex: 8,
                      child:MainButton(
                        title:getMessage('menu_restaurant'),
                        onClick: (){
                          _openMenuRest(cafe['menu']);
                        }
                      )
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      )
    );
  }
  void _openMenuRest(menu){
    popupContentShow(
      context: context,
      sizePopup: (menu.length/2).ceil()/10+0.05,
      content: GridView.count(
        primary: false,
        shrinkWrap: true,
        padding:EdgeInsets.all(10),
        crossAxisCount: 2,
        crossAxisSpacing: 10,
        mainAxisSpacing: 10,
        childAspectRatio: 4,
        children: menu.map<Widget>((e){
          return MainButton(
              title:e['name'],
              onClick: (){
                UrlLauncher.launch(e['src']);
                /*Navigator.of(context).push(
                    MaterialPageRoute(
                        builder: (BuildContext context){
                          return WebViewPage(
                            title:e['name'],
                            url:e['src'],
                          );
                        }
                    )
                );*/
              }
          );
        }).toList(),
      ),
    );
  }
}
