import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/confirm_dialog.dart';
import 'package:foapp/widgets/loader.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/price_format.dart';
import 'package:foapp/widgets/show_message.dart';
import 'package:foapp/widgets/warning_mess.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/screens/checkout.dart';
import 'package:foapp/widgets/cart_item.dart';

class CartScreen extends StatefulWidget {
  @override
  _CartScreenState createState() => _CartScreenState();
}

class _CartScreenState extends State<CartScreen>{
  Map basketObj={};
  final TextEditingController _couponlControl = new TextEditingController();
  void _checkCoupon([bool noChEq=false]) async{
    WidgetsBinding.instance!.addPostFrameCallback((_) async{
      //DataProvider dataProv=Provider.of<DataProvider>(context, listen: false);
      UserProvider userProv = Provider.of<UserProvider>(context, listen: false);
      String oldCp = userProv.user['discount']['coupon'];
      if(noChEq && _couponlControl.text==oldCp) return;
      final overlay = LoadingOverlay.of(context);
      overlay.show();
      Map res = await userProv.basketCoupon(coupon:_couponlControl.text,amount:basketObj['amount']);
      overlay.hide();
      if (res['type'] != '') {
        showMessage(type:res['type'],message:res['mess'], context:context);
        //FocusScope.of(context).unfocus();
      }
    });

  }

  @override
  void dispose() {

    _couponlControl.dispose();
    super.dispose();
  }
  @override
  void initState() {
    super.initState();
    //UserProvider userProv=Provider.of<UserProvider>(context, listen: false);
    FbAnalitics.pageSet('Cart');
  }

  Widget build(BuildContext context) {
    //DataProvider dataProv=Provider.of<DataProvider>(context, listen: false);
    UserProvider userProv=Provider.of<UserProvider>(context, listen: false);
    basketObj=userProv.basketGetCart();
    _couponlControl.text=userProv.user['discount']['coupon'];
    double couponRadius=getConstantStyle('radiusSizeCoupon');


    return Scaffold(
      appBar: AppBarSimple(
        noBack: true,
        title: getMessage('basket'),
        act: basketObj['basket'].length<=0?null:InkWell(
          child: Container(
            padding:EdgeInsets.fromLTRB(10, 5, 15, 5),
            child:Row(
              children:[
                Icon(
                  FontAwesomeIcons.trashAlt,
                  size: 18,
                  color: getConstantColor('red'),
                ),
              ],
            ),
          ),
          onTap: (){
            confirmDialog(
              context: context,
              title:getMessage('clear_basket_confirm'),
              funcYes: (){
                userProv.basketReset();
              }
            );
          },
        ),
      ),
      body: Container(
        child: widgetByCondition(
          cond:basketObj['basket'].length<=0,
          func:()=>Padding(
            padding: EdgeInsets.fromLTRB(10.0,0,10.0,0),
            child:Center(
              child:Text(
                getMessage('basket_is_empty'),
                style: TextStyle(
                  fontSize:22,
                  fontWeight: FontWeight.w500
                ),
              ),
            ),
          ),
          funcFls:()=>Column(
            children:[
              Expanded(
                child:Padding(
                  padding: EdgeInsets.fromLTRB(0.0,0,0.0,0),
                  child:ListView(
                    children: [
                      WarningMess(text:basketObj['warning'],type:basketObj['wartype']),
                      Column(
                        children: basketObj['basket'].map<Widget>((product){
                          String plType='c';
                          if(product['index']==0) plType='f';
                          else if(product['index']==basketObj['basket'].length-1) plType='l';
                          return CartItem(product: product, addAction:userProv.basketAdd,changeBasketQty:userProv.basketChangeQty,placeType:plType);
                        }).toList(),
                      )
                    ],
                  ),
                ),
              ),
              widgetByCondition(
                cond:basketObj['amount']>0,
                func:()=>Card(
                  elevation: 4.0,
                  //color: Colors.transparent,
                  child: Container(
                    height: 120,
                    child: ListView(
                      physics: NeverScrollableScrollPhysics(),
                      children: <Widget>[
                        Padding(
                          padding: EdgeInsets.all(10),
                          child: Container(
                            decoration: BoxDecoration(
                              color: Colors.grey[200],
                              borderRadius: BorderRadius.all(
                                Radius.circular(couponRadius),
                              ),
                            ),
                            child: Stack(
                              alignment: Alignment.centerRight,
                              children: <Widget>[
                                TextField(
                                  style: TextStyle(
                                    fontSize: 15.0,
                                    color: Colors.black,
                                  ),
                                  decoration: InputDecoration(
                                    contentPadding: EdgeInsets.all(7.0),
                                    errorText: null,
                                    border: OutlineInputBorder(
                                      borderRadius: BorderRadius.circular(couponRadius),
                                      borderSide: BorderSide(color: Colors.grey[200]!,),
                                    ),
                                    enabledBorder: OutlineInputBorder(
                                      borderSide: BorderSide(color: Colors.grey[200]!,),
                                      borderRadius: BorderRadius.circular(couponRadius),
                                    ),
                                    prefixIcon: Icon(
                                      Icons.card_giftcard,
                                      color: getConstantColor('main'),
                                    ),
                                    hintText: getMessage('coupon'),
                                    hintStyle: TextStyle(
                                      fontSize: 15.0,
                                      color: Colors.grey,
                                    ),
                                  ),
                                  maxLines: 1,
                                  controller: _couponlControl,
                                ),
                                IconButton(
                                  onPressed: () => _checkCoupon(),
                                  icon: Icon(Icons.send),
                                  color: getConstantColor('main'),
                                ),
                              ],
                            ),
                          ),
                        ),

                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[

                            Padding(
                              padding: EdgeInsets.fromLTRB(10,5,5,0),
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: <Widget>[
                                  Row(
                                    crossAxisAlignment: CrossAxisAlignment.end,
                                    children: [
                                      Text(
                                        getMessage('discount')+": ",
                                        style: TextStyle(
                                          fontSize: 14,
                                          fontWeight: FontWeight.w400,
                                        ),
                                      ),
                                      PriceFormatText(
                                        price:basketObj['discount'],
                                        fontSize: 14,
                                        //fontWeight: FontWeight.w700,
                                      ),
                                    ],
                                  ),
                                  Row(
                                    crossAxisAlignment: CrossAxisAlignment.end,
                                    children: [
                                      Text(
                                        getMessage('amount')+": ",
                                        style: TextStyle(
                                          fontSize: 16,
                                          fontWeight: FontWeight.w400,
                                        ),
                                      ),
                                      PriceFormatText(
                                        price:basketObj['amount'],
                                        fontSize: 16,
                                        fontWeight: FontWeight.w700,
                                      ),
                                    ],
                                  ),
                                ],
                              ),
                            ),
                            widgetByCondition(
                              cond: !basketObj['noorder'],
                              func: ()=>Container(
                                padding: EdgeInsets.fromLTRB(5,5,10,5),
                                width: 190.0,
                                height: 50.0,
                                child: MainButton(
                                  contIn: Text(
                                    getMessage('make_order_b').toUpperCase(),
                                    style: TextStyle(
                                      color: getConstantColor('btnText'),
                                      fontWeight: FontWeight.w600
                                    ),
                                  ),
                                  onClick: (){
                                    _checkCoupon(true);
                                    Navigator.of(context).push(
                                      MaterialPageRoute(
                                        builder: (BuildContext context){
                                          return Checkout();
                                        },
                                      ),
                                    );
                                  },
                                ),
                              ),
                            ),

                          ],
                        ),



                      ],
                    ),
                  ),
                ),
              ),
            ],
          )
        ),
      ),
    );
  }

}
