import 'package:flutter/material.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/menu_category.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/basket_icon.dart';
import 'package:foapp/widgets/grid2_product.dart';
import 'package:foapp/widgets/grid_product.dart';
import 'package:foapp/widgets/line_product.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:foapp/util/fbanalitics.dart';

class Category extends StatefulWidget {
  @required final MenuCategory category;
  Category({
    Key? key,
    required this.category,
  })
      :super(key: key);
  @override
  _CategoryState createState() => _CategoryState();
}

class _CategoryState extends State<Category> {

  @override
  void initState() {
    FbAnalitics.pageCategory(cat:widget.category.name);
    super.initState();
  }


  @override
  Widget build(BuildContext context) {
    UserProvider userProv=Provider.of<UserProvider>(context, listen: false);
    DataProvider dataProv=Provider.of<DataProvider>(context, listen: false);
    userProv.basketGetCart();
    MenuCategory category=widget.category;
    List<MenuProduct> foods=dataProv.products.where((e) => e.sid.contains(category.id)).toList();
    List<Widget> arRows=[];
    if(category.desc.length>0){
      arRows.add(Container(
        padding: EdgeInsets.all(10),
        child:Text(
          category.desc,
          style: TextStyle(
            fontSize: getConstantStyle('fontSizeCatText'),
          ),
        ),
      ));
    }
    if(!category.noitems) {
      double pd=getConstantStyle('gridSpaceSize');
      bool isLargeDevice=MediaQuery.of(context).size.width>600;
      int maxCl=Constants.styles['categoryTypeList']=='grid2'?2:1;
      maxCl*=(isLargeDevice?2:1);
      double wb=(MediaQuery.of(context).size.width/maxCl-pd*2)-pd*2;
      List<Widget> arProds = [];
      if(Constants.styles['categoryTypeList']=='grid') {
        foods.forEach((food) {
          food.prices.forEach((price) {
            price['inBasket']=userProv.basketCodeQty(price['code']);
          });
          arProds.add(
              Container(
                width: wb,
                margin: EdgeInsets.only(top: pd * 2, bottom: pd * 2),
                child: GridProduct(
                    product: food, addAction: userProv.basketAdd),
              )
          );
        });
        arRows.add(Container(
          padding:EdgeInsets.only(left:pd,right:pd),
          child:Wrap(
            alignment: WrapAlignment.spaceAround,
            children:arProds,
          ),
        ));
      }else if(Constants.styles['categoryTypeList']=='grid2') {
        int ln=0;
        int ifood=0;
        wb=(MediaQuery.of(context).size.width/maxCl-pd*2)-pd*1.2;
        List<List<Widget>> arProds = List.generate((foods.length/maxCl).ceil(), (index) => []);
        foods.forEach((food) {
          food.prices.forEach((price) {
            price['inBasket']=userProv.basketCodeQty(price['code']);
          });
          arProds[ln].add(
              Container(
                width: wb,
                margin:EdgeInsets.only(top:pd*1.5,bottom:pd*1.5),
                child:Grid2Product(product: food, addAction: userProv.basketAdd),
              )
          );
          ifood++;
          if(ifood==maxCl){
            ln++;
            ifood=0;
          }
        });
        arProds.forEach((arP) {
          arRows.add(IntrinsicHeight(
            child:Container(
              padding:EdgeInsets.only(left:pd*2,right:pd*2),
              child:Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children:arP
              ),
            ),
          ));
        });
      }else{
        foods.forEach((food) {
          food.prices.forEach((price) {
            price['inBasket']=userProv.basketCodeQty(price['code']);
          });
          arProds.add(
            LineProduct(product: food, addAction: userProv.basketAdd),
          );
        });
        arRows.add(Container(
          padding:EdgeInsets.only(left:pd,right:pd),
          child:Wrap(
            alignment: WrapAlignment.spaceAround,
            children:arProds,
          ),
        ));
      }

    }

    return Scaffold(
      appBar: AppBarSimple(
        title:category.name,
        act: BasketIcon(),
      ),
      body: ListView(
        //padding: EdgeInsets.only(left: 10,right:10),
        children: arRows,
      ),
    );
  }
}
