import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/util/dadata.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/html_block.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/data_provider.dart';

class DeliveryMap extends StatefulWidget {

  @override
  _DeliveryMapState createState() => _DeliveryMapState();
}

class _DeliveryMapState extends State<DeliveryMap> {
  late InAppWebViewController _webViewController;
  late TextEditingController controllerDadata;
  late FocusNode focusNodeDadata;

  void _setSearchPoint({required double lat,required double lon, required String title}){
    _webViewController.evaluateJavascript(source:'setPointFlutter('+lat.toString()+','+lon.toString()+',"'+title+'");');
  }

  @override
  void initState() {
    super.initState();
    controllerDadata=TextEditingController();
    focusNodeDadata=FocusNode();
    FbAnalitics.pageSet('DeliveryMap');
  }
  @override
  void dispose() {
    controllerDadata.dispose();
    focusNodeDadata.dispose();
    super.dispose();

  }

  void _selectDadata(Map data){
    String house=data['data']['house']!=null?data['data']['house'].toString():'';
    controllerDadata.text=data['value'];
    if(house.length<=0){
      controllerDadata.text=data['value']+' ';
      Timer(Duration(milliseconds: 100), (){
        focusNodeDadata.requestFocus();
      });
    }else{
      _setSearchPoint(
        lat:double.parse(data['data']['geo_lon'].toString()),
        lon:double.parse(data['data']['geo_lat'].toString()),
        title:data['value']
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    DataProvider dataProv=Provider.of<DataProvider>(context, listen: false);

    return Scaffold(
      appBar: AppBarSimple(
        title:getMessage('delivery_map'),
      ),

      body: Column(
        children:[
          Container(
            padding:EdgeInsets.fromLTRB(10, 0, 10, 0),
            child:Column(
              children:[
                Container(
                  height: 80,
                  child:dadataTextField(
                      context: context,
                      controller: controllerDadata,
                      focusNode: focusNodeDadata,
                      field:{
                        'name':getMessage('about_check_your_address'),
                        'value':'',
                        'error':'',
                      },
                      selectItem: _selectDadata,
                      autofocus: false,
                      readonly:false
                  ),
                ),
              ],
            ),
          ),
          Expanded(
            child:Container(
              child:HtmlBlock(
                url:dataProv.deliveryOptions['deliveryMap'],
                onWebViewCreated:(InAppWebViewController controller) async{
                  _webViewController = controller;
                },
              ),
            ),
          ),
        ],
      ),
    );
  }
}
