import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/basket_icon.dart';
import 'package:foapp/widgets/disabled_button.dart';
import 'package:foapp/widgets/empty_widget.dart';
import 'package:foapp/widgets/image_cust.dart';
import 'package:foapp/widgets/lable_radio.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/plus_minus.dart';
import 'package:foapp/widgets/price_format.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/menu_category.dart';
import 'package:foapp/util/tools.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:foapp/util/fbanalitics.dart';

class ProductDetails extends StatefulWidget {
  @required final MenuProduct product;
  @required final addAction;
  @required final String curFoCode;

  ProductDetails({
    required this.product,
    required this.addAction,
    this.curFoCode='',
  })
      :super();
  @override
  _ProductDetailsState createState() => _ProductDetailsState();
}

class _ProductDetailsState extends State<ProductDetails> {
  Map parts={};
  Map ings={};
  late MenuProduct product;
  late UserProvider userProv;
  late DataProvider dataProv;
  String curFoCode='';
  double gorPg=10.0;
  bool isLargeDevice=false;

  bool startAdding=false;
  double animSize=getConstantStyle('detailAnimateSizePic');
  int animTime=getConstantStyle('detailAnimateSizeTime').toInt();

  @override
  void initState() {
    product=widget.product;
    curFoCode=widget.curFoCode;
    userProv=Provider.of<UserProvider>(context, listen: false);
    dataProv=Provider.of<DataProvider>(context, listen: false);
    product.prices.forEach((price) {
      price['code']=price['code'].toString();
      if(curFoCode.length<=0) curFoCode=price['code'];
      if(price['parts']!=null){
        parts[price['code']]={};
        price['parts'].forEach((k,e){
          parts[price['code']][k]=e[0];
        });
      }
      //price['ings']=['100000753','100007313'];
      if(price['ings']!=null) {
        ings[price['code']] = {};
        price['ings'].forEach((gr){
          ings[price['code']][gr['name']]={};
          if(gr['req']) ings[price['code']][gr['name']][gr['items'].first]=1;
        });
      }
    });
    int sid=product.sid.first;
    MenuCategory cat=dataProv.categories.where((el) => el.id.toString()==sid.toString()).first;
    FbAnalitics.pageItem(id:product.id.toString(),name:product.name,cat: cat.name);
    super.initState();
  }
  void resetCurPrice(priceCode) {
    setState(() {
      curFoCode = priceCode;
    });
  }


  @override
  Widget build(BuildContext context) {
    isLargeDevice=MediaQuery.of(context).size.width>600;
    Map curPrice=product.prices.where((e) => e['code']==curFoCode).first;
    bool hasIngs=false;
    product.energy!.forEach((k, v) {
      if(v.length>0) hasIngs=true;
    });

    return Scaffold(
      appBar:AppBarSimple(
        title: '',
        act: BasketIcon(),
      ),
      extendBodyBehindAppBar: true,
      body:Stack(
        children:[
          Column(
            children:[
              Expanded(
                child: ListView(
                  padding: EdgeInsets.zero,
                  primary: false,
                  children:[
                    Stack(
                      children: <Widget>[
                        Container(
                          constraints: BoxConstraints(
                              minHeight: MediaQuery.of(context).size.width*getConstantStyle('detailPicHeight'),
                          ),
                          child:Center(
                            child:ImageCust(pic:product.pic),
                          ),
                        ),
                        Positioned(
                          right:5,
                          bottom:10,
                          child: _getIcons(),
                        ),
                        Positioned(
                          left:gorPg,
                          bottom:(getConstantStyle('icoSizeProduct')-10)/2,
                          child: _getSizeSelector(),
                        ),
                      ],
                    ),
                    Container(
                      //margin:EdgeInsets.only(top: MediaQuery.of(context).size.width*getConstantStyle('detailPicHeight')),
                      padding: EdgeInsets.fromLTRB(gorPg,15,gorPg,0),
                      //padding: EdgeInsets.only(right:gorPg),
                      child:Row(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        mainAxisSize: MainAxisSize.min,
                        children:[
                          Expanded(
                            child:Text(
                              product.name,
                              style: TextStyle(
                                fontSize: product.name.length>36?24:26,
                                height: 1.1,
                                fontWeight: FontWeight.w600,
                              ),
                              softWrap: true,
                            ),
                          ),
                          widgetByCondition(
                              cond: hasIngs,
                              func: ()=>GestureDetector(
                                onTap: (){
                                  showBZhUPopup();
                                },
                                child:SvgPicture.asset(
                                  'assets/bzhu.svg',
                                  fit: BoxFit.cover,
                                  height: 28.0,
                                ),
                              )
                          ),
                        ],
                      ),
                    ),
                    Container(
                      padding: EdgeInsets.fromLTRB(gorPg,0,gorPg,0),
                      margin:EdgeInsets.only(top:5,bottom:10),
                      child:Text(
                        product.desc,
                        style: TextStyle(
                          fontSize: 15,
                          height: 1.2,
                        ),
                      ),
                    ),
                    Container(
                      //padding: EdgeInsets.only(right:gorPg),
                      margin:EdgeInsets.only(top:10),
                      child:getPriceBlock(curPrice),
                    ),
                  ],
                ),
              ),
              widgetByCondition(
                cond:product.btnType=='float',
                func:()=>oneFloatBtn(curPrice),
              ),
            ],
          ),
          AnimatedPositioned(
            width: startAdding?5.0:animSize,
            height: startAdding?5.0:animSize,
            bottom: startAdding?MediaQuery.of(context).size.height-70:-animSize,
            left:startAdding?MediaQuery.of(context).size.width-animSize/5:MediaQuery.of(context).size.width/2-animSize/2,
            duration: Duration(milliseconds: animTime),
            curve: Curves.easeInOut,
            child: AnimatedOpacity(
              opacity: startAdding?1:0,
              duration: Duration(milliseconds: (animTime/2).floor()),
              child:Opacity(
                opacity: startAdding?1:0,
                child:ImageCust(pic: product.picm),
              ),
            ),
          ),
        ],
      ),
    );
  }
  Widget getParts(Map price){
    if(price['ingType']=='pic') return getPartsPic(price);
    List<Widget> arBlocks=[];
    price['parts'].forEach((k,e){
      List<Widget> arLines=[
        Container(
          padding:EdgeInsets.only(left:3),
          margin:EdgeInsets.only(bottom: 10),
          child:Text(
            k,
            style:TextStyle(
              fontWeight: FontWeight.w600,
              fontSize: 18,
            ),
          ),
        )
      ];
      e.forEach((cd){
        MenuProduct? prod=dataProv.getMenuProdByFO(cd);
        if(prod==null) return;
        arLines.add(
            LableRadio(
              padding: const EdgeInsets.symmetric(vertical: 5.0),
              label: prod.name,
              groupValue: parts[price['code']][k],
              value: cd,
              onChanged: (vl){
                setState(() {
                  parts[price['code']][k] = vl;
                });
              },
            )
        );
      });
      arBlocks.add(
        Container(
          margin:EdgeInsets.only(bottom: 20),
          child:Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children:arLines
          ),
        ),
      );
    });
    return Container(
      padding:EdgeInsets.only(left:gorPg,right:gorPg),
      child:Column(
        children: arBlocks,
      ),
    );
  }
  Widget _getHotIco(MenuProduct prod){
    int hotI=0;
    prod.energy!.forEach((k,el) {
      if(k=='ostrota') hotI=int.parse(el.toString());
    });
    Widget hot=EmptyWidget();
    if(hotI>0){
      List<Widget> hts=[];
      hts.add(Container(
        child: SvgPicture.asset(
          "assets/fire.svg",
          fit: BoxFit.cover,
          width: 8.0,
          height: 12.0,
        ),
      ));
      if(hotI>1){
        hts.add(Container(
          margin:EdgeInsets.only(left:3),
          child: SvgPicture.asset(
            "assets/fire.svg",
            fit: BoxFit.cover,
            width: 10.0,
            height: 17.0,
          ),
        ));
      }
      hot=Container(
        child:Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            mainAxisAlignment: MainAxisAlignment.end,
            children:hts
        ),
      );
    }
    return hot;
  }
  Widget getPartsPic(Map price){

    List<Widget> arBlocks=[];
    price['parts'].forEach((k,e){
      List<Widget> arLines=[];
      arBlocks.add(
          Container(
            padding:EdgeInsets.only(left:gorPg,right:gorPg),
            margin:EdgeInsets.only(bottom: 10),
            child:Text(
              k,
              style:TextStyle(
                fontWeight: FontWeight.w600,
                fontSize: 20,
              ),
            ),
          )
      );
      e.forEach((cd){
        MenuProduct? prod=dataProv.getMenuProdByFO(cd);
        if(prod==null) return;
        bool isSel=parts[price['code']][k]==cd;
        Widget hotW=_getHotIco(prod);

        arLines.add(
            GestureDetector(
              onTap: (){
                setState(() {
                  parts[price['code']][k] = cd;
                });
              },
              child:AnimatedContainer(
                duration: Duration(milliseconds: 300),
                curve: Curves.fastOutSlowIn,
                decoration: BoxDecoration(
                  color: Colors.white,
                  border: Border.all(
                    color: isSel?getConstantColor('main'):Colors.white,
                  ),
                  borderRadius: BorderRadius.all(Radius.circular(getConstantStyle('radiusSizeIngs'))),
                  boxShadow: [
                    BoxShadow(
                      color: Colors.grey.withOpacity(0.3),
                      spreadRadius: 3,
                      blurRadius: 3,
                      //offset: Offset(0, 3), // changes position of shadow
                    ),
                  ],
                ),
                child:ClipRRect(
                  borderRadius: BorderRadius.all(Radius.circular(getConstantStyle('radiusSizeIngs'))),
                  child:Stack(
                    children: [
                      Column(
                        children: [
                          Container(
                            height: 80.0,
                            alignment: Alignment.center,
                            child:ImageCust(pic:prod.picm),
                          ),
                          Container(
                            padding:EdgeInsets.fromLTRB(5,10,5,0),
                            child:Column(
                              children:[
                                Container(
                                  alignment: Alignment.center,
                                  height: 50.0,
                                  child: Center(
                                    child:Text(
                                      prod.name,
                                      textAlign: TextAlign.center,
                                      maxLines: 3,
                                      style: TextStyle(
                                        height: 1.1,
                                        fontSize: getConstantStyle('fontSizeIngName'),
                                        color:Colors.black,
                                      ),
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          )
                        ],
                      ),
                      Positioned(
                        left:5,
                        top:5,
                        child:hotW,
                      ),
                      widgetByCondition(
                        cond:isSel,
                        func:()=>Positioned(
                          right:7,
                          top:7,
                          child: Container(
                              child:Icon(
                                FontAwesomeIcons.check,
                                color:getConstantColor('main'),
                                size: 14,
                              )
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            )
        );
      });
      int cntInLine=isLargeDevice?6:3;
      double wbl=MediaQuery.of(context).size.width/cntInLine;

      arBlocks.add(
          Container(
            margin:EdgeInsets.only(bottom:15),
            child:GridView.count(
              primary: false,
              shrinkWrap: true,
              padding:EdgeInsets.all(10),
              crossAxisCount: cntInLine,
              childAspectRatio: wbl/170,
              crossAxisSpacing: 10,
              mainAxisSpacing: 10,
              children: arLines,
            ),
          )
      );
    });
    return Column(
      children: arBlocks,
    );
  }
  Widget getIngs(Map price){
    if(price['ingType']=='pic') return getIngsPic(price);

    List<Widget> arLines=[
      /*Container(
        padding:EdgeInsets.only(left:gorPg+3,right:gorPg),
        margin:EdgeInsets.only(bottom: 10),
        child:Text(
          getMessage('choose_ings'),
          style:TextStyle(
            fontWeight: FontWeight.w600,
            fontSize: 18,
          ),
        ),
      )*/
    ];
    price['ings'].forEach((gr){
      void changeIngQty(String cd,int add){
        if(ings[price['code']][gr['name']][cd]==null) ings[price['code']][gr['name']][cd]=0;
        int oldQ=ings[price['code']][gr['name']][cd];
        if(gr['req']){
          ings[price['code']][gr['name']]={cd:oldQ};
        }
        ings[price['code']][gr['name']][cd]+=add;
        if(gr['req']) {
          if(ings[price['code']][gr['name']][cd]>1) ings[price['code']][gr['name']][cd]=1;
          if(ings[price['code']][gr['name']][cd]<=0) ings[price['code']][gr['name']][cd]=1;
        }
        if(ings[price['code']][gr['name']][cd]<0) ings[price['code']][gr['name']][cd]=0;
        setState(() {

        });
      }
      if(gr['name'].length>0){
        arLines.add(
            Container(
              padding:EdgeInsets.only(left:3),
              margin:EdgeInsets.only(bottom: 10),
              child:Text(
                gr['name'],
                style:TextStyle(
                  fontWeight: FontWeight.w600,
                  fontSize: 18,
                ),
              ),
            )
        );
      }
      String grVal='';
      ings[price['code']][gr['name']].forEach((cd,q){
        if(q>0) grVal=cd;
      });
      gr['items'].forEach((cd) {
        MenuProduct? prod = dataProv.getMenuProdByFO(cd);
        if(prod==null) return;
        int priceIng = 0;
        prod.prices.forEach((price) {
          if (price['code'] == cd) {
            priceIng = price['price'];
          }
        });
        if(gr['req']){
          arLines.add(
              LableRadio(
                padding: const EdgeInsets.symmetric(vertical: 5.0),
                label: prod.name,
                groupValue: grVal,
                value: cd,
                onChanged: (vl){
                  changeIngQty(cd, 1);
                },
              )
          );
        }else {
          arLines.add(
              Container(
                padding: const EdgeInsets.symmetric(vertical: 5.0),
                child: Row(
                  children: [
                    PlusMinus(
                      cnt: ings[price['code']][gr['name']][cd] != null
                          ? ings[price['code']][gr['name']][cd]
                          : 0,
                      funcMinus: () => changeIngQty(cd, -1),
                      funcPlus: () => changeIngQty(cd, 1),
                      size: 's',
                    ),
                    Expanded(
                      child: Container(
                        margin: EdgeInsets.only(left: 10.0),
                        child: RichText(
                          text: TextSpan(
                            style: TextStyle(
                              fontSize: 16,
                              color: Colors.black,
                            ),
                            children: [
                              TextSpan(
                                text: prod.name + ' - ',
                              ),
                              priceFormatTextSpan(
                                price:priceIng,
                                fontWeight: FontWeight.w700,
                              ),
                            ],
                          ),
                        ),
                      ),
                    ),
                  ]
                ),
              )
          );
        }
      });
    });
    return Container(
      padding:EdgeInsets.only(left:gorPg,right:gorPg),
      margin:EdgeInsets.only(bottom: 15),
      child:Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: arLines,
      ),
    );
  }
  Widget getIngsPic(Map price){
    /** сделано без +- только по 1 добавляется и убавляется **/
    List<Widget> arBlocks=[];
    price['ings'].forEach((gr){
      void changeIngQty(String cd,int add){
        if(ings[price['code']][gr['name']][cd]==null) ings[price['code']][gr['name']][cd]=0;
        int oldQ=ings[price['code']][gr['name']][cd];
        if(gr['req']){
          ings[price['code']][gr['name']]={cd:oldQ};
        }
        ings[price['code']][gr['name']][cd]+=add;
        if(gr['req']) {
          if(ings[price['code']][gr['name']][cd]<=0) ings[price['code']][gr['name']][cd]=1;
        }
        if(ings[price['code']][gr['name']][cd]<0) ings[price['code']][gr['name']][cd]=0;
        setState(() {

        });
      }
      List<Widget> arLines=[];
      if(gr['name'].length>0) {
        arBlocks.add(
            Container(
              padding: EdgeInsets.only(left: gorPg, right: gorPg),
              margin: EdgeInsets.only(bottom: 10),
              child: Text(
                gr['name'],
                style: TextStyle(
                  fontWeight: FontWeight.w600,
                  fontSize: 20,
                ),
              ),
            )
        );
      }
      gr['items'].forEach((cd){
        MenuProduct? prod=dataProv.getMenuProdByFO(cd);
        if(prod==null) return;
        int priceIng = 0;
        prod.prices.forEach((price) {
          if (price['code'] == cd) {
            priceIng = price['price'];
          }
        });
        bool isSel=ings[price['code']][gr['name']][cd]!=null && ings[price['code']][gr['name']][cd]>0;
        Widget hotW=_getHotIco(prod);
        arLines.add(
            GestureDetector(
              onTap: (){
                changeIngQty(cd,(isSel?-1:1));
              },
              child:AnimatedContainer(
                duration: Duration(milliseconds: 300),
                curve: Curves.fastOutSlowIn,
                decoration: BoxDecoration(
                  color: Colors.white,
                  border: Border.all(
                    color: isSel?getConstantColor('main'):Colors.white,
                  ),
                  borderRadius: BorderRadius.all(Radius.circular(getConstantStyle('radiusSizeIngs'))),
                  boxShadow: [
                    BoxShadow(
                      color: Colors.grey.withOpacity(0.3),
                      spreadRadius: isSel?1:3,
                      blurRadius: 3,
                      //offset: Offset(0, 3), // changes position of shadow
                    ),
                  ],
                ),
                child:ClipRRect(
                  borderRadius: BorderRadius.all(Radius.circular(getConstantStyle('radiusSizeIngs'))),
                  child:Stack(
                    children: [
                      Column(
                        children: [
                          Container(
                            height: 80.0,
                            alignment: Alignment.center,
                            child:ImageCust(pic:prod.picm),
                          ),
                          Container(
                            padding:EdgeInsets.fromLTRB(5,5,5,0),
                            child:Column(
                              children:[
                                Container(
                                  alignment: Alignment.center,
                                  height: 45.0,
                                  child: Center(
                                    child:Text(
                                      prod.name,
                                      textAlign: TextAlign.center,
                                      maxLines: 3,
                                      style: TextStyle(
                                        height: 1.1,
                                        fontSize: getConstantStyle('fontSizeIngName'),
                                        color:Colors.black,
                                      ),
                                    ),
                                  ),
                                ),
                                Container(
                                  margin:EdgeInsets.only(top:5),
                                  child:PriceFormatText(
                                    price: priceIng,
                                    fontSize: getConstantStyle('fontSizeIngName')+2,
                                    height: 1,
                                  )
                                ),
                                /*plusMinusContainer(
                              cnt: ings[price['code']][cd] != null
                                  ? ings[price['code']][cd]
                                  : 0,
                              funcMinus: () => changeIngQty(cd, -1),
                              funcPlus: () => changeIngQty(cd, 1),
                              size: 's',
                            ),*/
                              ],
                            ),
                          )
                        ],
                      ),
                      Positioned(
                        left:5,
                        top:5,
                        child:hotW,
                      ),
                      widgetByCondition(
                        cond:isSel,
                        func:()=>Positioned(
                          right:7,
                          top:7,
                          child: Container(
                              child:Icon(
                                FontAwesomeIcons.check,
                                color:getConstantColor('main'),
                                size: 14,
                              )
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            )
        );
      });
      int cntInLine=isLargeDevice?6:3;
      double wbl=MediaQuery.of(context).size.width/cntInLine;

      arBlocks.add(
          Container(
            margin:EdgeInsets.only(bottom:15),
            child:GridView.count(
              primary: false,
              shrinkWrap: true,
              padding:EdgeInsets.all(10),
              crossAxisCount: cntInLine,
              childAspectRatio: wbl/180,
              crossAxisSpacing: 10,
              mainAxisSpacing: 10,
              children: arLines,
            ),
          )
      );
    });
    return Column(
      children: arBlocks,
    );
  }
  num getAmount(Map price){
    num amount=int.parse(price['price'].toString());
    if(ings.containsKey(price['code'])) {
      ings[price['code']].forEach((gn, gq) {
        gq.forEach((cd, q) {
          if (q != null && q > 0) {
            List prices = dataProv.getMenuProdByFO(cd)!.prices;
            prices.forEach((price) {
              if (price['code'] == cd) {
                amount += q * price['price'];
              }
            });
          }
        });
      });
    }
    return amount;
  }
  Widget getPriceBlock(Map price){
    num amount=getAmount(price);
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children:[
        widgetByCondition(
          cond:price['parts']!=null,
          func:()=>getParts(price),
        ),
        widgetByCondition(
          cond:price['ings']!=null,
          func:()=>getIngs(price),
        ),
        widgetByCondition(
            cond:product.btnType!='float',
            func:()=>Row(
              mainAxisAlignment:MainAxisAlignment.spaceBetween,
              crossAxisAlignment: CrossAxisAlignment.end,
              //textDirection: (price['parts']!=null || price['ings']!=null)?TextDirection.rtl:TextDirection.ltr,
              children: [
                Container(
                  padding: EdgeInsets.fromLTRB(gorPg,0,gorPg,0),
                  child: Column(
                    //crossAxisAlignment: (price['parts']!=null || price['ings']!=null)?CrossAxisAlignment.end:CrossAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children:[
                      widgetByCondition(
                        cond:(price['parts']==null && price['ings']==null),
                        func:()=>Container(
                          margin:EdgeInsets.only(bottom: 5),
                          child:Text(
                            (price['weight'].length>0?(getMessage('weight')+': '+price['weight']+"\n"):''),
                            style: TextStyle(
                              fontSize: 16,
                              height: 0.5,
                            ),
                          ),
                        ),
                      ),
                      Container(
                        child:PriceFormatText(
                          price:amount,
                          fontSize: 26,
                          height: 1,
                          fontWeight: FontWeight.w700,
                        ),
                      ),
                    ],
                  ),
                ),
                Container(
                    padding:EdgeInsets.only(left:gorPg,right:gorPg),
                    height: 38,
                    child:widgetByCondition(
                      cond:price['stop'],
                      func:()=>DisabledButton(title:getMessage('product_in_stop')),
                      funcFls:()=>widgetByCondition(
                        cond:(price['parts']==null && price['ings']==null && price['inBasket']!=null && price['inBasket']>0),
                        func:()=>PlusMinus(
                          cnt:price['inBasket'],
                          funcMinus: ()=>widget.addAction(price['code'].toString(),-1),
                          funcPlus: ()=>widget.addAction(price['code'].toString(),1),
                        ),
                        funcFls:()=>Container(
                          margin:EdgeInsets.only(right:0),
                          child:MainButton(
                              title: getMessage('in_basket'),
                              onClick:(){
                                addToCard(price);
                              }
                          ),
                        )
                      )
                    )
                ),
              ],
            )
        ),
      ],
    );
  }
  void addToCard(Map price){
    Map? prs;
    if(parts[price['code']]!=null) prs=Map.from(parts[price['code']]);
    Map? igs;
    if(ings[price['code']]!=null){
      ings[price['code']].forEach((gn,gq) {
        gq.forEach((c, q) {
          if (q > 0) {
            if (igs == null) igs = {};
            if(igs![c.toString()]==null) igs![c.toString()]=0;
            igs![c.toString()] += q;
          }
        });
      });
    }
    int bid=widget.addAction(price['code'].toString(),1,{'parts':prs,'ings':igs,'page':'detail'});
    if(bid>0) {
      setState(() {
        startAdding = true;
      });
      Timer(Duration(milliseconds: animTime), () {
        setState(() {
          startAdding = false;
          if(dataProv.globalSettings['closeDetailAfterAdd']=='y'){
            Navigator.pop(context);
          }
        });
      });
    }
  }
  Widget oneFloatBtn(Map curPrice){
    return Container(
      height: 60.0+(MediaQuery.of(context).padding.bottom>0?(MediaQuery.of(context).padding.bottom-10.0):0.0),
      padding:EdgeInsets.only(top:gorPg,left:gorPg,right:gorPg,bottom:(MediaQuery.of(context).padding.bottom>0?MediaQuery.of(context).padding.bottom:10.0)),
      child: widgetByCondition(
        cond:curPrice['stop'],
        func:()=>DisabledButton(
          title:getMessage('product_in_stop'),
          minWidth: double.infinity,
          fontSize:getConstantStyle('fontSizeDetailBtnDisabled'),
        ),
        funcFls:()=>MainButton(
          minWidth: double.infinity,
          radius: getConstantStyle('radiusSizeBtnBig'),
          contIn:RichText(
            text:TextSpan(
              children:[
                TextSpan(text:getMessage('in_basket').toUpperCase()+' '),
                priceFormatTextSpan(
                  price:getAmount(curPrice),
                  color:getConstantColor('btnText'),
                  fontSize:getConstantStyle('fontSizeBtn'),
                  height: 1.2
                ),
              ],
              style: TextStyle(
                  color: getConstantColor('btnText'),
                  fontWeight: FontWeight.w600,
                  fontSize: getConstantStyle('fontSizeDetailFloatBtn')+2
              ),
            ),
          ),
          title: getMessage('in_basket')+' '+getAmount(curPrice).toString(),
          onClick: () async{
            addToCard(curPrice);
          },
        ),
      ),
    );
  }

  Widget _getIcons(){
    Widget hot=EmptyWidget();
    if(product.icons!=null && product.icons!.length>0){
      List<Widget> hts=[];
      product.icons!.forEach((el) {
        el=el.toString();
        hts.add(Container(
          margin:EdgeInsets.only(right:7.0),
          child: ImageCust(pic:el,width:getConstantStyle('icoSizeProduct'),height:getConstantStyle('icoSizeProduct')),
        ));
      });
      hot=Container(
        child:Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            mainAxisAlignment: MainAxisAlignment.end,
            children:hts
        ),
      );
    }
    return hot;
  }
  Widget _getSizeSelector(){
    List<Widget> szl=[];
    product.prices.forEach((price){
      if(price['weight'].length>0){
        szl.add(GestureDetector(
          child:Container(
            child:Text(
              price['weight'],
              style:TextStyle(
                fontSize: getConstantStyle('fontSizeCatSizeText'),
                color: price['code']==curFoCode?getConstantColor('btnText'):getConstantColor('text'),
              ),
            ),
            padding: EdgeInsets.fromLTRB(10.0, 3, 10.0, 3),
            margin: EdgeInsets.only(right:15.0),
            decoration: BoxDecoration(
              color: price['code']==curFoCode?getConstantColor('main'):Colors.transparent,
              border: Border.all(width: 1, color: getConstantColor('main')),
              borderRadius: BorderRadius.all(
                Radius.circular(getConstantStyle('radiusSizeBtn')*0.75),
              ),
            ),
          ),
          onTap: (){
            resetCurPrice(price['code']);
          },
        ));
      }
    });
    return Container(
      margin:EdgeInsets.only(top:10.0),
      child:Row(
        children: szl,
      ),
    );
  }
  void showBZhUPopup(){
    double pdW=getConstantStyle('paddingSizeBJU');
    Map curPrice=product.prices.where((e) => e['code']==curFoCode).first;
    Widget detBZhU=Column(
      mainAxisSize: MainAxisSize.min,
      children:[
        Container(
          padding:EdgeInsets.fromLTRB(pdW,pdW,pdW,pdW-3),
          child:Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children:[
              Container(
                margin:EdgeInsets.only(bottom:10.0),
                child:Text(
                  product.name,
                  style: TextStyle(
                    fontSize: 23.0,
                    height: 1.1
                  ),
                ),
              ),
              Container(
                child:Text(
                  getMessage('porc_energy'),
                  style:TextStyle(
                      color:getConstantColor('main'),
                      fontSize:16.0
                  )
                )
              ),
              Container(
                margin:EdgeInsets.only(top:20.0),
                child:Column(
                  children:dataProv.globalSettings['detailInfoFields'].map<Widget>((cd){
                    if(!curPrice['energy'].containsKey(cd) || curPrice['energy'][cd].length==0) return EmptyWidget();
                    return Container(
                      margin:EdgeInsets.only(bottom:10.0),
                      child:Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children:[
                          Text(
                            getMessage(cd),
                            style:TextStyle(
                              fontSize: 18.0,
                            ),
                          ),
                          Text(
                            curPrice['energy'][cd],
                            style:TextStyle(
                              fontSize: 18.0,
                            ),
                          ),
                        ],
                      )
                    );
                  }).toList(),
                )
              ),
              Container(
                margin:EdgeInsets.only(top:10.0),
                child:MainButton(
                  title:getMessage('ok'),
                  height: 35.0,
                  minWidth: double.infinity,
                  onClick: (){
                    Navigator.pop(context);
                  }
                ),
              ),
            ],
          ),
        ),
      ],
    );
    Dialog popup = Dialog(
      child: detBZhU,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.all(Radius.circular(getConstantStyle('radiusSizeBtn')*1.2)),
      ),
    );
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return popup;
      },
    );
  }
}
