import 'dart:io' show Platform;
import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/local_auth.dart';
import 'package:foapp/util/phone.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/show_message.dart';
import 'package:sms_autofill/sms_autofill.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:mask_text_input_formatter/mask_text_input_formatter.dart';



class LoginScreen extends StatefulWidget {
  bool closeBtn=false;
  Function? closeAction;

  LoginScreen({
    this.closeBtn=false,
    this.closeAction,
  })
      :super();

  @override
  _LoginScreenState createState() => _LoginScreenState();
}

class _LoginScreenState extends State<LoginScreen> {


  final _scaffoldKey = GlobalKey<ScaffoldState>();

  final TextEditingController _phoneNumberController = TextEditingController();
  String _verificationCode='';
  final SmsAutoFill _autoFill = SmsAutoFill();

  String errorMess='';
  bool validPage=false;
  bool isLoading=false;
  void switchLoading([bool on=true]){
    setState(() {
      isLoading=on;
    });
  }
  /** methods and callbacks**/

  void verifyPhoneNumber() async {
    String phone=getPhoneClear(_phoneNumberController.text);
    errorMess='';
    if(phone.length==10) {
      switchLoading();
      LocalAuthPhone.sendCode(
          phoneNum: phone,
          funcOk: () {
            setState(() {
              isLoading = false;
              validPage = true;
            });
          },
          funcError: (String error) {
            switchLoading(false);
            showMessage(
              type: 'error',
              message: getMessage('has_error') + ": $error",
              context:context,
            );
          }
      );
    }else{
      setState(() {
        errorMess=getMessage('wrong_phone');
      });
    }
  }

  void signInWithPhoneNumber() async {
    switchLoading();
    LocalAuthPhone.checkCode(
        phoneNum: getPhoneClear(_phoneNumberController.text),
        code: _verificationCode,
        funcOk: (){
          Provider.of<UserProvider>(context, listen: false).userLogin();
        },
        funcError: (String error){
          switchLoading(false);
          showMessage(
            type:'error',
            message:getMessage('has_error')+": $error",
            context:context,
          );
        }
    );
  }

  /**-------------**/

  @override
  void initState() {
    super.initState();
    _phoneNumberController.text='';
  }
  @override
  void dispose() {
    _phoneNumberController.dispose();
    super.dispose();

  }
  void _closePage(){
    FocusScope.of(context).unfocus();
    if(widget.closeAction!=null) widget.closeAction!();
    else if(Navigator.canPop(context)) Navigator.pop(context);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBarSimple(
        title:getMessage('authorize'),
        noBack:!validPage,
        noBackEvent:true,
        backFrom:(){
          FocusScope.of(context).unfocus();
          setState(() {
            validPage=false;
          });
        },
        act:(widget.closeBtn?
        IconButton(
          tooltip: getMessage('back'),
          icon: Icon(
            Icons.clear,
            color: Constants.colors['red'],
          ),
          onPressed: _closePage,
        )
            :null),
      ),
      key: _scaffoldKey,
      //resizeToAvoidBottomPadding: false,
      body: Padding(padding: const EdgeInsets.all(16),
          child: ListView(
            children: <Widget>[
              validPage?getPhoneConfirm():getPhoneInput(),
          ],
        ),
      ),
    );
  }

  Widget getPhoneInput(){
    var maskFormatter = new MaskTextInputFormatter(mask: '(###) ###-##-##', filter: { "#": RegExp(r'[0-9]') });
    UnderlineInputBorder borderDef=UnderlineInputBorder(
      borderSide: BorderSide(color: Constants.colors['inputBottomBorder'], width: 0.5),
    );
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        widgetByCondition(
          cond:errorMess.length>0,
          func:()=>Text(
            errorMess,
            style:TextStyle(
              color:Colors.red,
            )
          ),
        ),
        Stack(
          alignment: Alignment.centerRight,
          children: <Widget>[
            Positioned(
              left:0,
              top:10,
              child: Text(
                getMessage('phone_code_country'),
                style: TextStyle(
                  fontSize: 22,
                ),
              ),
            ),
            TextField(
              autofocus: true,
              inputFormatters: [maskFormatter],
              controller: _phoneNumberController,
              keyboardType: TextInputType.phone,
              style: TextStyle(
                fontSize: 22.0,
              ),
              decoration: InputDecoration(
                enabledBorder: borderDef,
                focusedBorder: borderDef,
                border: borderDef,
                contentPadding: EdgeInsets.only(right:30.0,left:30.0),
                hintText: getMessage('your_phone'),
                hintStyle: TextStyle(
                  fontSize: 18.0,
                  color: Colors.grey,
                ),
              ),
              maxLines: 1,
            ),
            widgetByCondition(
              cond:Platform.isAndroid,
              func:()=>IconButton(
                onPressed: () async => {
                  _phoneNumberController.text = getPhoneFormat(await _autoFill.hint)
                },
                icon: Icon(Icons.settings_phone_rounded),
              ),
            ),
          ],
        ),
        Container(
          padding: const EdgeInsets.symmetric(vertical: 16.0),
          alignment: Alignment.center,
          child: widgetByCondition(
            cond:isLoading,
            func:()=>SpinKitThreeBounce(color: getConstantColor('main')),
            funcFls:()=>MainButton(
              title:getMessage('send_code'),
              onClick: verifyPhoneNumber,
              minWidth: 180,
            ),
          ),
        ),

      ],
    );
  }
  Widget getPhoneConfirm(){
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Padding(
          padding: EdgeInsets.only(bottom: 15),
          child: Text(
            getMessage('now_get_code'),
          ),
        ),
        PinFieldAutoFill(
          decoration: UnderlineDecoration(
            //textStyle: TextStyle(fontSize: 20, color: Colors.black),
            colorBuilder: FixedColorBuilder(Constants.colors['inputBottomBorder']),

          ),
          autoFocus: true,
          currentCode: '',
          onCodeSubmitted: (code) {},
          onCodeChanged: (String? code) {
            if (code!=null && code.length == 6) {
              _verificationCode=code;
              signInWithPhoneNumber();
            }
          },
        ),
        Container(
          padding: const EdgeInsets.only(top: 16.0),
          alignment: Alignment.center,
          child: widgetByCondition(
            cond:isLoading,
            func:()=>SpinKitThreeBounce(color: getConstantColor('main')),
            funcFls:()=>MainButton(
              title:getMessage('send'),
              onClick: signInWithPhoneNumber,
              minWidth: 180,
            ),
          ),
        ),
      ],
    );
  }
}

