import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/icons.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/util/loadingmess.dart';
import 'package:foapp/util/local_auth.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/screens/cart.dart';
import 'package:foapp/screens/home.dart';
import 'package:foapp/screens/about.dart';
import 'package:foapp/screens/personal.dart';
import 'package:foapp/screens/login.dart';
import 'package:foapp/widgets/bar_bottom.dart';



class MainScreen extends StatefulWidget {
  @override
  _MainScreenState createState() => _MainScreenState();
}

class _MainScreenState extends State<MainScreen> {
  PageController _pageController=PageController();
  int _page = 0;
  //final FirebaseAuth _auth = FirebaseAuth.instance;

  void _selectedTab(int index) {
    _pageController.jumpToPage(index);
  }

  @override
  Widget build(BuildContext context) {
    AppProvider appProv=Provider.of<AppProvider>(context, listen: false);
    UserProvider userProv=Provider.of<UserProvider>(context, listen: false);
    //DataProvider dataProv=Provider.of<DataProvider>(context, listen: false);

    return MediaQuery(
      data: MediaQuery.of(context).copyWith(
        textScaleFactor: 1,
      ),
      child:WillPopScope(
        onWillPop: ()=>Future.value(false),
        child: Scaffold(
          //key: _scaffoldKey,
          body: PageView(
            physics: NeverScrollableScrollPhysics(),
            controller: _pageController,
            onPageChanged: onPageChanged,
            children: <Widget>[
              Home(),
              AboutScreen(),
              (LocalAuthPhone.login()?Personal():LoginScreen(closeBtn: true, closeAction: ()=>appProv.changePage(0))),
              CartScreen(),
            ],
          ),
          bottomNavigationBar: RBottomAppBar(
            color: Theme.of(context).textTheme.caption!.color!,
            selectedColor: getConstantColor('main'),
            backgroundColor: Theme.of(context).primaryColor,
            notchedShape: CircularNotchedRectangle(),
            onTabSelected: _selectedTab,
            items: [
              RBottomAppBarItem(iconData: iconByName(Constants.styles['icoMenuCatalog']), text: getMessage('menu')),
              RBottomAppBarItem(iconData: iconByName(Constants.styles['icoMenuAbout']), text: getMessage('about_us')),
              RBottomAppBarItem(iconData: iconByName(Constants.styles['icoMenuProfile']), text: getMessage('profile')),
              RBottomAppBarItem(iconData: iconByName(Constants.styles['icoMenuCart']), text: getMessage('basket'),indec:(userProv.getBasketCnt()>0?userProv.getBasketCnt().toString():'')),
            ],
          ),
        ),
      ),
    );
  }


  void navigationTapped(int page) {
    _pageController.jumpToPage(page);
    onPageChanged(page);
  }

  @override
  void initState() {
    UserProvider userProv=Provider.of<UserProvider>(context, listen: false);
    userProv.mainContext=context;

    super.initState();
    Timer(Duration(milliseconds: 200), (){
      LoadingMess.checkMess(context);
    });

  }

  @override
  void dispose() {
    super.dispose();
    _pageController.dispose();
  }

  void onPageChanged(int page) {
    setState(() {
      this._page = page;
    });
  }
}
