import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/empty_widget.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/price_format.dart';
import 'package:foapp/widgets/show_message.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:expandable/expandable.dart';

import 'details.dart';

class Orders extends StatefulWidget {

  @override
  _OrdersState createState() => _OrdersState();
}

class _OrdersState extends State<Orders> {
  late UserProvider userProv;
  late DataProvider dataProv;
  List _ordersList=[];
  bool initOrderList=false;

  @override
  void initState() {
    userProv = Provider.of<UserProvider>(context, listen: false);
    dataProv = Provider.of<DataProvider>(context, listen: false);
    super.initState();
    FbAnalitics.pageSet('MyOrders');
  }
  Future<void> _checkOrderTimeout({bool st=false}) async{
    if(!initOrderList) return;
    _ordersList=await userProv.userOrders();
    //print(_ordersList);
    Timer(Duration(seconds: 60), (){
      _checkOrderTimeout(st:true);
    });
    if(st && initOrderList){
      setState(() {});
    }
  }
  Future<bool> _initOrdersList() async{
    if(initOrderList) return true;
    initOrderList=true;
    await _checkOrderTimeout();
    return true;
  }

  @override
  void dispose() {
    initOrderList=false;
    super.dispose();

  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBarSimple(
        title: getMessage('my_orders'),
      ),
      body:FutureBuilder<bool>(
          future: _initOrdersList(),
          builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
            if (snapshot.hasData) {
              return getOrdersList();
            } else {
              return Center(
                child:CircularProgressIndicator(),
              );
            }
          }
      ),
    );
  }
  Widget getOrdersList(){
    if(_ordersList.length==0){
      return Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children:[
          Center(
            child: Text(
              getMessage('you_dont_have_orders'),
              style:TextStyle(
                fontSize:18,
              ),
            ),
          ),
        ],
      );
    }
    return ListView.builder(
      itemCount: _ordersList.length,
      itemBuilder: (BuildContext context, int index) {
        Map order = _ordersList[index];
        return orderItem(order);
      },
    );
  }
  Widget orderItem(Map order){
    order['canpay']=int.parse(order['canpay'].toString());
    return Card(
      elevation: 4.0,
      child:ExpandablePanel(
        collapsed:EmptyWidget(),
        header: Container(
          padding:EdgeInsets.all(15.0),
          child:Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children:[
              Text(
                getMessage('order_line_head',order),
                style:TextStyle(
                  fontSize: 16,
                  fontWeight: FontWeight.w600,
                )
              ),
              Container(
                margin:EdgeInsets.only(top:5.0),
                child:Text(
                  order['status'],
                  style:TextStyle(
                    fontWeight: FontWeight.w500,
                  ),
                ),
              ),
            ],
          ),
        ),
        expanded: Container(
          padding:EdgeInsets.fromLTRB(15.0,0,15.0,15.0),
          child:Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children:[
              Container(
                margin:EdgeInsets.only(bottom: 10),
                child:Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children:[
                    PriceFormatText(
                      price:double.parse(order['amount'].toString()),
                      fontWeight: FontWeight.w600,
                    ),
                    InkWell(
                      child: Text(
                        getMessage('order_copy'),
                        style:TextStyle(
                          color:Constants.colors['red'],
                        )
                      ),
                      onTap: (){
                        initOrderList=false;
                        userProv.basketCopyOrder(order);
                        while(Navigator.canPop(context)) Navigator.pop(context);
                        Provider.of<AppProvider>(context, listen: false).changePage(3);
                      },
                    ),
                  ],
                ),
              ),
              Container(
                margin:EdgeInsets.only(top:5,bottom:5),
                child:Text(
                  getMessage('order_basket'),
                  style:TextStyle(
                    fontSize:17,
                    fontWeight: FontWeight.w600,
                  )
                )
              ),
              Column(
                children: order['products'].map<Widget>((prod){
                  MenuProduct? prodCatalog=dataProv.getMenuProdByFO(prod['code']);
                  if(prodCatalog==null) return EmptyWidget();
                  return InkWell(
                    child:Container(
                      decoration: BoxDecoration(
                        border: Border(bottom: BorderSide(width: 1, color: Theme.of(context).dividerColor)),
                      ),
                      padding:EdgeInsets.fromLTRB(0,5,0,5),
                      child:Row(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children:[
                          Expanded(
                            child:Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children:[
                                Text(prod['name']),
                                widgetByCondition(
                                  cond: prod['desc'].length>0,
                                  func: ()=>Container(
                                    margin:EdgeInsets.only(left:7,top:3),
                                    child:Text(
                                      prod['desc'],
                                      style:TextStyle(
                                        fontSize: 10,
                                      ),
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          ),
                          Container(
                            padding:EdgeInsets.only(left:15),
                            child:Text(
                              prod['qty'].toString().replaceAll('.00','')+' '+getMessage('units'),
                              style:TextStyle(
                                fontWeight: FontWeight.w600,
                              )
                            )
                          ),
                        ],
                      ),
                    ),
                    onTap: (){
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (BuildContext context){
                            return ProductDetails(product: prodCatalog, addAction: userProv.basketAdd);
                          },
                        ),
                      );
                    },
                  );
                }).toList(),
              )
            ],
          ),
        ),
      ),
    );
  }
}
