import 'package:flutter/material.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/phone.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/empty_widget.dart';
import 'package:foapp/widgets/page_button.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/tools.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/screens/addresses.dart';
import 'package:foapp/screens/profile.dart';

import 'orders.dart';


class Personal extends StatefulWidget {
  @override
  _PersonalState createState() => _PersonalState();
}

class _PersonalState extends State<Personal> with AutomaticKeepAliveClientMixin<Personal> {
  String _bonuses='0';
  String _discount='0';
  late OrderMotor order;
  late UserProvider userProv;
  late DataProvider dataProv;
  _checkBonuses() async{
    UserProvider userProv=Provider.of<UserProvider>(context, listen: false);
    await userProv.userSync();
    setState(() {
      _bonuses=userProv.user['bonuses'];
      _discount=userProv.user['discount']['personal'].toString();
    });
  }
  @override
  void initState() {
    super.initState();
    userProv=Provider.of<UserProvider>(context, listen: false);
    dataProv=Provider.of<DataProvider>(context, listen: false);
    order=userProv.order;
    order.init();
    _checkBonuses();
    FbAnalitics.pageSet('Personal');
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);

    return Scaffold(
      appBar:AppBarSimple(
        title: '+7 '+getPhoneFormat(userProv.user['phone']),//getMessage('your_profile'),
        noBack: true,
        act: IconButton(
          icon: Icon(
              Icons.logout,
              color:getConstantColor('appBarActColor'),
              size: 22,
          ),
          onPressed: (){
            Provider.of<UserProvider>(context, listen: false).userLogout();
            //setState(() {});
            //Navigator.pop(context);
            //Provider.of<AppProvider>(context, listen: false).changePage(2);
          },
        ),
      ),
      body: Padding(
        padding: EdgeInsets.fromLTRB(10.0,0,10.0,0),

        child: Column(
          children: <Widget>[
            Row(
              children: [
                Expanded(
                  flex: 1,
                  child:PageButton(
                    text:getMessage('profile_profile'),
                    icon: Icons.account_box_outlined,
                    iconSize: 40.0,
                    onClick: (){
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (BuildContext context){
                            return Profile();
                          }
                        )
                      );
                    },
                  ),
                ),
                Expanded(
                  flex: 1,
                  child:PageButton(
                    text:getMessage('profile_orders'),
                    icon: Icons.list,
                    iconSize: 40.0,
                    onClick: (){
                      Navigator.of(context).push(
                          MaterialPageRoute(
                              builder: (BuildContext context){
                                return Orders();
                              }
                          )
                      );
                    },
                  ),
                ),
              ],
            ),
            Row(
              mainAxisSize: MainAxisSize.max,
              children: [
                Expanded(
                  flex: 1,
                  child:PageButton(
                    text:getMessage('profile_addresses'),
                    icon: Icons.place_outlined,
                    iconSize: 40.0,
                    onClick: (){
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (BuildContext context){
                            return Addresses(
                              addresses: order.addresses,
                              noCloseAfterNewAdd: true,
                              selectAddress: (String newId) async{
                                await order.init();
                                return order.addresses;
                              },
                            );
                          },
                        ),
                      );
                    },
                  ),
                ),
                widgetByCondition(
                  cond: dataProv.globalSettings['profileNoBonusesDiscount']!='y',
                  func:()=>widgetByCondition(
                    cond: dataProv.globalSettings['profileBonuses']!='y',
                    func: ()=>Expanded(
                      flex: 1,
                      child:PageButton(
                        text:getMessage('profile_discount'),
                        tlt: _discount+'%',
                        iconSize:28.0,
                        onClick: (){
                          _checkBonuses();
                        },
                      ),
                    ),
                    funcFls: ()=>Expanded(
                      flex: 1,
                      child:PageButton(
                        text:getMessage('profile_bonuses',{'#bonuses#':_bonuses}),
                        icon: FontAwesomeIcons.coins,
                        iconSize:32.0,
                        onClick: (){
                          _checkBonuses();
                        },
                      ),
                    ),
                  ),
                  funcFls: ()=>Expanded(
                      flex: 1,
                      child:EmptyWidget(),
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }

  @override
  bool get wantKeepAlive => true;

}
