import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/main_dropdown.dart';
import 'package:foapp/widgets/main_input_field.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:intl/intl.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/util/const.dart';


class Profile extends StatefulWidget {

  @override
  _ProfileState createState() => _ProfileState();
}

class _ProfileState extends State<Profile> {
  late UserProvider userProv;
  Map _textControllers={};
  late FocusNode _birthdayFocusNode;
  List<DropdownMenuItem<String>> _genderList=[];
  late DateTime birthdayDate;

  @override
  void initState() {
    userProv = Provider.of<UserProvider>(context, listen: false);
    _textControllers['name']=TextEditingController();
    _textControllers['name'].text=userProv.user['name'];

    _textControllers['email']=TextEditingController();
    _textControllers['email'].text=userProv.user['email'];

    _textControllers['birthday']=TextEditingController();
    _textControllers['birthday'].text=userProv.user['birthday'];
    birthdayDate=DateTime(DateTime.now().year-10);
    if(userProv.user['birthday'].length>0){
      DateFormat dateFormat = DateFormat("dd.MM.yyyy");
      birthdayDate=dateFormat.parse(userProv.user['birthday']);
    }
    _birthdayFocusNode=FocusNode();
    _genderList=[];
    Map gnL={'':getMessage('of_gender_no'),'M':getMessage('of_gender_m'),'F':getMessage('of_gender_f')};
    gnL.forEach((k,lb){
      _genderList.add(
          DropdownMenuItem<String>(
            value: k,
            child: Container(
              width: double.infinity,
              child: Text(
                  lb,
                  style: TextStyle(
                    fontSize: 16,
                    height: 1
                  )
              ),
            ),
          )
      );
    });
    FbAnalitics.pageSet('Profile');
    super.initState();
  }
  @override
  void dispose() {
    _textControllers.forEach((k,e) {e.dispose();});
    _birthdayFocusNode.dispose();
    super.dispose();

  }

  _setField(String fName,String val){
    if(fName=='sms'){
      userProv.user[fName]=(val=='y');
    }else {
      userProv.user[fName] = val;
    }
    userProv.userSave();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {

    return Scaffold(
      appBar: AppBarSimple(
        title:getMessage('my_profile'),
        act:TextButton(
          onPressed: () {
            Navigator.pop(context);
          },
          child: Text(
            getMessage('save'),
            style: TextStyle(
              color:getConstantColor('red'),
            ),
          )
        ),
      ),
      body: Column(
        children:[
          Expanded(
            child:Padding(
              padding: EdgeInsets.fromLTRB(10.0,0,10.0,0),
              child: ListView(
                children: <Widget>[
                  Container(
                    margin:EdgeInsets.only(top:10.0),
                    child:MainInputField(
                        //keyboardType:keyBordType,
                        lines:1,
                        labelText:getMessage('of_fio'),
                        controller: _textControllers['name'],
                        //errorText:field['error'],
                        onChanged:(v){
                          _setField('name',v);
                        }
                    ),
                  ),
                  Container(
                    margin:EdgeInsets.only(top:20.0),
                    child:MainInputField(
                      keyboardType:TextInputType.emailAddress,
                        lines:1,
                        labelText:getMessage('of_email'),
                        controller: _textControllers['email'],
                        //errorText:field['error'],
                        onChanged:(v){
                          _setField('email',v);
                        }
                    ),
                  ),
                  Container(
                    margin:EdgeInsets.only(top:20.0),
                    child:Row(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children:[
                        Expanded(
                          flex:1,
                          child:Stack(
                            children:[
                              Container(
                                margin:EdgeInsets.only(right:10.0),
                                child:MainInputField(
                                  keyboardType:TextInputType.datetime,
                                  lines:1,
                                  labelText:getMessage('of_birthday'),
                                  controller: _textControllers['birthday'],
                                  focusNode: _birthdayFocusNode,
                                  //errorText:field['error'],
                                  onChanged:(v){
                                    _setField('birthday',v);
                                  },
                                  onTap:() async{
                                    _birthdayFocusNode.unfocus();
                                    DateTime? bd=await showDatePicker(
                                      context: context,
                                      locale : const Locale("ru","RU"),
                                      initialDatePickerMode: DatePickerMode.year,
                                      initialDate: birthdayDate,
                                      firstDate: DateTime(1900),
                                      lastDate: DateTime(DateTime.now().year-10),
                                    );
                                    if(bd!=null){
                                      birthdayDate=bd;
                                      _textControllers['birthday'].text=DateFormat('dd.MM.yyyy').format(birthdayDate);
                                      _setField('birthday',_textControllers['birthday'].text);
                                    }
                                  }
                                ),
                              ),
                              widgetByCondition(
                                cond: userProv.user['birthday']!=null && userProv.user['birthday'].length>0,
                                func: ()=>Positioned(
                                  right:3,
                                  bottom:0,
                                  child:InkWell(
                                    child: Container(
                                      padding:EdgeInsets.fromLTRB(10, 10, 10, 8),
                                      child:Icon(
                                        FontAwesomeIcons.timesCircle,
                                        size: 16,
                                        color: getConstantColor('text').withAlpha(200),
                                      ),
                                    ),
                                    onTap: (){
                                      _textControllers['birthday'].text='';
                                      _setField('birthday','');
                                    }
                                  ),
                                ),
                              ),
                            ],
                          ),
                        ),
                        Expanded(
                          flex:1,
                          child:Container(
                            margin:EdgeInsets.only(left:10.0),
                            child:Column(
                              children:[
                                Container(
                                  alignment: Alignment.bottomLeft,
                                  child:Text(
                                    getMessage('of_gender'),
                                    style:TextStyle(
                                      color:Colors.grey,
                                      fontSize: 12,
                                      height: 1
                                    ),
                                  ),
                                ),
                                Container(
                                  height: 29.0,
                                  padding: const EdgeInsets.only(top:4.0),
                                  child:MainDropDown(
                                    value: userProv.user['gender'],
                                    items: _genderList,
                                    padding:EdgeInsets.fromLTRB(0, 0, 0, 10),
                                    onChanged: (String? v){
                                      _setField('gender',v!);
                                    },
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                  Container(
                    margin:EdgeInsets.only(top:10.0),
                    decoration: BoxDecoration (
                      border: Border(
                        bottom: BorderSide(
                          color: Constants.colors['inputBottomBorder'],
                          width: 0.5,
                        ),
                      ),
                    ),
                    child:Row(
                      children:[
                        Expanded(
                          child:Text(
                            getMessage('of_get_sms_adv'),
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w500,
                            ),
                          ),
                        ),
                        Switch(
                          value: userProv.user['sms'],
                          onChanged: (v) async{
                            _setField('sms',v?'y':'n');
                          },
                          activeColor: getConstantColor('main'),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

}
