import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:foapp/util/phone.dart';
import 'package:foapp/widgets/app_bar_simple.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/main_dropdown.dart';
import 'package:foapp/widgets/main_input_field.dart';
import 'package:mask_text_input_formatter/mask_text_input_formatter.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/tools.dart';


class Reservation extends StatefulWidget {
  final String? defRest;

  Reservation({
    Key? key,
    this.defRest,
  })
      :super(key: key);
  @override
  _ReservationState createState() => _ReservationState();
}

class _ReservationState extends State<Reservation> {
  late DataProvider dataProv;
  late UserProvider userProv;
  Map formFields={
    'phone':{
      'req':true,
      'contrl':true,
      'val':'',
    },
    'name': {
      'contrl':true,
      'req':true,
      'val':'',
    },
    'persons': {
      'contrl':true,
      'req':true,
      'val':'2',
    },
    'rest':{
      'contrl':false,
      'req':true,
      'val':'',
    },
    'date':{
      'contrl':false,
      'req':true,
      'val':'',
    },
    'time':{
      'contrl':false,
      'req':true,
      'val':'',
    },
    'comment':{
      'contrl':true,
      'req':false,
      'val':'',
    },
  };
  late Map serverData;
  bool initServerData=false;
  bool hasSubmit=false;
  bool hasError=false;
  bool isLoading=false;
  Map _textControllers={};
  var maskFormatter = new MaskTextInputFormatter(mask: '+7 (###) ###-##-##', filter: { "#": RegExp(r'[0-9]') });

  List<DropdownMenuItem<String>> _restList=[];
  List<DropdownMenuItem<String>> _datesList=[];
  List<DropdownMenuItem<String>> _timesList=[];

  @override
  void initState() {
    userProv = Provider.of<UserProvider>(context, listen: false);
    dataProv = Provider.of<DataProvider>(context, listen: false);

    formFields['name']['val']=userProv.user['name'];
    formFields['phone']['val']='+7'+getPhoneFormat(userProv.user['phone']);
    formFields.forEach((k, el) {
      if(el['contrl']) {
        _textControllers[k] = TextEditingController();
        _textControllers[k].text = el['val'];
      }
    });
    if(widget.defRest!=null) formFields['rest']['val']=widget.defRest;

    FbAnalitics.pageSet('Reservation');
    _initServerData();
    super.initState();
  }
  @override
  void dispose() {
    _textControllers.forEach((k,e) {e.dispose();});
    super.dispose();

  }

  _setField(String fName,String val){
    if(fName=='phone' && val==''){
      val='+7';
      _textControllers['phone'].text=val;
      _textControllers['phone'].selection=TextSelection.collapsed(offset: val.length);
    }
    setState(() {
      formFields[fName]['val']=val;
    });
  }
  Future _sendReservation() async{
    hasSubmit=true;
    hasError=false;
    if(getPhoneClear(formFields['phone']['val']).length<10 || formFields['name']['val'].length<1) hasError=true;
    if(!hasError){
      await dataProv.putReservData(formFields);
    }else{
      isLoading=false;
    }
    setState(() {});
  }
  Future _initServerData() async{
    if(initServerData) return true;
    serverData=await dataProv.getReservData();
    formFields['rest']['val']=serverData['rest'].first['id'].toString();
    serverData['rest'].first['date'].forEach((k,e){
      if(formFields['date']['val'].length==0){
        formFields['date']['val']=k;
      }
    });
    serverData['rest'].first['time'][formFields['date']['val']].forEach((k,e){
      if(formFields['time']['val'].length==0){
        formFields['time']['val']=k;
      }
    });

    serverData['rest'].forEach((el){
      _restList.add(
        DropdownMenuItem<String>(
          value: el['id'].toString(),
          child: Container(
            width: double.infinity,
            child: Text(
              el['name'],
              style:TextStyle(
                fontSize: 16,
              )
            ),
          ),
        )
      );
    });


    initServerData=true;
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBarSimple(
        title: getMessage('make_reservation'),
      ),
      body: widgetByCondition(
        cond: !initServerData,
        func: () =>
            Center(
              child: CircularProgressIndicator(),
            ),
        funcFls: () {
          if(hasSubmit && !hasError) return _successReserv();
          else return _formReserv();

        },
      ),
    );
  }
  Widget _successReserv(){
    return Center(
      child: Container(
        padding: EdgeInsets.all(20),
        child: Text(
          getMessage('reservation_success'),
          textAlign: TextAlign.center,
          style:TextStyle(
            fontSize: 24.0,
            fontWeight: FontWeight.w500,
          )
        ),
      ),
    );
  }
  Widget _formReserv(){
    _timesList=[];
    _datesList=[];
    Map rest=serverData['rest'].where((e)=>e['id']==formFields['rest']['val']).first;
    if(rest['date'][formFields['date']['val']]==null){
      formFields['date']['val']='';
      rest['date'].forEach((k,e){
        if(formFields['date']['val'].length==0){
          formFields['date']['val']=k;
        }
      });
    }
    rest['date'].forEach((k,el){
      _datesList.add(
          DropdownMenuItem<String>(
            value: k,
            child: Container(
              width: double.infinity,
              child: Text(
                  el,
                  style:TextStyle(
                    fontSize: 16,
                  )
              ),
            ),
          )
      );
    });
    rest['time'][formFields['date']['val']].forEach((k,el){
      _timesList.add(
          DropdownMenuItem<String>(
            value: k,
            child: Container(
              width: double.infinity,
              child: Text(
                  el,
                  style:TextStyle(
                    fontSize: 16,
                  )
              ),
            ),
          )
      );
    });
    bool fndTm=false;
    rest['time'][formFields['date']['val']].forEach((k,e){
      if(k==formFields['time']['val']){
        fndTm=true;
      }
    });
    if(!fndTm){
      rest['time'][formFields['date']['val']].forEach((k,e){
        if(!fndTm){
          formFields['time']['val']=k;
          fndTm=true;
        }
      });
    }
    return Column(
      children:[
        Expanded(
          child:Padding(
            padding: EdgeInsets.fromLTRB(10.0,0,10.0,0),
            child: ListView(
              physics: BouncingScrollPhysics(),
              children: <Widget>[
                Container(
                  margin:EdgeInsets.only(top:10.0),
                  child:MainInputField(
                      keyboardType:TextInputType.phone,
                      inputFormatters: [maskFormatter],
                      lines:1,
                      labelText:getMessage('of_phone'),
                      controller: _textControllers['phone'],
                      errorText:hasSubmit && getPhoneClear(formFields['phone']['val']).length<10?getMessage('req_field'):null,
                      onChanged:(v){
                        _setField('phone',v);
                      }
                  ),
                ),
                Row(
                  children: [
                    Expanded(
                      flex:2,
                      child:Container(
                        margin: EdgeInsets.only(right:20.0),
                        child:Container(
                          child:MainInputField(
                            //keyboardType:keyBordType,
                            lines:1,
                            labelText:getMessage('of_fio'),
                            controller: _textControllers['name'],
                            errorText:hasSubmit && formFields['name']['val'].length<1?getMessage('req_field'):null,
                            onChanged:(v){
                              _setField('name',v);
                            }
                          ),
                        ),
                      ),
                    ),
                    Expanded(
                      flex:1,
                      child:Container(
                        child:Container(
                          child:MainInputField(
                            lines:1,
                            labelText:getMessage('persons'),
                            controller: _textControllers['persons'],
                            errorText:hasSubmit && formFields['persons']['val'].length<1?getMessage('req_field'):null,
                            onChanged:(v){
                              _setField('persons',v);
                            }
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
                Container(
                  margin:EdgeInsets.only(top:20.0),
                  child:Column(
                    children:[
                      Container(
                        alignment: Alignment.bottomLeft,
                        child:Text(
                          getMessage('restaurant'),
                          style:TextStyle(
                            color:Colors.grey,
                            fontSize: 12,
                          ),
                        ),
                      ),
                      MainDropDown(
                        value: formFields['rest']['val'],
                        items: _restList,
                        padding:EdgeInsets.fromLTRB(0, -2, 0, 10),
                        onChanged: (v){
                          _setField('rest',v!);
                        },
                      ),
                    ],
                  ),
                ),

                Container(
                  margin:EdgeInsets.only(top:20.0),
                  child:Stack(
                    clipBehavior: Clip.none,
                    children:[
                      Positioned(
                        top:-5,
                        child:Text(
                          getMessage('reservation_date_time'),
                          style: TextStyle(
                          ),
                        ),
                      ),
                      Row(
                        children: [
                          Expanded(
                            flex:2,
                            child:Container(
                              margin: EdgeInsets.only(right:20.0),
                              child:MainDropDown(
                                value: formFields['date']['val'],
                                items: _datesList,
                                onChanged: (String? newValue) {
                                  _setField('date', newValue!);
                                }
                              ),
                            ),
                          ),
                          Expanded(
                            flex:1,
                            child:MainDropDown(
                                value: formFields['time']['val'],
                                items: _timesList,
                                onChanged: (String? newValue) {
                                  _setField('time', newValue!);
                                }
                            ),
                          ),
                        ],
                      ),
                    ],
                  ),
                ),
                Container(
                  margin:EdgeInsets.only(top:10.0),
                  child:MainInputField(
                    //keyboardType:keyBordType,
                      lines:3,
                      labelText:getMessage('comment'),
                      controller: _textControllers['comment'],
                      //errorText:field['error'],
                      onChanged:(v){
                        _setField('comment',v);
                      }
                  ),
                ),
              ],
            ),
          ),
        ),
        Container(
          height: 60.0+(MediaQuery.of(context).padding.bottom>0?(MediaQuery.of(context).padding.bottom-10.0):0.0),
          padding:EdgeInsets.only(top:10.0,left:10.0,right:10.0,bottom:(MediaQuery.of(context).padding.bottom>0?MediaQuery.of(context).padding.bottom:10.0)),
          child: MainButton(
            minWidth: double.infinity,
            radius: getConstantStyle('radiusSizeBtnBig'),
            contIn:widgetByCondition(
              cond:isLoading,
              func:()=>SpinKitThreeBounce(color:getConstantColor('primary')),
              funcFls:()=>Text(
                getMessage('send'),
                style: TextStyle(
                    color:getConstantColor('btnText'),
                    fontWeight: FontWeight.w500,
                    fontSize: getConstantStyle('fontSizeBtn')
                ),
              ),
            ),
            onClick: () async{
              if(isLoading) return;
              isLoading=true;
              _sendReservation();
              setState(() {});
              /** TODO SEND **/
            },
          ),
        ),
      ],
    );
  }

}
