import 'dart:async';
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';

import 'const.dart';

class DadataSuggest{
  static Map config={'url':'','key':'','kladr_id':'','lang':'ru'};
  static Future<List> autocomplate(String q) async{
    List arRes=[];
    if(config['url'].length>0) {
      Map req= {
        'query': q,
        'count': 10,
        'language': config['lang'].length > 0 ? config['lang'] : 'ru',
      };
      if(config['kladr_id'].toString().length>0) {
        req['locations_boost'] = [
          {"kladr_id": config['kladr_id'].toString()}
        ];
      }
      if(config.containsKey('city') && config['city'].length>0 || config.containsKey('region') && config['region'].length>0 || config.containsKey('cities') && config['cities'].length>0 ) {
        req['locations']=[];
        if(config.containsKey('city') && config['city'].length>0) req['locations'].add({'city':config['city']});
        if(config.containsKey('region') && config['region'].length>0) req['locations'].add({'region':config['region']});
        if(config.containsKey('cities') && config['cities'].length>0){
          config['cities'].forEach((e){
            req['locations'].add({'city':e});
          });
        }
      }
      final http.Response response = await http.post(
        Uri.parse(config['url']),
        headers: <String, String>{
          'Content-Type': 'application/json; charset=UTF-8',
          'Accept': 'application/json',
          'Authorization': 'Token '+config['key'],
        },
        body: jsonEncode(req),
      ).timeout(const Duration(seconds: 5)).catchError((error) {
        print(error.toString());
      });

      if (response.statusCode == 200) {
        Map res=json.decode(response.body);
        try{
          if(res.containsKey('suggestions')){
            res['suggestions'].forEach((el){
              if(config.containsKey('removes') && config['removes'].length>0){
                String v=el['value'];
                config['removes'].forEach((e){
                  v=v.replaceAll(e, '');
                });
                el['value']=v;
              }
              arRes.add(el);
            });
          }
        } on Exception catch (_) {
          print('Dadata error');
        }
      }
    }

    return arRes;
  }
}


Widget dadataTextField({required BuildContext context,required Map field,bool autofocus=false,required Function selectItem, bool readonly=false,required TextEditingController controller,FocusNode? focusNode}){
  controller.text=field['value'];
  controller.selection=TextSelection.collapsed(offset: field['value'].length);

  UnderlineInputBorder borderDef=UnderlineInputBorder(
    borderSide: BorderSide(color: Constants.colors['inputBottomBorder'], width: 0.5),
  );
  return TypeAheadField(
    hideOnEmpty: true,
    hideOnError: true,
    hideOnLoading: true,
    suggestionsBoxVerticalOffset:-15,
    textFieldConfiguration: TextFieldConfiguration(
      enabled: !readonly,
      autofocus: (!autofocus?false:(field['value'].length<=0 || autofocus)),
      controller: controller,
      focusNode: focusNode,
      decoration: InputDecoration(
        enabledBorder: borderDef,
        focusedBorder: borderDef,
        border: borderDef,
        labelStyle: TextStyle(
          color: Colors.grey,
        ),
        labelText: field['name'],
        errorText: field['error'].length>0?field['error']:null,
        helperText: ' ',
        suffix: IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(Icons.clear),
          color: getConstantColor('red'),
          onPressed:(){
            controller.text='';
          }
        ),
      ),
    ),
    suggestionsCallback: (pattern) async {
      return await DadataSuggest.autocomplate(pattern);
    },
    itemBuilder: (context, dynamic suggestion) {
      return ListTile(
        visualDensity: VisualDensity(horizontal: 0, vertical: -3),
        title: Text(suggestion['value']),
      );
    },
    onSuggestionSelected: (suggestion) {
      selectItem(suggestion);
    },
    suggestionsBoxDecoration: SuggestionsBoxDecoration(
      color: getConstantColor('dropDownBg'),
    ),
  );

}