import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:crypto/crypto.dart';
import 'package:http/http.dart' as http;
import 'package:foapp/util/platform_info.dart';

Future<bool> haveInternet() async{
  bool res=false;
  try {
    final http.Response response = await http.get(Uri.parse(FoSend.appUrl)).timeout(const Duration(seconds: 5));
    if (response.statusCode == 200) res=true;
  } on SocketException catch (_) {
  }
  return res;
}
class FoSend{
  static String appUrl='https://aventa.tmweb.ru/foapp/app.php';
  String appUrlTime='';
  String privatKey='appfobitrix_app';
  String errorFo='';
  String getPublicKey(){
    return generateMd5(DateTime.now().millisecondsSinceEpoch.toString());
  }
  String getHash(String func,String pubKey){
    var key=utf8.encode(privatKey);
    var hmacMd5 = Hmac(md5, key);
    return hmacMd5.convert(utf8.encode(func+pubKey)).toString();
  }
  String getAppUrl(){
    return appUrlTime.length>0?appUrlTime:appUrl;
  }
  String getUrlAction(String func,[List data=const []]){
    String pubKey=getPublicKey();
    data.add('action='+func);
    data.add('pubkey='+pubKey);
    data.add('hash='+getHash(func,pubKey));
    data.add('platform='+Platform.operatingSystem);
    return getAppUrl()+'?'+data.join('&');
  }
  send(func,data) async{
    errorFo='';
    Map devInfo=await getPlatformInfo();
    String pubKey=getPublicKey();
    data['action']=func;
    data['pubkey']=pubKey;
    data['hash']=getHash(func,pubKey);
    data['platform']=devInfo['platform'];
    data['version']=devInfo['version'];
    data['device']=devInfo['device'];
    data['platformVer']=devInfo['platformVer'];
    Map res={};
    try{
      final http.Response response = await http.post(
        Uri.parse(getAppUrl()),
        headers: <String, String>{
          'Content-Type': 'application/json; charset=UTF-8',
        },
        body: jsonEncode(data),
      ).timeout(const Duration(seconds: 20));
      //print(func);
      //print(response);
      bool isJson=false;
      if(response.headers.containsKey('content-type') && response.headers['content-type']=='application/json'){
        isJson=true;
      }
      if (response.statusCode == 200 && response.body!='[]' && isJson) {
        var dt=json.decode(response.body);
        if(dt==null){
          errorFo=response.body;
        }else{
          if(dt!=null && dt.containsKey('newurl') && dt['newurl'].length>0){
            setNewUrl(dt['newurl']);
            res=send(func,dt);
          }else {
            res = dt;
          }
        }
      } else {
        if(response.statusCode!=200 || !isJson){
          errorFo=response.body;
        }
      }
    } catch (e) {
      //print(e);
      errorFo=e.toString();
      if(!(await haveInternet())){

      }
    }
    if(errorFo.isNotEmpty){
      print(errorFo);
    }
    return res;
  }
  setNewUrl(String newUrl){
    int l=privatKey.length;
    newUrl=newUrl.substring(l);
    Codec<String, String> stringToBase64 = utf8.fuse(base64);
    newUrl=stringToBase64.decode(newUrl);
    //print(newUrl);
    appUrlTime=newUrl;
    Timer(Duration(seconds: 600), (){
      appUrlTime='';
    });
  }
  String generateMd5(String input) {
    return md5.convert(utf8.encode(input)).toString();
  }

}