import 'dart:io';
import 'package:device_info/device_info.dart';
import 'package:flutter/services.dart';

import 'fosend.dart';

class LocalAuthPhone{
  static bool enabled=false;
  static String phone='';
  static String? uniqDeviceId;
  static late String platform;

  static Future init() async{
    await getUniqCode();
    await initPhone();
  }

  static bool login(){
    return phone.length>0;
  }

  static void sendCode({required String phoneNum,required Function funcOk, required Function funcError}) async{
    var fo=FoSend();
    Map data = await fo.send('authcode', {'phone':phoneNum,'deviceId':uniqDeviceId,'platform':platform});
    if(data['error'].length>0){
      funcError(data['error']);
    }else{
      funcOk();
    }
  }
  static void checkCode({required String phoneNum,required String code,required Function funcOk, required Function funcError}) async{
    var fo=FoSend();
    Map data = await fo.send('authcheck', {'phone':phoneNum,'code':code,'deviceId':uniqDeviceId,'platform':platform});
    //print(data);
    if(data['error'].length>0){
      funcError(data['error']);
    }else{
      phone=phoneNum;
      funcOk();
    }
  }
  static void logout() async{
    if(enabled){
      var fo=FoSend();
      fo.send('authlogout', {'phone':phone,'deviceId':uniqDeviceId});
    }else {
      //final FirebaseAuth _auth = FirebaseAuth.instance;
      //_auth.signOut();
    }
    phone='';
  }
  static Future initPhone() async{
    if(enabled){
      var fo=FoSend();
      Map data = await fo.send('authget', {'deviceId':uniqDeviceId,'platform':platform});
      if(data['phone'].length>0){
        phone=data['phone'];
      }else{
        /*final FirebaseAuth _auth = FirebaseAuth.instance;
        if(_auth.currentUser!=null) phone = _auth.currentUser.phoneNumber;*/
      }
    }else {
      //final FirebaseAuth _auth = FirebaseAuth.instance;
      //if(_auth.currentUser!=null) phone = _auth.currentUser.phoneNumber;
    }
  }

  static Future<String?> getUniqCode() async{
    if(uniqDeviceId==null) {
      final DeviceInfoPlugin deviceInfoPlugin = new DeviceInfoPlugin();
      try {
        if (Platform.isAndroid) {
          var build = await deviceInfoPlugin.androidInfo;
          uniqDeviceId = build.androidId; //UUID for Android
          platform='android';
        } else if (Platform.isIOS) {
          var data = await deviceInfoPlugin.iosInfo;
          uniqDeviceId = data.identifierForVendor; //UUID for iOS
          platform='ios';
        }
      } on PlatformException {
        print('Failed to get platform version');
      }
    }
    return uniqDeviceId;
  }
}