import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/screens/category.dart';
import 'package:foapp/screens/details.dart';
import 'package:foapp/util/menu_category.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/image_cust.dart';
import 'package:foapp/widgets/main_button.dart';
import 'package:foapp/widgets/show_message.dart';
import 'package:foapp/widgets/webview_page.dart';
import 'package:provider/provider.dart';

import 'menu_product.dart';




void popupShow({required BuildContext context,required Map dataPopup}) async{
  void closePopup() {
    Navigator.pop(context);
  }

  List<Widget> arContext=[];
  if(dataPopup.containsKey('pic') && dataPopup['pic'].length>0){
    arContext.add(
        Container(
          margin:EdgeInsets.only(top:10,bottom: 10),
          child: ClipRRect(
            borderRadius: BorderRadius.circular(8.0),
            child:ImageCust(pic:dataPopup['pic']),
          ),
        )
    );
  }
  arContext.add(
      Container(
        margin:EdgeInsets.only(bottom:3),
        child:Align(
          alignment: Alignment.centerLeft,
          child:Text(
            dataPopup['name'],
            style:TextStyle(
              fontSize: 18,
            ),
          ),
        ),
      )
  );
  arContext.add(
      Expanded(
        child: ListView(
          physics: BouncingScrollPhysics(),
          children:[
            Align(
              alignment: Alignment.centerLeft,
              child:Text(
                dataPopup['desc'],
                style:TextStyle(
                  fontSize: 14,
                ),
              ),
            ),
          ],
        ),
      )
  );
  if(!dataPopup.containsKey('btnTitle')) dataPopup['btnTitle']='';
  if(dataPopup['action']=='detail') {
    if (dataPopup['btnTitle'].length <= 0) dataPopup['btnTitle'] = getMessage('look');
    arContext.add(
        MainButton(
          title: dataPopup['btnTitle'],
          minWidth: double.infinity,
          height: 35.0,
          onClick: () async {
            closePopup();
            UserProvider userProv = Provider.of<UserProvider>(context, listen: false);
            DataProvider dataProv = Provider.of<DataProvider>(context, listen: false);
            MenuProduct? prod = dataProv.getMenuProdByID(int.parse(dataPopup['value'].toString()));
            if(prod==null) prod = dataProv.getMenuProdByFO(dataPopup['value'].toString());
            if (prod != null) {
              Navigator.of(context).push(
                MaterialPageRoute(
                  builder: (BuildContext context) {
                    return ProductDetails(
                        product: prod!, addAction: userProv.basketAdd);
                  },
                ),
              );
            }
          },
        )
    );
  }else if(dataPopup['action']=='category'){
    if(dataPopup['btnTitle'].length<=0) dataPopup['btnTitle']=getMessage('look');
    arContext.add(
        MainButton(
          title: dataPopup['btnTitle'],
          minWidth: double.infinity,
          height: 35.0,
          onClick: () async{
            closePopup();
            DataProvider dataProv=Provider.of<DataProvider>(context, listen: false);
            int catId=int.parse(dataPopup['value'].toString());
            String catCode=dataPopup['value'].toString();
            //print(catId);
            MenuCategory? cat;
            dataProv.categories.forEach((e){
              if(e.id==catId || e.code==catCode) cat=e;
            });
            if(cat!=null) {
              Navigator.of(context).push(
                MaterialPageRoute(
                  builder: (BuildContext context) {
                    return Category(
                      category: cat!,
                    );
                  },
                ),
              );
            }
          },
        )
    );
  }else if(dataPopup['action']=='coupon'){
    if(dataPopup['btnTitle'].length<=0) dataPopup['btnTitle']=getMessage('apply_coupon');
    arContext.add(
      MainButton(
        title: dataPopup['btnTitle'],
        height: 35.0,
        minWidth: double.infinity,
        onClick: () async{
          closePopup();
          UserProvider userProv = Provider.of<UserProvider>(context, listen: false);
          Map res=await userProv.basketCoupon(coupon:dataPopup['value'].toString());
          if (res['type'] != '') {
            showMessage(type:res['type'],message:res['mess'], context:context);
          }
        }
      )
    );
  }else if(dataPopup['action']=='webview'){
    if(dataPopup['btnTitle'].length<=0) dataPopup['btnTitle']=getMessage('look');
    arContext.add(
      MainButton(
        title: dataPopup['btnTitle'],
        minWidth: double.infinity,
        height: 35.0,
        onClick: () async{
          closePopup();
          Navigator.of(context).push(
            MaterialPageRoute(
              builder: (BuildContext context) {
                return WebViewPage(
                  title:dataPopup['webviewTitle']!=null?dataPopup['webviewTitle']:'',
                  url: dataPopup['value'],
                );
              },
            ),
          );
        },
      )
    );
  }else if(dataPopup['action']=='close'){
    if(dataPopup['btnTitle'].length<=0) dataPopup['btnTitle']=getMessage('close');
    arContext.add(
      MainButton(
        height: 35.0,
        title: dataPopup['btnTitle'],
        minWidth: double.infinity,
        onClick: () async{
          closePopup();
        },
      )
    );
  }else if(dataPopup['action']=='function'){
    if(dataPopup['btnTitle'].length<=0) dataPopup['btnTitle']=getMessage('open');
    arContext.add(
      MainButton(
        title: dataPopup['btnTitle'],
        minWidth: double.infinity,
        height: 35.0,
        onClick: () async{
          closePopup();
          dataPopup['btnFunc']();
        },
      )
    );
  }
  popupContentShow(
    context: context,
    sizePopup: dataPopup.containsKey('size')?dataPopup['size']:0.6,
    content:Container(
      padding:EdgeInsets.only(right: 10,left:10,bottom:30.0),
      child: Column(
        children: arContext,
      ),
    ),
  );
  return;

}

void popupContentShow({required BuildContext context,required Widget content,double sizePopup=0.6,bool enableDrag=true}) async{
  /*void closePopup() {
    Navigator.pop(context);
  }*/
  Widget insert=Container(
    child:Column(
      children: [
        widgetByCondition(
          cond: enableDrag,
          func: ()=>Center(
            child: Container(
              margin: EdgeInsets.only(top:8,bottom:10),
              width: 70.0,
              height: 4.0,
              decoration: new BoxDecoration(
                color: Colors.grey,
                borderRadius: BorderRadius.circular(2.0),
              ),
            ),
          ),
        ),
        Expanded(
          child:content,
        ),
      ],
    ),
  );
  showModalBottomSheet(
    context: context,
    isScrollControlled: true,
    enableDrag:enableDrag,
    shape: RoundedRectangleBorder(
      borderRadius: BorderRadius.only(
        topLeft: const Radius.circular(10),
        topRight: const Radius.circular(10),
      ),
    ),
    builder: (context){
      return FractionallySizedBox(
        heightFactor: sizePopup,
        child:insert,
      );
    },
  );


  return;

}