import 'package:flutter/material.dart';
import 'package:badges/badges.dart';
import 'package:foapp/util/const.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/app_provider.dart';

class RBottomAppBarItem {
  RBottomAppBarItem({required this.iconData, required this.text,this.indec=''});
  IconData iconData;
  String text;
  String indec;
}

class RBottomAppBar extends StatefulWidget {
  RBottomAppBar({
    required this.items,
    this.height: 60.0,
    this.iconSize: 24.0,
    required this.backgroundColor,
    required this.color,
    required this.selectedColor,
    this.notchedShape,
    required this.onTabSelected,
  }) {
    assert(this.items.length == 2 || this.items.length == 4);
  }
  final List<RBottomAppBarItem> items;
  final double height;
  final double iconSize;
  final Color backgroundColor;
  final Color color;
  final Color selectedColor;
  final NotchedShape? notchedShape;
  final ValueChanged<int> onTabSelected;

  @override
  State<StatefulWidget> createState() => RBottomAppBarState();
}

class RBottomAppBarState extends State<RBottomAppBar> {
  int _selectedIndex = 0;

  _updateIndex(int index) {
    widget.onTabSelected(index);
    setState(() {
      _selectedIndex = index;
    });
  }

  @override
  Widget build(BuildContext context) {
    Provider.of<AppProvider>(context, listen: false).changePage=_updateIndex;
    List<Widget> items = List.generate(widget.items.length, (int index) {
      return _buildTabItem(
        item: widget.items[index],
        index: index,
        onPressed: _updateIndex,
      );
    });

    return BottomAppBar(
      shape: widget.notchedShape,
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        children: items,
      ),
      color: widget.backgroundColor,
    );
  }

  Widget _buildTabItem({
    required RBottomAppBarItem item,
    required int index,
    required ValueChanged<int> onPressed,
  }) {
    bool isSel=_selectedIndex == index;
    Color color = isSel ? widget.selectedColor : widget.color;
    Widget icoO=Icon(item.iconData, color: color, size: widget.iconSize+(isSel?4:0));
    if(item.indec.length>0) {
      icoO = Badge(
        position: BadgePosition.topEnd(top: -7, end: -7),
        animationDuration: Duration(milliseconds: 300),
        animationType: BadgeAnimationType.slide,
        badgeColor: getConstantColor('main'),
        badgeContent: Text(
          item.indec,
          style: TextStyle(
            color: getConstantColor('btnText'),
            fontWeight: FontWeight.w600,
            fontSize: 12,
            height: 1
          ),
        ),
        child: icoO,
      );
    }
    return Expanded(
      child: SizedBox(
        height: widget.height,
        child: Material(
          type: MaterialType.transparency,
          child: InkWell(
            onTap: () => onPressed(index),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                icoO,
                Text(
                  item.text,
                  style: TextStyle(color: color),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }
}