import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/tools.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:foapp/widgets/plus_minus.dart';
import 'package:foapp/widgets/price_format.dart';

import 'delete_basket.dart';
import 'disabled_button.dart';
import 'image_cust.dart';


class CartItem extends StatelessWidget {
  final Map product;
  final addAction;
  final changeBasketQty;
  final String? placeType;


  CartItem({
    Key? key,
    required this.product,
    required this.addAction,
    required this.changeBasketQty,
    this.placeType
    })
      :super(key: key);
  @override
  Widget build(BuildContext context) {
    return  Slidable(
      endActionPane: ActionPane(
        motion: const ScrollMotion(),
        //dismissible: DismissiblePane(onDismissed: () {}),
        children: [
          SlidableAction(
            onPressed: (_)=>changeBasketQty(product['id'],0),
            backgroundColor: Colors.red,
            foregroundColor: Colors.white,
            icon: Icons.delete,
            label: getMessage('delete'),
          ),
        ],
      ),
      child: Container(
        //width: MediaQuery.of(context).size.width,
        padding:EdgeInsets.only(top:10,bottom:10,left:10.0,right:10.0),
        //margin:EdgeInsets.only(bottom:15),
        decoration: BoxDecoration(
          border: Border(
            top: (placeType!='f'?BorderSide(width: 0.5, color: Theme.of(context).dividerColor):BorderSide(width: 0, color: Colors.transparent)),
            bottom: (placeType!='l'?BorderSide(width: 0.5, color: Theme.of(context).dividerColor):BorderSide(width: 0, color: Colors.transparent)),
            //top:null,
          ),
        ),
        child:Stack(
          children: <Widget>[
            Container(
              width: 100,
              alignment: Alignment.centerLeft,
              child:ImageCust(pic:product['pic']),
            ),
            Container(
              margin:EdgeInsets.only(left:110),
              child:Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text(
                    product['name'],
                    style: TextStyle(
                      fontSize: 18,
                      height: 1.1,
                      fontWeight: FontWeight.w700,
                    ),
                  ),
                  SizedBox(height: 6.0),
                  widgetByCondition(
                    cond:product['weight']!=null && product['weight'].length>0,
                    func:()=>Container(
                      margin:EdgeInsets.only(bottom:6.0),
                      child:Text(
                        product['weight'],
                        style: TextStyle(
                          fontWeight: FontWeight.bold,
                          fontSize: 14,
                          height: 1.2,
                        ),
                      ),
                    )
                  ),
                  widgetByCondition(
                    cond:product['parts'].length>0,
                    func:()=>Container(
                      margin:EdgeInsets.only(bottom:7),
                      child: Text(
                        product['parts'],
                        style: TextStyle(
                          fontSize: 14,
                          height: 1.2,
                        ),
                      ),
                    ),
                  ),
                  widgetByCondition(
                    cond:product['stop'],
                    func:()=>Text(
                      getMessage('product_in_stop2'),
                      style:TextStyle(
                        color:Colors.red,
                        fontSize:14,
                        height: 1,
                      )
                    )
                  ),
                  Container(
                    margin: EdgeInsets.only(top:10),
                    child: Row(
                      mainAxisAlignment:MainAxisAlignment.spaceBetween,
                      children: [
                        widgetByCondition(
                          cond:!product['stop'],
                          func:()=>Container(
                            child:PriceFormatText(
                              price: product['amount']*product['qty'],
                              fontSize: 18.0,
                              fontWeight: FontWeight.w700,
                              color: getConstantColor('main'),
                            ),
                          ),
                        ),
                        widgetByCondition(
                          cond:product['stop'],
                          func:()=>DisabledButton(
                            title:getMessage('delete'),
                              onClick: ()=>changeBasketQty(product['id'],0)
                          ),
                          funcFls:()=>widgetByCondition(
                            cond:product['isPresent'],
                            func:()=>DeleteBasket(func: ()=>changeBasketQty(product['id'],0)),
                            funcFls:()=>PlusMinus(
                              cnt:product['qty'],
                              funcMinus: ()=>changeBasketQty(product['id'],product['qty']-1),
                              funcPlus: ()=>changeBasketQty(product['id'],product['qty']+1),
                            ),
                          ),
                        )
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );

  }
}
