import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/dadata.dart';
import 'package:foapp/util/icons.dart';

class DadataInputField extends StatelessWidget{
  const DadataInputField({
    Key? key,
    required this.field,
    this.autofocus=false,
    required this.selectItem,
    this.readonly=false,
    required this.controller,
    this.focusNode
  }) : super(key: key);
  final Map field;
  final bool autofocus;
  final Function selectItem;
  final bool readonly;
  final TextEditingController controller;
  final FocusNode? focusNode;

  @override
  Widget build(BuildContext context) {
    controller.text = field['value'];
    controller.selection = TextSelection.collapsed(offset: field['value'].length);

    UnderlineInputBorder borderDef = UnderlineInputBorder(
      borderSide: BorderSide(
          color: getConstantColor('inputBottomBorder'),
          width: 0.5,
      ),
    );
    return TypeAheadField(
      hideOnEmpty: true,
      hideOnError: true,
      hideOnLoading: true,
      suggestionsBoxVerticalOffset: 0,
      textFieldConfiguration: TextFieldConfiguration(
        enabled: !readonly,
        autofocus: (!autofocus ? false : (field['value'].length <= 0 ||
            autofocus)),
        controller: controller,
        focusNode: focusNode,
        decoration: InputDecoration(
          contentPadding: const EdgeInsets.fromLTRB(0,0,0,4),
          enabledBorder: borderDef,
          focusedBorder: borderDef,
          border: borderDef,
          labelStyle: TextStyle(
            height: 1.0,
            fontSize:16,
            color: getConstantColor('mainGray').withAlpha(120),
          ),
          isDense:true,
          labelText: field['name'],
          errorText: field['error'].length > 0 ? field['error'] : null,
          suffix: IconButton(
              padding: EdgeInsets.zero,
              constraints: const BoxConstraints(),
              icon: Icon(iconByName('fa.times'),size:16.0),
              color: getConstantColor('red'),
              onPressed: () {
                controller.text = '';
              }
          ),
        ),
      ),
      suggestionsCallback: (pattern) async {
        return await DadataSuggest.autocomplate(pattern);
      },
      itemBuilder: (context, dynamic suggestion) {
        return ListTile(
          visualDensity: const VisualDensity(horizontal: 0, vertical: -3),
          title: Text(
              suggestion['value'],
              style: const TextStyle(
                  height: 1
              )
          ),
        );
      },
      onSuggestionSelected: (suggestion) {
        selectItem(suggestion);
      },
      suggestionsBoxDecoration: SuggestionsBoxDecoration(
        color: getConstantColor('dropDownBg'),
      ),
    );
  }
}