import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/screens/details.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/plus_minus.dart';
import 'package:foapp/widgets/price_format.dart';
import 'package:foapp/widgets/empty_widget.dart';

import 'disabled_button.dart';
import 'image_cust.dart';
import 'main_button.dart';

class Grid2Product extends StatefulWidget {
  final MenuProduct product;
  final addAction;

  Grid2Product({
    Key? key,
    required this.product,
    required this.addAction,
  })
      :super(key: key);

  @override
  _Grid2ProductState createState() => _Grid2ProductState();
}

class _Grid2ProductState extends State<Grid2Product> {
  late MenuProduct product;
  String curFoCode='';

  @override
  void initState() {
    product=widget.product;
    if(curFoCode=='') curFoCode=product.prices.first['code'];

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    //int maxCl=Constants.styles['categoryTypeList']=='grid2'?2:1;
    double pd=getConstantStyle('gridSpaceSize')*1.2;
    //double lnsTitle=getConstantStyle('gridSizeLines');
    double fsTitle=getConstantStyle('fontSizeCatName');
    double fsDesc=getConstantStyle('fontSizeCatDesc');
    double fsPrice=getConstantStyle('fontSizeGrid2Price');
    if(fsTitle>16) fsTitle=16.0;
    if(fsDesc>14) fsDesc=14.0;
    bool noDesc=getConstantStyle('catListNoDesc')=='Y';
    Map curPrice=product.prices.where((e) => e['code']==curFoCode).first;
    Widget hotW=_getIcons();
    return Container(
      //margin:EdgeInsets.only(top:14.0,bottom:14.0),
      //padding:EdgeInsets.only(top:10.0),
      clipBehavior: Clip.antiAlias,
      decoration: BoxDecoration(
        //border: Border(bottom: BorderSide(width: 1, color: Theme.of(context).dividerColor)),
        //border: Border(bottom: BorderSide(width: 1, color: Colors.red)),
        color: getConstantColor('bg'),
        borderRadius: BorderRadius.all(Radius.circular(getConstantStyle('radiusSizeTovars'))),
        boxShadow: [
          BoxShadow(
            color: getConstantColor('grayMain').withOpacity(0.3),
            spreadRadius: 3,
            blurRadius: 8,
            //offset: Offset(0, 3), // changes position of shadow
          ),
        ],
      ),
      child:InkWell(
        child:Column(
          mainAxisSize: MainAxisSize.max,
          children:[
            Stack(
              children:[
                Container(
                  alignment: Alignment.center,
                  child:Opacity(
                    opacity: curPrice['stop']?0.6:1.0,
                    child:AspectRatio(
                      aspectRatio: 1,
                      child: ImageCust(pic:product.picm),
                    ),
                  ),
                ),
                Positioned(
                  right:0,
                  top:10,
                  child:hotW,
                ),
              ],
            ),
            Expanded(
              child:Container(
                padding:EdgeInsets.all(pd),
                child:Column(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Container(
                      child:Column(
                        children:[
                          Container(
                            alignment: Alignment.topLeft,
                            padding:EdgeInsets.only(bottom: 5),
                            //height: lnsTitle*(fsTitle*1.3),
                            child: Text(
                              product.name,
                              //maxLines: lnsTitle.toInt(),
                              //overflow: TextOverflow.ellipsis,
                              style: TextStyle(
                                fontSize: fsTitle,
                                height: 1.1,
                                fontWeight: FontWeight.w500,
                              ),
                            ),
                          ),
                          widgetByCondition(
                            cond: !noDesc,
                            func: ()=>Padding(
                              padding:EdgeInsets.only(bottom: 5),
                              child:Text(
                                product.desc,
                                style: TextStyle(
                                  fontSize: fsDesc,
                                  height: 1.1,
                                ),
                                maxLines: 3,
                              ),
                            ),
                          ),
                          widgetByCondition(
                            cond: product.prices.length>1,
                            func: ()=>_getSizeSelector(),
                          ),
                        ],
                      ),
                    ),
                    Container(
                      child: Container(
                        width: double.infinity,
                        margin:EdgeInsets.only(top:5),
                        child: Stack(
                          alignment: AlignmentDirectional.bottomCenter,
                          children: [
                            widgetByCondition(
                              cond:curPrice['inBasket']>0 && !curPrice['stop'],
                              func:()=>Positioned(
                                left:0,
                                top:0,
                                right:0,
                                child:Container(
                                  padding:EdgeInsets.only(left:10.0),
                                  //width: double.infinity,
                                  alignment: Alignment.center,
                                  child:PriceFormatText(
                                    price: curPrice['price'],
                                    fontSize: fsPrice,
                                    height: 1,
                                    fontWeight: FontWeight.w700,
                                  ),
                                ),
                              ),
                            ),
                            Container(
                              margin:EdgeInsets.only(top:10.0),
                              height: 38.0,
                              child:widgetByCondition(
                                cond:curPrice['stop'],
                                func:()=>DisabledButton(
                                  title:getMessage('product_in_stop'),
                                  minWidth: double.infinity,
                                  fontSize:getConstantStyle('fontSizeBtn')-1,
                                ),
                                funcFls:()=>widgetByCondition(
                                  cond:curPrice['inBasket']>0,
                                  func:()=>Container(
                                    margin:EdgeInsets.only(right:10.0),
                                    child:PlusMinus(
                                      cnt:curPrice['inBasket'],
                                      funcMinus: ()=>widget.addAction(curPrice['code'].toString(),-1),
                                      funcPlus: ()=>widget.addAction(curPrice['code'].toString(),1),
                                    ),
                                  ),
                                  funcFls:()=>Container(
                                    child:MainButton(
                                      minWidth: double.infinity,
                                      contIn:PriceFormatText(
                                        price:curPrice['price'],
                                        fontSize: getConstantStyle('fontSizeBtn')-2,
                                        color:getConstantColor('btnText'),
                                        fontWeight: FontWeight.w600,
                                      ),
                                      title: getMessage('in_basket'),
                                      onClick:() {
                                        //if (curPrice['parts'] == null && curPrice['ings'] == null)
                                        if(curPrice['needDetail']!='y')
                                          widget.addAction(curPrice['code'].toString(), 1);
                                        else
                                          openProduct();
                                      }
                                    ),
                                  ),
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ),
        onTap: (){
          openProduct();
        },
      ),
    );
  }

  Widget _getSizeSelector(){
    List<Widget> szl=[];
    product.prices.forEach((price){
      szl.add(GestureDetector(
        child:Container(
          child:Text(
            price['weight'],
            style:TextStyle(
              fontSize: getConstantStyle('fontSizeCatSizeText')*.9,
              color: price['code']==curFoCode?getConstantColor('btnText'):getConstantColor('text'),
            ),
          ),
          padding: EdgeInsets.fromLTRB(7.0, 3, 7.0, 3),
          //margin: EdgeInsets.only(bottom:5.0),
          decoration: BoxDecoration(
            color: price['code']==curFoCode?getConstantColor('main'):Colors.transparent,
            border: Border.all(width: 1, color: getConstantColor('main')),
            borderRadius: BorderRadius.all(
              Radius.circular(getConstantStyle('radiusSizeBtn')*0.75),
            ),
          ),
        ),
        onTap: (){
          setState(() {
            curFoCode=price['code'];
          });
        },
      ));
    });
    return Container(
      alignment: Alignment.topLeft,
      margin:EdgeInsets.only(top:10.0),
      child:Wrap(
        alignment: WrapAlignment.spaceAround,
        spacing: 5.0,
        children: szl,
      ),
    );
  }
  void openProduct(){
    Navigator.of(context).push(
      MaterialPageRoute(
        builder: (BuildContext context){
          return ProductDetails(product: widget.product, addAction: widget.addAction,curFoCode:curFoCode);
        },
      ),
    );
  }
  Widget _getIcons(){
    Widget hot=EmptyWidget();
    if(product.icons!=null && product.icons!.length>0){
      List<Widget> hts=[];
      product.icons!.forEach((el) {
        el=el.toString();
        hts.add(Container(
          margin:EdgeInsets.only(right:7.0),
          child: ImageCust(pic:el,width:getConstantStyle('icoSizeProduct'),height:getConstantStyle('icoSizeProduct')),
        ));
      });
      hot=Container(
        child:Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            mainAxisAlignment: MainAxisAlignment.end,
            children:hts
        ),
      );
    }
    return hot;
  }
}
