import 'dart:convert';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:url_launcher/url_launcher.dart' as urlLauncher;

class HtmlBlock extends StatelessWidget{
  const HtmlBlock({
    Key? key,
    this.url='',
    this.html='',
    this.webViewHandle,
    this.onWebViewCreated,
  }) : super(key: key);
  final String url;
  final String html;
  final dynamic webViewHandle;
  final dynamic onWebViewCreated;

  @override
  Widget build(BuildContext context) {
    InAppWebViewGroupOptions options = InAppWebViewGroupOptions(
        crossPlatform: InAppWebViewOptions(
          useShouldOverrideUrlLoading: true,
          mediaPlaybackRequiresUserGesture: false,
          transparentBackground: true,
        ),
        android: AndroidInAppWebViewOptions(
          useHybridComposition: true,
          forceDark: AndroidForceDark.FORCE_DARK_OFF,
        ),
        ios: IOSInAppWebViewOptions(
          allowsInlineMediaPlayback: true,
        ));
    Uri uri=Uri.parse(url);
    if(url.isEmpty){
      String fontMain='PT Sans';
      uri=Uri.dataFromString('<html><head><meta name="viewport" content="width=device-width,initial-scale=1.0,user-scalable=0"><link href="https://fonts.googleapis.com/css2?family='+fontMain+':ital,wght@0,400;0,700;1,400;1,700&display=swap" rel="stylesheet"><style>html,body{font-family: "'+fontMain+'",Arial;padding:0;margin:0;}</style></head><body><div id="page-div">'+html+'</div><script>var onResizeSetHeight=function(){window.addEventListener("flutterInAppWebViewPlatformReady", function(event) {const args=[{"height":document.getElementById("page-div").offsetHeight+15}];window.flutter_inappwebview.callHandler("webViewHandle", ...args).then(function(result) {console.log(JSON.stringify(result));});});};window.addEventListener("resize", function(event) {onResizeSetHeight();}, true);onResizeSetHeight();setTimeout(onResizeSetHeight,1000);</script></body></html>', mimeType: 'text/html',encoding: Encoding.getByName('utf-8'));
    }
    return InAppWebView(
      gestureRecognizers: url.isEmpty?(Set()..add(Factory<VerticalDragGestureRecognizer>(() => VerticalDragGestureRecognizer()))):null,

      initialUrlRequest: URLRequest(url: uri),
      initialOptions: options,
      onWebViewCreated: (controller) {
        if(onWebViewCreated!=null) onWebViewCreated(controller);
        controller.addJavaScriptHandler(
            handlerName: 'webViewHandle',
            callback: (args) {
              if(webViewHandle!=null) return webViewHandle(args);
              return {};
            }
        );
        controller.addJavaScriptHandler(
            handlerName: 'webViewOpenLink',
            callback: (args) {
              urlLauncher.launch(args[0]);
            }
        );

      },
    );
  }
}