import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:foapp/util/const.dart';

class ImageCust extends StatelessWidget{
  const ImageCust({
    Key? key,
    required this.pic,
    this.width,
    this.height,
    this.boxFit=BoxFit.contain,
  }) : super(key: key);
  final String pic;
  final double? width;
  final double? height;
  final BoxFit boxFit;

  @override
  Widget build(BuildContext context) {
    if(pic.substring(pic.length-4).toLowerCase()=='.svg'){
      return ImageCustSvg(
        pic:pic,
        width: width,
        height:height,
      );
    }else{
      return ImageCustPic(
        pic:pic,
        width: width,
        height:height,
        boxFit:boxFit,
      );
    }
  }
}


class ImageCustSvg extends StatelessWidget{
  const ImageCustSvg({
    Key? key,
    required this.pic,
    this.width,
    this.height,
  }) : super(key: key);
  final String pic;
  final double? width;
  final double? height;

  @override
  Widget build(BuildContext context) {
    if(pic.substring(0,4)!='http'){
      return SvgPicture.asset(
        pic,
        width: width,
        height: height,
        placeholderBuilder: (BuildContext context) =>
            Center(
              child: SizedBox(
                width: 50.0,
                height: 50.0,
                child: CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation<Color>(
                      Constants.colors['grayMain']),
                ),
              ),
            ),
      );
    }else{
      return SvgPicture.network(
        pic,
        width: width,
        height: height,
        placeholderBuilder: (BuildContext context) =>
            Center(
              child: SizedBox(
                width: 50.0,
                height: 50.0,
                child: CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation<Color>(
                      Constants.colors['grayMain']),
                ),
              ),
            ),
      );
    }
  }
}
class ImageCustPic extends StatelessWidget{
  const ImageCustPic({
    Key? key,
    required this.pic,
    this.width,
    this.height,
    this.boxFit=BoxFit.contain,
  }) : super(key: key);
  final String pic;
  final double? width;
  final double? height;
  final BoxFit boxFit;

  @override
  Widget build(BuildContext context) {
    if(pic.substring(0,4)!='http'){
      return Image.asset(
        pic,
        width: width,
        height: height,
      );
    }else {
      return CachedNetworkImage(
        imageUrl: pic,
        width: width,
        height: height,
        fit: boxFit,
        placeholder: (context, url) =>
            Center(
              child: SizedBox(
                width: 50.0,
                height: 50.0,
                child: CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation<Color>(
                      Constants.colors['grayMain']),
                ),
              ),
            ),
        errorWidget: (context, url, error) => const Icon(Icons.error),
      );
    }
  }
}