import 'package:flutter/material.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/screens/details.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/menu_product.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/plus_minus.dart';
import 'package:foapp/widgets/price_format.dart';
import 'package:foapp/widgets/empty_widget.dart';

import 'disabled_button.dart';
import 'image_cust.dart';
import 'main_button.dart';

class LineProduct extends StatefulWidget {
  final MenuProduct product;
  final addAction;

  LineProduct({
    Key? key,
    required this.product,
    required this.addAction,
  })
      :super(key: key);

  @override
  _LineProductState createState() => _LineProductState();
}

class _LineProductState extends State<LineProduct> {
  late MenuProduct product;
  String curFoCode='';

  @override
  void initState() {
    product=widget.product;
    if(curFoCode=='') curFoCode=product.prices.first['code'];

    super.initState();
  }

  @override
  Widget build(BuildContext context) {

    Map curPrice=product.prices.where((e) => e['code']==curFoCode).first;
    Widget hotW=_getIcons();
    bool noDesc=getConstantStyle('catListNoDesc')=='Y';
    return Container(
      margin: EdgeInsets.only(left:15.0,right:15.0),
      decoration: BoxDecoration(
        border: Border(bottom: BorderSide(width: 1, color: Theme.of(context).dividerColor)),
      ),
      child:InkWell(
        child:Stack(
          children: [
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children:[
                Flexible(
                  flex:2,
                  child:Container(
                    alignment: Alignment.center,
                    padding:EdgeInsets.only(right:10.0),
                    child:Opacity(
                      opacity: curPrice['stop']?0.6:1.0,
                      child:AspectRatio(
                        aspectRatio: 1,
                        child: ImageCust(pic:product.picm),
                      ),
                    ),
                  ),
                ),
                Flexible(
                  flex:3,
                  child:Container(
                    padding:EdgeInsets.only(top:15.0,bottom:15.0),
                    child:Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Padding(
                          padding:EdgeInsets.only(bottom: 5),
                          child: Text(
                            product.name,
                            style: TextStyle(
                              fontSize: getConstantStyle('fontSizeCatName'),
                              fontWeight: FontWeight.w600,
                            ),
                          ),
                        ),
                        widgetByCondition(
                            cond: !noDesc,
                            func: ()=>Padding(
                              padding:EdgeInsets.only(bottom: 5),
                              child:Text(
                                product.desc,
                                style: TextStyle(
                                  fontSize: getConstantStyle('fontSizeCatDesc'),
                                ),
                                maxLines: 3,
                              ),
                            ),
                        ),
                        widgetByCondition(
                          cond: product.prices.length>1,
                          func: ()=>_getSizeSelector(),
                        ),
                        Container(
                          child: Padding(
                            padding:EdgeInsets.only(top: 20),
                            child: Row(
                              mainAxisAlignment:MainAxisAlignment.spaceBetween,
                              children: [
                                Container(
                                  child:PriceFormatText(
                                    price: curPrice['price'],
                                    fontSize: getConstantStyle('fontSizePrice'),
                                    height: 1,
                                    fontWeight: FontWeight.w700,
                                  ),
                                ),
                                Container(
                                  height: 38,
                                  child:widgetByCondition(
                                    cond:curPrice['stop'],
                                    func:()=>DisabledButton(title:getMessage('product_in_stop')),
                                    funcFls:()=>widgetByCondition(
                                      cond:curPrice['inBasket']>0,
                                      func:()=>PlusMinus(
                                        cnt:curPrice['inBasket'],
                                        funcMinus: ()=>widget.addAction(curPrice['code'].toString(),-1),
                                        funcPlus: ()=>widget.addAction(curPrice['code'].toString(),1),
                                      ),
                                      funcFls:()=>Container(
                                        margin:EdgeInsets.only(right:0),
                                        child:MainButton(
                                          title: getMessage('in_basket'),
                                          onClick:(){
                                            //if(curPrice['parts']==null && curPrice['ings']==null)
                                            if(curPrice['needDetail']!='y')
                                              widget.addAction(curPrice['code'].toString(),1);
                                            else
                                              openProduct();
                                          }
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
              ],
            ),
            Positioned(
              left:5,
              top:15,
              child:hotW,
            ),
          ],
        ),
        onTap: (){
          openProduct();
        },
      ),
    );
  }

  Widget _getSizeSelector(){
    List<Widget> szl=[];
    product.prices.forEach((price){
      szl.add(GestureDetector(
        child:Container(
          child:Text(
            price['weight'],
            style:TextStyle(
              fontSize: getConstantStyle('fontSizeCatSizeText'),
              color: price['code']==curFoCode?getConstantColor('btnText'):getConstantColor('text'),
            ),
          ),
          padding: EdgeInsets.fromLTRB(10.0, 3, 10.0, 3),
          margin: EdgeInsets.only(right:15.0),
          decoration: BoxDecoration(
            color: price['code']==curFoCode?getConstantColor('main'):Colors.transparent,
            border: Border.all(width: 1, color: getConstantColor('main')),
            borderRadius: BorderRadius.all(
              Radius.circular(getConstantStyle('radiusSizeBtn')*0.75),
            ),
          ),
        ),
        onTap: (){
          setState(() {
            curFoCode=price['code'];
          });
        },
      ));
    });
    return Container(
      margin:EdgeInsets.only(top:10.0),
      child:Row(
        children: szl,
      ),
    );
  }
  void openProduct(){
    Navigator.of(context).push(
      MaterialPageRoute(
        builder: (BuildContext context){
          return ProductDetails(product: widget.product, addAction: widget.addAction,curFoCode:curFoCode);
        },
      ),
    );
  }
  Widget _getIcons(){
    Widget hot=EmptyWidget();
    if(product.icons!=null && product.icons!.length>0){
      List<Widget> hts=[];
      product.icons!.forEach((el) {
        el=el.toString();
        hts.add(Container(
          margin:EdgeInsets.only(right:7.0),
          child: ImageCust(pic:el,width:getConstantStyle('icoSizeProduct'),height:getConstantStyle('icoSizeProduct')),
        ));
      });
      hot=Container(
        child:Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            mainAxisAlignment: MainAxisAlignment.end,
            children:hts
        ),
      );
    }
    return hot;
  }
}
