import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/icons.dart';

class MainDropDown extends StatelessWidget{
  const MainDropDown({
    Key? key,
    required this.value,
    required this.items,
    this.onChanged,
    this.hint,
    this.selectedItemBuilder,
    this.padding=const EdgeInsets.fromLTRB(0, 15, 0, 7),
  }) : super(key: key);
  final String value;
  final List<DropdownMenuItem<String>> items;
  final void Function(String?)? onChanged;
  final Widget? hint;
  final List<Widget> Function(BuildContext)? selectedItemBuilder;
  final EdgeInsetsGeometry? padding;

  @override
  Widget build(BuildContext context) {
    UnderlineInputBorder borderDef = UnderlineInputBorder(
      borderSide: BorderSide(
          color: Constants.colors['inputBottomBorder'], width: 0.5),
    );
    return DropdownButtonFormField<String>(
      value: value,
      onChanged: onChanged,
      items: items,
      hint: hint,
      isDense:true,
      selectedItemBuilder: selectedItemBuilder,
      isExpanded: true,
      icon:Icon(
        iconByName('keyboard_arrow_down'),
        size: 24,
      ),
      decoration: InputDecoration(
        isDense: true,
        contentPadding: padding,
        enabledBorder: borderDef,
        focusedBorder: borderDef,
        border: borderDef,
      ),
    );
  }
}