import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:foapp/util/const.dart';

class MainInputField extends StatelessWidget{
  const MainInputField({
    Key? key,
    this.keyboardType,
    this.lines=1,
    this.enabled=true,
    this.readOnly=false,
    this.isPass=false,
    this.labelText,
    this.errorText,
    this.controller,
    this.onChanged,
    this.onTap,
    this.focusNode,
    this.inputFormatters,
  }) : super(key: key);
  final TextInputType? keyboardType;
  final int lines;
  final bool enabled;
  final bool readOnly;
  final bool isPass;
  final String? labelText;
  final String? errorText;
  final TextEditingController? controller;
  final void Function(String)? onChanged;
  final void Function()? onTap;
  final FocusNode? focusNode;
  final List<TextInputFormatter>? inputFormatters;

  @override
  Widget build(BuildContext context) {
    UnderlineInputBorder borderDef=UnderlineInputBorder(
      borderSide: BorderSide(color: Constants.colors['inputBottomBorder'], width: 0.5),
    );
    return TextField(
      enabled:enabled,
      onTap: onTap,
      readOnly: readOnly,
      keyboardType: keyboardType,
      focusNode: focusNode,
      obscureText:isPass,
      inputFormatters:inputFormatters,
      //keyboardType: (fldName=='fio'?TextInputType.name:TextInputType.text),
      maxLines: lines,
      autocorrect: true,
      decoration: InputDecoration(
        contentPadding: const EdgeInsets.fromLTRB(0,0,0,4),
        enabledBorder: borderDef,
        focusedBorder: borderDef,
        border: borderDef,
        labelText: labelText,
        labelStyle: TextStyle(
          height: 1.0,
          fontSize:16,
          color: getConstantColor('mainGray').withAlpha(120),
        ),
        errorText: (errorText!=null && errorText!.isNotEmpty?errorText:null),
        isDense:true,
      ),
      style: TextStyle(
        color:getConstantColor('text').withOpacity(enabled?1.0:0.5),
      ),
      controller: controller,
      onChanged: onChanged,
      textInputAction: TextInputAction.done,
      onEditingComplete: () {
        FocusScope.of(context).unfocus();
      }
    );
  }
}