import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:flutter_svg/svg.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/screens/checkout.dart';
import 'package:foapp/screens/orderfinish.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/fbanalitics.dart';
import 'package:foapp/util/icons.dart';
import 'package:foapp/util/phone.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/empty_widget.dart';
import 'package:foapp/widgets/plus_minus.dart';
import 'package:foapp/widgets/show_message.dart';
import 'package:foapp/widgets/price_format.dart';
import 'package:foapp/widgets/warning_mess.dart';
import 'package:provider/provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/widgets/address_select.dart';

import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'main_button.dart';
import 'main_dropdown.dart';
import 'main_input_field.dart';

class MakeOrder extends StatefulWidget {
  @override
  _MakeOrderState createState() => _MakeOrderState();
}

class _MakeOrderState extends State<MakeOrder> {
  late DataProvider dataProv;
  late UserProvider userProv;
  late Map basketObj;
  late OrderMotor order;
  bool isLoading=false;
  Map warningObj={'type':'','mess':'','key':''};
  String deliveryTime='';
  Map<String,TextEditingController> _textControllers={};
  bool disposed=false;
  Future _checkOrderDiscount() async{
    await order.checkDiscount();
    basketObj = userProv.basketGetCart();

    setState(() {

    });
  }

  Future _checkWorning({bool noSt=false}) async{

    String key=order.fields['type']!['value']+order.fields['address']!['value']+order.fields['department']!['value']+order.fields['date']!['value']+order.fields['time']!['value'];
    if(key!=warningObj['key']){
      warningObj['key']=key;
      Map res=await order.checkAddress();
      //print(res);
      if(res['type']=='dp') {
        order.deliveryPrice = double.parse(res['delivery'].toString());
      }else{
        order.deliveryPrice=0.0;
      }
      if(res.containsKey('department') && res['department']){
        userProv.setDepartment(res['department']);
      }
      deliveryTime=res['deliveryTime']!=null?res['deliveryTime'].toString():'';
      warningObj['type']=res['type']=='dp'?'warning':'error';
      warningObj['mess']=res['mess'];
      if(!noSt && !disposed) {
        setState(() {

        });
      }
    }
  }

  @override
  void initState() {
    dataProv = Provider.of<DataProvider>(context, listen: false);
    userProv = Provider.of<UserProvider>(context, listen: false);
    basketObj = userProv.basketGetCart();
    order=userProv.order;
    order.inited=false;
    _initOrder();
    super.initState();

  }
  @override
  void dispose() {
    disposed=true;
    _textControllers.forEach((k,e) {e.dispose();});
    super.dispose();

  }

  @override
  Widget build(BuildContext context) {
    if(!order.inited){
      return Center(
        child:CircularProgressIndicator(),
      );
    }else{
      return _orderPage();
    }
  }
  Future _initOrder() async{
    if(order.inited) return true;
    await userProv.userSync();
    await order.init(noDeliv: basketObj['nodelivery']);
    if(dataProv.globalSettings['no_samovivoz']=='y'){
      order.fields['type']!['value']='delivery';
    }
    if(basketObj['nodelivery']){
      basketObj = userProv.basketGetCart();
    }

    FbAnalitics.pageSet('OrderStart');
    FbAnalitics.checkoutStart(sum:basketObj['amount']);
    _checkWorning();
  }
  Widget _orderPage(){
    bool isNal=false;
    order.payments.forEach((el) {
      if(el['id']==order.fields['payment']!['value'] && el['isnal']) isNal=true;
    });
    basketObj['itogo']=basketObj['amount'];
    int bonuses=int.parse(order.fields['bonuses']!['value'].toString());
    if(bonuses>0){
      basketObj['itogo']-=bonuses;
    }
    _checkWorning();
    return Column(
      children:[
        Expanded(
          child:Padding(
            padding: EdgeInsets.fromLTRB(10.0,0,10.0,0),
            child: ListView(
              physics: BouncingScrollPhysics(),
              children: <Widget>[
                WarningMess(text:warningObj['mess'],type:warningObj['type']),
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Text(
                      '+7 '+getPhoneFormat(userProv.user['phone']),
                      style: TextStyle(
                        fontSize: 20,
                        fontWeight: FontWeight.bold,
                      ),
                    ),

                    IconButton(
                      icon: Icon(Icons.logout, size: 22),
                      onPressed: (){
                        Provider.of<UserProvider>(context, listen: false).userLogout();
                        Navigator.pop(context);
                        Navigator.of(context).push(
                          MaterialPageRoute(
                            builder: (BuildContext context){
                              return Checkout();
                            },
                          ),
                        );
                      },
                    ),
                  ],
                ),
                Row(
                  children: [
                    Expanded(
                      child:getOrderField('fio',TextInputType.name),
                    ),
                    getOrderField('persons'),
                  ],
                ),
                widgetByCondition(
                  cond:!basketObj['nodelivery'] && dataProv.globalSettings['no_samovivoz']!='y',
                  func:()=>getOrderField('type'),
                ),
                (order.fields['type']!['value']=='samovivoz'?typeSamovivoz():typeDelivery()),
                Container(
                  margin:EdgeInsets.only(top:15),
                  child:Text(
                    getMessage(order.fields['type']!['value']=='samovivoz'?'of_when_samovivoz':'of_when_delivery'),
                    style: TextStyle(
                      fontSize: 20,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ),
                Row(
                  children: [
                    Expanded(
                      flex:2,
                      child:Container(
                        padding:EdgeInsets.only(right: 30),
                        child:getOrderField('date'),
                      ),
                    ),
                    Expanded(
                      flex:1,
                      child: widgetByCondition(
                        cond:order.fields['date']!['value']!='now',
                        func:()=>getOrderField('time'),
                        funcFls:()=>Container(
                          margin:EdgeInsets.only(top:15.0),
                          child:Text(
                            deliveryTime.length>0?getMessage('delivery_time_phrase',{'minutes':deliveryTime}):'',
                            //textAlign: TextAlign.right,
                            style:TextStyle(
                              color: Colors.grey,
                            )
                          )
                        )
                      ),
                    ),
                  ],
                ),
                Container(
                  margin:EdgeInsets.only(top:30),
                  child:Text(
                    getMessage('of_payment_type'),
                    style: TextStyle(
                      fontSize: 20,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ),
                getOrderField('payment'),
                widgetByCondition(
                  cond:isNal,
                  func:()=>Container(
                    //margin:EdgeInsets.only(top:15,bottom: 15),
                    child:getOrderField('odd',TextInputType.number),
                  )
                ),
                widgetByCondition(
                  cond:order.fields['bonuses']!['max']!='0',
                  func:()=>Container(
                    //margin:EdgeInsets.only(top:15,bottom: 15),
                    child:getOrderField('bonuses'),
                  )
                ),
                Container(
                  margin:EdgeInsets.only(top:15,bottom: 1),
                  child:getOrderField('remark'),
                ),

              ],
            ),
          ),
        ),
        Container(
          height: 60.0+(MediaQuery.of(context).padding.bottom>0?(MediaQuery.of(context).padding.bottom-10.0):0.0),
          padding:EdgeInsets.only(top:10.0,left:10.0,right:10.0,bottom:(MediaQuery.of(context).padding.bottom>0?MediaQuery.of(context).padding.bottom:10.0)),
          child: MainButton(
            minWidth: double.infinity,
            radius: getConstantStyle('radiusSizeBtnBig'),
            title:getMessage('make_order_on').toUpperCase(),
            contIn: widgetByCondition(
              cond:isLoading,
              func:()=>SpinKitThreeBounce(color: getConstantColor('primary')),
              funcFls:()=>Text(
                getMessage('make_order_on',{'#sum#':priceFormat(basketObj['itogo']+order.deliveryPrice)}).toUpperCase(),
                style: TextStyle(
                  color:getConstantColor('btnText'),
                  fontWeight: FontWeight.w600,
                  fontSize: 16,
                  fontFamily: 'Roboto',
                ),
              ),
            ),
            onClick: () async{
              if(isLoading) return;
              setState(() {isLoading=true;});
              Map data=await order.send();
              setState(() {isLoading=false;});
              if(data['error'].length>0){
                showMessage(type:'error',message:data['error'], context: context);
              }else{
                if(data['hasError']){
                  setState(() {});
                }else{
                  Navigator.pop(context);
                  Provider.of<AppProvider>(context, listen: false).changePage(0);
                  Navigator.of(context).push(
                    MaterialPageRoute(
                      builder: (BuildContext context){
                        return OrderFinish(params:data['data']);
                      },
                    ),
                  );
                }
              }
            },
          ),
        ),
      ],
    );
  }
  Widget typeSamovivoz(){
    return Container(
      margin:EdgeInsets.only(top:15,bottom: 7),
      child:Theme(
        data: Constants.themeDropDown(context),
        child:DropdownButton<String>(
          value: order.fields['department']!['value']==''?null:order.fields['department']!['value'],
          isExpanded: true,
          elevation: 18,
          itemHeight: 60.0,
          hint: Container(
            padding:EdgeInsets.only(top:18),
            child:Text(
              getMessage('of_department_choose'),
            ),
          ),
          onChanged: (String? newValue) {
            order.setField('department', newValue!=null?newValue:'');
            setState((){});
          },
          selectedItemBuilder: (BuildContext context) {
            return order.departments.map<Widget>((el) {
              return Container(
                padding: EdgeInsets.only(top:7,left:0),
                child:Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children:[
                    Text(
                        el['name'],
                        style:TextStyle(
                          fontWeight: FontWeight.bold,
                          fontSize: 16,
                        )
                    ),
                    Text(
                        el['desc'],
                        maxLines:1,
                        style:TextStyle(
                          color: Colors.grey,
                          fontSize: 14,
                        )
                    ),
                  ],
                ),
              );
            }).toList();
          },
          items: order.departments.map<DropdownMenuItem<String>>((el) {
            return DropdownMenuItem<String>(
              value: el['code'],
              child: Container(
                width: double.infinity,
                padding: EdgeInsets.only(top:3,bottom: 10),
                margin: EdgeInsets.only(top:3,bottom: 3),
                decoration: BoxDecoration (
                  border: Border(
                    bottom: BorderSide(
                      color: Constants.colors['grayMain'],
                      width: 1.0,
                    ),
                  ),
                ),
                child:Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children:[
                    Text(
                      el['name'],
                      style:TextStyle(
                        fontWeight: FontWeight.bold,
                        fontSize: 16,
                      )
                    ),
                    Text(
                      el['desc'],
                      style:TextStyle(
                        color: Colors.grey,
                        fontSize: 14,
                      )
                    ),
                  ],
                ),
              ),
            );
          }).toList(),
        ),
      ),
    );
  }
  Widget typeDelivery(){
    return Container(
      margin:EdgeInsets.only(top:22,bottom: 15),
      child:AddressSelect(curAddress: order.fields['address']!['value'],addresses: order.addresses,selectAddress: _selectAddress),
    );
  }
  void _selectAddress(String addr){
    order.setField('address', addr);
    setState(() {

    });
  }
  Widget addressTypeSelector(String fldName,Map field){
    double radiusB=getConstantStyle('radiusSizeSelDelivery');
    List<Widget> arBtns=[];
    field['list'].forEach((k,tlt){
      arBtns.add(Expanded(
        flex: 1,
        child: InkWell(
          child: Container(
            padding: EdgeInsets.all(8),
            alignment: Alignment.center,
            decoration: BoxDecoration (
              color: field['value']==k?getConstantColor('main'):getConstantColor('orderBgSelectType'),
              borderRadius: BorderRadius.circular(radiusB-2),
            ),
            child:Text(
              tlt,
              style:TextStyle(
                fontWeight: field['value']==k?FontWeight.bold:FontWeight.normal,
                color: field['value']==k?getConstantColor('btnText'):getConstantColor('text'),
              )
            ),
          ),
          onTap: (){
            order.setField(fldName, k);
            setState((){});
            _checkOrderDiscount();
          },
        ),
      ));
    });
    return Container(
      margin:EdgeInsets.only(top:20,bottom:5),
      padding: EdgeInsets.all(5),
      decoration: BoxDecoration (
        color: Constants.colors['orderBgSelectType'],
        borderRadius: BorderRadius.circular(radiusB),
      ),
      child: Row(
        children: arBtns,
      ),
    );
  }
  void _addTextController(String fldName){
    _textControllers[fldName]=TextEditingController();
    _textControllers[fldName]!.addListener(() {
      if(order.fields[fldName]!['error'].length>0 && _textControllers[fldName]!.text.length>0){
        order.fields[fldName]!['error']='';
        setState(() {});
        int ofs=_textControllers[fldName]!.text.length;
        _textControllers[fldName]!.selection = TextSelection.collapsed(offset: ofs);
      }
    });
  }
  Widget getOrderField(String fldName,[TextInputType? keyBordType]){
    if(order.fields[fldName]==null) return EmptyWidget();
    Map field=order.fields[fldName]!;
    Widget res=EmptyWidget();
    if(field['type']=='text' || field['type']=='textarea'){
      if(keyBordType==null) keyBordType=TextInputType.text;
      int lines=1;
      if(field['type']=='textarea'){
        lines=2;
      }
      if(!_textControllers.containsKey(fldName)){
        _addTextController(fldName);
      }
      if(_textControllers[fldName]!.text!=field['value']){
        _textControllers[fldName]!.text=field['value'];
      }
      res=MainInputField(
          keyboardType:keyBordType,
          lines:lines,
          labelText:field['title'],
          errorText:field['error'],
          controller:_textControllers[fldName],
          onChanged:(v){
            order.setField(fldName,v);
          }
      );
    }else if(field['type']=='plm'){
      int v=int.parse(field['value'].toString());
      return Container(
        padding:EdgeInsets.only(top:15),
        child:Column(
          children:[
            Text(
              field['title'],
              style:TextStyle(
                color:Colors.grey,
              )
            ),
            PlusMinus(
              cnt:v,
              funcMinus: () {
                v--;
                if(v<0) v=0;
                order.setField(fldName, v.toString());
                setState((){});
              },
              funcPlus: (){
                v++;
                if(v>20) v=20;
                order.setField(fldName, v.toString());
                setState((){});
              },
            ),
          ],
        ),
      );
    }else if(field['type']=='typeaddress'){
      res=addressTypeSelector(fldName,field);
    }else if(field['type']=='list') {
      Map lst=field['list'];
      if(fldName=='time'){
        if(!field['list'].containsKey(order.fields['date']!['value'])) return EmptyWidget();
        lst=field['list'][order.fields['date']!['value']];
      }
      if(!lst.containsKey(field['value'])){
        field['value']=lst.keys.length>0?lst.keys.first:'';
      }
      List<DropdownMenuItem<String>> drpList=[];
      lst.forEach((k,lb){
        drpList.add(
          DropdownMenuItem<String>(
            value: k,
            child: Container(
              width: double.infinity,
              child: Text(
                  lb,
                  style: TextStyle(
                    fontWeight: FontWeight.bold,
                    fontSize: 16,
                  )
              ),
            ),
          )
        );
      });
      res=MainDropDown(
        value: field['value'],
        items: drpList,
        onChanged: (String? newValue) {
          order.setField(fldName, newValue!);
          setState((){});
        }
      );
    }else if(field['type']=='payment'){
      res=_getOrderFieldPayment();
    }else if(field['type']=='bonuses'){
      res=_getOrderFieldBonuses();
    }

    return res;
  }
  Widget _getOrderFieldBonuses(){
    return Row(
      children:[
        Switch(
          value: order.fields['bonuses']!['value']!='0',
          onChanged: (v){
            order.setField('bonuses', (v?order.fields['bonuses']!['max']:'0'));
            setState((){});
          },
          activeColor: getConstantColor('main'),
        ),
        Text(
          getMessage('of_bonuses')+order.fields['bonuses']!['max'],
          style:TextStyle(
            fontSize: 18,
          )
        )
      ]
    );
  }
  Widget _getOrderFieldPayment(){
    Widget res=EmptyWidget();
    List<Widget> selectedItems=[];
    List<DropdownMenuItem<String>> listItems=[];
    bool nof=true;
    String firstPID='';

    order.payments.forEach((el) {
      if(el['type']=='apple-google-pay') return;
      if(el['id']==order.fields['payment']!['value']) nof=false;
      if(firstPID.length==0) firstPID=el['id'];
      selectedItems.add(
          Container(
            padding: EdgeInsets.only(top:0,left:0),
            child:Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              children:[
                Text(
                    el['name'],
                    style:TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 16,
                    )
                ),
              ],
            ),
          )
      );
      listItems.add(
          DropdownMenuItem<String>(
            value: el['id'],
            child: Container(
              width: double.infinity,
              padding: EdgeInsets.only(top:3,bottom: 10),
              margin: EdgeInsets.only(top:3,bottom: 3),
              decoration: BoxDecoration (
                border: Border(
                  bottom: BorderSide(
                    color: Constants.colors['grayMain'],
                    width: 1.0,
                  ),
                ),
              ),
              child:Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                children:[
                  Text(
                      el['name'],
                      style:TextStyle(
                        fontWeight: FontWeight.bold,
                        fontSize: 16,
                      )
                  ),
                ],
              ),
            ),
          )
      );
    });
    if(nof) order.fields['payment']!['value']=firstPID;
    res=Container(
      margin:EdgeInsets.only(bottom: 7),
      child:MainDropDown(
        value: order.fields['payment']!['value'],
        items:listItems,
        hint:Container(
          padding:EdgeInsets.only(top:12),
          child:Text(
            getMessage('of_payment_type_choose'),
          ),
        ),
        onChanged: (String? newValue) {
          order.setField('payment', newValue!);
          setState((){});
        },
        selectedItemBuilder: (BuildContext context) =>selectedItems,
      ),
    );
    return res;
  }
}
