import 'package:flutter/material.dart';

class PingCodeInput extends StatefulWidget {
  final int cnt;
  final TextEditingController controller;
  final Function autoSend;

  PingCodeInput({
    Key? key,
    required this.cnt,
    required this.controller,
    required this.autoSend,
  })
      :super(key: key);

  @override
  _PingCodeInputState createState() => _PingCodeInputState();
}

class _PingCodeInputState extends State<PingCodeInput> {
  List<TextEditingController> _symbolControllers=[];
  List<String> arValue = [];
  List<FocusNode> focusNodes = [];
  bool autoSend=false;
  /**TODO backspace click to prev input**/

  void listenSymbolController(int i){
    String symb = _symbolControllers[i].text;
    if (symb.length > 1) {
      symb = symb.substring(symb.length - 1);
      _symbolControllers[i].text = symb;
      _symbolControllers[i].selection =
          TextSelection.collapsed(offset: symb.length);
    }
    arValue[i] = symb;
    widget.controller.text = arValue.join('');
    if (symb.length > 0) {
      focusNodes[i].unfocus();
      if (focusNodes.length - 1 > i && _symbolControllers[i + 1].text.length == 0){
        FocusScope.of(context).requestFocus(focusNodes[i + 1]);
      }
      if(focusNodes.length-1<=i && !autoSend){
        autoSend=true;
        widget.autoSend();
      }
    }
  }
  @override
  void dispose() {
    _symbolControllers.forEach((element) {element.dispose();});
    super.dispose();
  }
  @override
  void initState() {
    super.initState();
    for (int i = 0; i < widget.cnt; i++) {
      arValue.add('');
      _symbolControllers.add(TextEditingController());
      focusNodes.add(FocusNode());
      _symbolControllers[i].addListener(() {
        listenSymbolController(i);
      });
    }
  }
  @override
  Widget build(BuildContext context) {
    List<Widget> inputs = [];
    for (int i = 0; i < widget.cnt; i++) {
      inputs.add(
        Container(
          width: 40,
          child: TextField(
            onTap: () {
              _symbolControllers[i].text = '';
            },
            controller: _symbolControllers[i],
            autofocus: (i == 0),
            focusNode: focusNodes[i],
            keyboardType: TextInputType.number,
            textAlign: TextAlign.center,
            style: TextStyle(
              fontSize: 20.0,
            ),
            decoration: InputDecoration(
              contentPadding: EdgeInsets.all(7.0),
              errorText: null,
              fillColor: Colors.grey,
              border: OutlineInputBorder(
                borderRadius: BorderRadius.circular(5.0),
                borderSide: BorderSide(color: Colors.grey[200]!,),
              ),
              enabledBorder: OutlineInputBorder(
                borderSide: BorderSide(color: Colors.grey[200]!,),
                borderRadius: BorderRadius.circular(5.0),
              ),
            ),
            maxLines: 1,
          ),
        ),
      );
    }
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: inputs,
    );
  }
}