import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/util/const.dart';

String priceFormat(num price,{bool currency=true}){
  var f=currency?NumberFormat.currency(locale: 'ru',decimalDigits: 2,symbol: getMessage('vl_symbol')):NumberFormat.decimalPattern("ru");
  f.minimumFractionDigits=0;
  f.maximumFractionDigits=2;
  String res=f.format(price);
  res=res.replaceAll(',','.');

  return res;
}
class PriceFormatText extends StatelessWidget{
  const PriceFormatText({
    Key? key,
    required this.price,
    this.fontSize=16,
    this.color,
    this.fontWeight=FontWeight.w600,
    this.height=1.2,
    this.fontFamily=''
  }) : super(key: key);
  final num price;
  final double fontSize;
  final Color? color;
  final FontWeight? fontWeight;
  final String fontFamily;
  final double height;

  /*@override
  Widget build(BuildContext context) {
    String res = priceFormat(price);
    return Text(
      res,
      style: TextStyle(
        color: color??getConstantColor('text'),
        fontSize: fontSize,
        fontWeight: fontWeight,
        fontFamily: 'Roboto',
        height: height,
      ),
    );
  }*/
  @override
  Widget build(BuildContext context) {
    String ff=fontFamily.isNotEmpty?fontFamily:getConstantStyle('fontMain');
    String res = priceFormat(price,currency:false);
    return RichText(
      text: TextSpan(
        children: [
          TextSpan(
            text:res,
            style: TextStyle(
              color: color??getConstantColor('text'),
              fontSize: fontSize,
              fontWeight: fontWeight,
              fontFamily: ff,
              height: height,
            ),
          ),
          TextSpan(
            text:' '+getMessage('vl_symbol'),
            style: TextStyle(
              color: color??getConstantColor('text'),
              fontSize: fontSize-1,
              fontWeight: fontWeight,
              fontFamily: 'Roboto',
              height: height,
            ),
          )
        ],
      ),
    );
  }
}
TextSpan priceFormatTextSpan({required num price,double fontSize=16,Color? color, FontWeight fontWeight=FontWeight.normal,String? fontFamily,double height=1.2}){
  String res=priceFormat(price,currency: false);
  color ??= getConstantColor('text');
  fontFamily ??= getConstantStyle('fontMain');
  return TextSpan(
    children: [
      TextSpan(
        text:res,
        style: TextStyle(
          color: color,
          fontSize: fontSize,
          fontWeight: fontWeight,
          fontFamily: fontFamily,
          height: height,
        ),
      ),
      TextSpan(
        text:' '+getMessage('vl_symbol'),
        style: TextStyle(
          color: color,
          fontSize: fontSize-1,
          fontWeight: fontWeight,
          fontFamily: 'Roboto',
          height: height,
        ),
      )
    ],
  );
}