import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:foapp/providers/app_provider.dart';
import 'package:foapp/providers/data_provider.dart';
import 'package:foapp/providers/user_provider.dart';
import 'package:foapp/util/const.dart';
import 'package:foapp/util/dadata.dart';
import 'package:foapp/util/popup.dart';
import 'package:foapp/util/tools.dart';
import 'package:foapp/widgets/empty_widget.dart';
import 'package:foapp/widgets/warning_mess.dart';
import 'package:provider/provider.dart';

import 'address_select.dart';
import 'main_button.dart';

void openReqDepartment({required BuildContext context,required Map addingTovar}) async{
  popupContentShow(
    context: context,
    sizePopup: 0.85,
    enableDrag: false,
    content:Container(
      padding:EdgeInsets.only(top:10,right: 10,left:10,bottom:20.0),
      child: ReqDepartment(
        addingTovar:addingTovar,
      ),
    ),
  );
}


class ReqDepartment extends StatefulWidget {
  final Map addingTovar;

  ReqDepartment({
    Key? key,
    required this.addingTovar,
  }):super(key: key);

  @override
  ReqDepartmentState createState() => ReqDepartmentState();
}

class ReqDepartmentState extends State<ReqDepartment> {
  late DataProvider dataProv;
  late UserProvider userProv;
  late OrderMotor order;
  late Map addingTovar;
  late TextEditingController controllerDadata;
  late FocusNode focusNodeDadata;
  String addressId='N';
  bool sending=false;
  String errorMessage='';
  String noteMessage='';
  Map addressNew={
    'AddressStr':'',
    'Lat':'',
    'Long':'',
    'CityName':'',
    'StreetName':'',
    'House':'',
    'Building':'',
    'Corpus':'',
    'Flat':'',
  };

  @override
  void dispose() {
    super.dispose();
  }
  @override
  void initState() {
    addingTovar=widget.addingTovar;
    dataProv = Provider.of<DataProvider>(context, listen: false);
    userProv = Provider.of<UserProvider>(context, listen: false);
    order=userProv.order;
    order.inited=false;
    controllerDadata=TextEditingController();
    focusNodeDadata=FocusNode();
    _initOrder();
    super.initState();
  }
  Future _initOrder() async {
    if(order.inited) return true;
    await order.init();
    if(order.addresses.length>0) addressId=order.addresses.first['id'];
    setState(() {});
  }
  @override
  Widget build(BuildContext context) {

    return Column(
      children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children:[
            Container(
              padding:EdgeInsets.only(bottom:5.0),
              child:GestureDetector(
                child: Icon(
                  Icons.close,
                  size:20.0,
                ),
                onTap: (){
                  Navigator.pop(context);
                },
              )
            ),
          ]
        ),
        Container(
          margin:EdgeInsets.only(bottom:20.0),
          child:Text(
           getMessage('req_department_title'),
           textAlign: TextAlign.center,
           style:TextStyle(
             height: 1.1,
             fontSize:getConstantStyle('fontSizeReqDepartmentTitle'),
             fontWeight: FontWeight.w600,
           )
          )
        ),
        widgetByCondition(
          cond: order.inited,
          func: ()=>_reqPage(),
          funcFls: ()=>Center(
            child:CircularProgressIndicator(),
          ),
        ),
      ],
    );
  }


  /****functions blocks***/
  Widget _reqPage(){
    String dType=order.fields['type']?['value'];

    return Column(
      children:[
        addressTypeSelector('type'),
        widgetByCondition(
          cond: dType=='delivery',
          func: ()=>inputAddress(),
          funcFls: ()=>inputDepartment(),
        ),
        widgetByCondition(
          cond:errorMessage.length>0,
          func:()=>WarningMess(text:errorMessage,type:'error'),
        ),
        widgetByCondition(
          cond:noteMessage.length>0,
          func:()=>WarningMess(text:noteMessage,type:'ok'),
          funcFls: ()=>Container(
            child:widgetByCondition(
              cond: sending,
              func:()=>SpinKitThreeBounce(color: getConstantColor('main')),
              funcFls: ()=>MainButton(
                minWidth: double.infinity,
                title:getMessage('save'),
                onClick: (){
                  saveAction();
                }
              ),
            ),
          ),
        ),
      ]
    );
  }
  void saveAction() async{
    String dType=order.fields['type']?['value'];
    String dp='';
    if(dType=='delivery'){
      setState(() {
        sending=true;
        errorMessage='';
        noteMessage='';
      });
      Map address=addressNew;
      if(addressId!='N'){
        address=order.addresses.firstWhere((el) => addressId==el['id']);
      }
      Map res=await order.checkAddressAnonim(address);
      if(res['error'].length>0){
        errorMessage=res['error'];
      }else {
        dp=res['department'];
        noteMessage=res['mess'];
      }
      setState(() {
        sending=false;
      });
    }else{
      dp=order.fields['department']?['value'];
    }
    if(dp.length>0){
      userProv.addressNew=addressNew;
      userProv.setDepartment(dp);
      if(addingTovar.containsKey('fo')){
        userProv.basketAdd(addingTovar['fo'], addingTovar['qty'],addingTovar['props']);
      }
      if(dType=='delivery') {
        Timer(Duration(seconds: 5), () {
          Navigator.pop(context);
        });
      }else{
        Navigator.pop(context);
      }
    }
  }
  Widget inputAddress(){
    Map field={
      'name':getMessage('addr_start_enter'),
      'error':'',
      'value':addressNew['AddressStr'],
    };
    return Column(
      children:[
        widgetByCondition(
          cond: order.addresses.length>0,
          func: ()=>Container(
            margin:EdgeInsets.only(top:22,bottom: 15),
            child:AddressSelect(curAddress: addressId,addresses: order.addresses,selectAddress: _selectAddress),
          ),
        ),
        widgetByCondition(
          cond: addressId=='N',
          func: ()=>Row(
            children: [
              Expanded(
                child:dadataTextField(
                  context: context,
                  controller: controllerDadata,
                  focusNode: focusNodeDadata,
                  field:field,
                  selectItem: _selectDadata,
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }
  void _selectAddress(String addr){
    order.setField('address', addr);
    addressId=addr;
    setState(() {

    });
  }
  void _selectDadata(Map data) {
    setState(() {
      addressNew['AddressStr']=data['value'];
      data['data']['qc_geo']=int.parse(data['data']['qc_geo']);
      if(data['data']['qc_geo']<4) {
        addressNew['Lat'] = data['data']['geo_lat'];
        addressNew['Long'] = data['data']['geo_lon'];
      }
      addressNew['CityName']=data['data']['city']!=null?data['data']['city']:'';
      addressNew['StreetName']=data['data']['street_with_type']!=null?data['data']['street_with_type'].toString():'';
      addressNew['House']=data['data']['house']!=null?data['data']['house'].toString():'';
      addressNew['Building']=data['data']['block']!=null?data['data']['block'].toString():'';
      addressNew['Flat']=data['data']['flat']!=null?data['data']['flat'].toString():'';
      if(addressNew['House'].length<=0){
        addressNew['AddressStr']+=' ';
        Timer(Duration(milliseconds: 100), (){
          focusNodeDadata.requestFocus();
        });

      }
    });
  }
  Widget inputDepartment(){
    return Container(
      margin:EdgeInsets.only(top:15,bottom: 7),
      child:Theme(
        data: Constants.themeDropDown(context),
        child:DropdownButton<String>(
          value: order.fields['department']!['value']==''?null:order.fields['department']!['value'],
          isExpanded: true,
          elevation: 18,
          itemHeight: 60.0,
          hint: Container(
            padding:EdgeInsets.only(top:18),
            child:Text(
              getMessage('of_department_choose'),
            ),
          ),
          onChanged: (String? newValue) {
            order.setField('department', newValue!=null?newValue:'');
            setState((){});
          },
          selectedItemBuilder: (BuildContext context) {
            return order.departments.map<Widget>((el) {
              return Container(
                padding: EdgeInsets.only(top:7,left:0),
                child:Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children:[
                    Text(
                        el['name'],
                        style:TextStyle(
                          fontWeight: FontWeight.bold,
                          fontSize: 16,
                        )
                    ),
                    Text(
                        el['desc'],
                        maxLines:1,
                        style:TextStyle(
                          color: Colors.grey,
                          fontSize: 14,
                        )
                    ),
                  ],
                ),
              );
            }).toList();
          },
          items: order.departments.map<DropdownMenuItem<String>>((el) {
            return DropdownMenuItem<String>(
              value: el['code'],
              child: Container(
                width: double.infinity,
                padding: EdgeInsets.only(top:3,bottom: 10),
                margin: EdgeInsets.only(top:3,bottom: 3),
                decoration: BoxDecoration (
                  border: Border(
                    bottom: BorderSide(
                      color: Constants.colors['grayMain'],
                      width: 1.0,
                    ),
                  ),
                ),
                child:Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children:[
                    Text(
                        el['name'],
                        style:TextStyle(
                          fontWeight: FontWeight.bold,
                          fontSize: 16,
                        )
                    ),
                    Text(
                        el['desc'],
                        style:TextStyle(
                          color: Colors.grey,
                          fontSize: 14,
                        )
                    ),
                  ],
                ),
              ),
            );
          }).toList(),
        ),
      ),
    );
  }
  Widget addressTypeSelector(String fldName){
    Map? field=order.fields[fldName];
    if(field==null) return EmptyWidget();
    List<Widget> arBtns=[];
    field['list'].forEach((k,tlt){
      arBtns.add(Expanded(
        flex: 1,
        child: InkWell(
          child: Container(
            padding: EdgeInsets.all(10),
            alignment: Alignment.center,
            decoration: BoxDecoration (
              color: field['value']==k?getConstantColor('main'):getConstantColor('orderBgSelectType'),
              borderRadius: BorderRadius.circular(6),
            ),
            child:Text(
                tlt,
                style:TextStyle(
                  fontWeight: field['value']==k?FontWeight.bold:FontWeight.normal,
                  color: field['value']==k?getConstantColor('btnText'):getConstantColor('text'),
                )
            ),
          ),
          onTap: (){
            order.setField(fldName, k);
            setState((){});
          },
        ),
      ));
    });
    return Container(
      margin:EdgeInsets.only(bottom:10),
      padding: EdgeInsets.all(5),
      decoration: BoxDecoration (
        color: Constants.colors['orderBgSelectType'],
        borderRadius: BorderRadius.circular(8),
      ),
      child: Row(
        children: arBtns,
      ),
    );
  }
}